IMPLEMENTATION MODULE DDD;

IMPORT Guitool;
IMPORT WorldEd;
IMPORT SceneEd;
IMPORT NewTerm;
IMPORT NewIO;
IMPORT InOut;
IMPORT Handler;
IMPORT IntMen;

FROM Mouse    IMPORT WaitForAnything;

FROM Storage  IMPORT ALLOCATE, DEALLOCATE, Available;

FROM MathLib0 IMPORT cos, sin, sqrt, arctan;

FROM Strings  IMPORT Concat, Length;

CONST maxpoints = 100;

TYPE PtoHead = POINTER TO ObjHead;

     PtoStObj = POINTER TO StoredObj;

     ObjHead = RECORD
                 orgxy, orgxz, orgyz  : Grawin.Coord;
                 scale  : DScale;
                 ptoobj : PtoStObj;
               END;

     StoredObj = RECORD
                   size : CARDINAL;
                   coords : ARRAY[1..maxpoints] OF DPoint;
                   nextobj : PtoStObj;
                 END;

     pointertable = RECORD
                      ptohead : PtoHead;
                      name    : Grawin.LString;
                      hidden  : BOOLEAN;
                    END;

(* word object is used for objects AND SCENES (= more complex objects) ! *)

VAR wherecol, eyecol, orgcol, planecol, crocol : Grawin.RGB;  (* colors of special-objects and markers *)

    xywin, xzwin, yzwin, dddwin                : CARDINAL;  (* windows to work on *)

    whereframeobj, crossobj                    : CARDINAL;  (* special objects *)

    backupscene, backupobj                     : CARDINAL;
    i, firstobj, lastobj                       : CARDINAL;  (* range of objectnumbers of editors *)
    backupedobj, backupedscene                 : CARDINAL;

    r11, r12, r13, r21, r22, r23,
    r31, r32, r33, r41, r42, r43 : REAL;              (* used to speed-up 3d-rotations *)

    fz : INTEGER;                                     (* distance of projection-plane for 3d-operations *)
    rfz : REAL;
    eyepos : DCoord;                                  (* position of eye for 3d-operations *)
    ex, ey, ez : REAL;

    worldmode,                                        (* is TRUE, if DDD-capsle is used by WORLD-EDITOR, FALSE otherwise *)
    whereflag, planeflag, eyeflag, orgflag : BOOLEAN; (* are plane, eye & whereframe-objects visible? *)

    objptable   : ARRAY[SceneEd.firstobj..SceneEd.lastobj+1]     OF pointertable;
    sceneptable : ARRAY[WorldEd.firstscene..WorldEd.lastscene+1] OF pointertable;

    noscale : DScale;
    nocoord : Grawin.Coord;

(* normal ROUND-function *)

PROCEDURE ROUND(x : REAL):INTEGER;
BEGIN
  IF x>0.0 THEN
    RETURN(TRUNC(x+0.5));
  ELSE
    RETURN(TRUNC(x-0.5));
  END;
END ROUND;

PROCEDURE Max(a, b : INTEGER):INTEGER;
BEGIN
  IF a>b THEN RETURN(a); ELSE RETURN(b); END;
END Max;

PROCEDURE Min(a, b : INTEGER):INTEGER;
BEGIN
  IF a>b THEN RETURN(b); ELSE RETURN(a); END;
END Min;
(* perform central-projection *)

PROCEDURE Project(pd : DCoord):Grawin.Coord;
VAR pp : Grawin.Coord;
    mul, div, x, y, z, rfz : REAL;

BEGIN
  x:=FLOAT(pd.x);
  y:=FLOAT(pd.y);
  z:=FLOAT(pd.z);

  div:=ez-z;
  mul:=rfz-z;

  IF (div<>0.0) THEN
    pp.x:=ROUND( x+mul*(ex-x)/div );
    pp.y:=ROUND( y+mul*(ey-y)/div );
    RETURN(pp);
  ELSE
    pp.x:=0;
    pp.y:=0;
    RETURN(pp);
  END;
END Project;

PROCEDURE NameObj(name : Grawin.LString; objnum : CARDINAL);
BEGIN
  IF NOT(worldmode) THEN
    Concat(" : ",name,objptable[objnum].name);
    objptable[objnum].name[1]:=CHR(62+objnum);
  ELSE
    Concat(" : ",name,sceneptable[objnum].name);
    sceneptable[objnum].name[1]:=CHR(62+objnum);
  END;
END NameObj;

PROCEDURE GetObjName(VAR name : ARRAY OF CHAR; objnum : CARDINAL);
BEGIN
  IF worldmode THEN
    FOR i:=1 TO HIGH(sceneptable[objnum].name) DO
      name[i-1]:=sceneptable[objnum].name[i];
    END;
  ELSE
    FOR i:=1 TO HIGH(objptable[objnum].name) DO
      name[i-1]:=objptable[objnum].name[i];
    END;
  END;
END GetObjName;

(* if there any entry in localobjname-table? *)

PROCEDURE IsAName(string : ARRAY OF CHAR):BOOLEAN;
BEGIN
  IF string[4]="_" THEN RETURN(FALSE); ELSE RETURN(TRUE); END;
END IsAName;

PROCEDURE DoesObjExists(objnum : CARDINAL):BOOLEAN;
BEGIN
  IF worldmode THEN
    RETURN(IsAName(sceneptable[objnum].name));
  ELSE
    RETURN(IsAName(objptable[objnum].name));
  END;
END DoesObjExists;

PROCEDURE DestroyStObj(objnum : CARDINAL);
VAR pt : PtoHead;
    pt2, pt3 : PtoStObj;
BEGIN
  IF worldmode THEN
    pt:=sceneptable[objnum].ptohead;
    sceneptable[objnum].ptohead:=NIL;
  ELSE
    pt:=objptable[objnum].ptohead;
    objptable[objnum].ptohead:=NIL;
  END;
  IF pt<>NIL THEN
    pt2:=pt^.ptoobj;
    pt^.ptoobj:=NIL;
    DEALLOCATE(pt,SIZE(ObjHead));
    WHILE pt2<>NIL DO
      pt3:=pt2;
      DEALLOCATE(pt3,SIZE(StoredObj));
      pt2:=pt2^.nextobj;
    END;
  END;
END DestroyStObj;

PROCEDURE StoreObj(objnum : CARDINAL);
VAR pp : Grawin.Coord;
    poi : Grawin.Point;
    sc : Grawin.Scale;
    max, i, co : CARDINAL;
    pt : PtoHead;
    pt2, pt3 : PtoStObj;
          c: CHAR;
          flag : BOOLEAN;
BEGIN
  IF NOT(DoesObjExists(objnum)) THEN RETURN; END;

  DestroyStObj(objnum);

  IF NOT(Available(SIZE(ObjHead))) THEN
    IntMen.Error;
    RETURN;
  ELSE
    ALLOCATE(pt,SIZE(ObjHead));
  END;
  IF worldmode THEN
    sceneptable[objnum].ptohead:=pt;
    sceneptable[objnum].hidden:=NOT(Grawin.IsObjDrawable(xywin,objnum));
  ELSE
    objptable[objnum].ptohead:=pt;
    objptable[objnum].hidden:=NOT(Grawin.IsObjDrawable(xywin,objnum));
  END;

  pp:=Grawin.GetObjOrg(xywin,objnum);
  pt^.orgxy:=pp;

  pp:=Grawin.GetObjOrg(xzwin,objnum);
  pt^.orgxz:=pp;

  pp:=Grawin.GetObjOrg(yzwin,objnum);
  pt^.orgyz:=pp;

  sc:=Grawin.GetObjScale(xywin,objnum);
  pt^.scale.scx:=sc.scx;
  pt^.scale.scy:=sc.scy;
  sc:=Grawin.GetObjScale(xzwin,objnum);
  pt^.scale.scz:=sc.scy;

  max:=Grawin.GetObjMaxPointNumber(xywin,objnum);
  flag:=TRUE;
  co:=0;

  Grawin.SetObjPointNumber(xywin,objnum,1);
  Grawin.SetObjPointNumber(xzwin,objnum,1);

  IF max>0 THEN
    FOR i:=1 TO max DO
      INC(co);
      IF (co>maxpoints) OR flag THEN
        IF NOT(Available(SIZE(StoredObj))) THEN
          IntMen.Error;
          RETURN;
        ELSE
          ALLOCATE(pt3,SIZE(StoredObj));
        END;
        IF flag THEN
          pt^.ptoobj:=pt3;
          pt3^.size:=0;
          pt3^.nextobj:=NIL;
          flag:=FALSE;
        ELSE
          pt2^.nextobj:=pt3;
        END;
        pt2:=pt3;
        co:=1;
      END;
      pt2^.size:=co;
      poi:=Grawin.GetObjPoint(xywin,objnum);
      pt2^.coords[co].x:=poi.x;
      pt2^.coords[co].y:=poi.y;
      poi:=Grawin.GetObjPoint(xzwin,objnum);
      pt2^.coords[co].z:=poi.y;
      pt2^.coords[co].col:=poi.col;
      pt2^.coords[co].drawline:=poi.drawline;

    END;
    pt2^.nextobj:=NIL;
  ELSE
    pt^.ptoobj:=NIL;
  END;

  FOR i:=xywin TO dddwin DO
    Grawin.DeleteObj(i,objnum);
  END;
END StoreObj;

PROCEDURE CreateStObj(objnum : CARDINAL; sc : DDD.DScale;
                     orgxy, orgxz, orgyz : Grawin.Coord);
VAR pp : Grawin.Coord;
    poi : Grawin.Point;
    max, i, co : CARDINAL;
    pt : PtoHead;
    pt2, pt3 : PtoStObj;
          c: CHAR;
          flag : BOOLEAN;
BEGIN
  IF NOT(Available(SIZE(ObjHead))) THEN
    IntMen.Error;
    RETURN;
  ELSE
    ALLOCATE(pt,SIZE(ObjHead));
  END;

  IF worldmode THEN
    sceneptable[objnum].ptohead:=pt;
  ELSE
    objptable[objnum].ptohead:=pt;
  END;

  pt^.orgxy:=orgxy;
  pt^.orgxz:=orgxz;
  pt^.orgyz:=orgyz;

  pt^.scale:=sc;

  IF NOT(Available(SIZE(StoredObj))) THEN
    IntMen.Error;
    RETURN;
  ELSE
    ALLOCATE(pt3,SIZE(StoredObj));
  END;
  pt^.ptoobj:=pt3;
  pt3^.size:=0;
  pt3^.nextobj:=NIL;

END CreateStObj;

PROCEDURE RestoreStObj(objnum : CARDINAL);
VAR pp : Grawin.Coord;
    poi : Grawin.Point;
    sc : DScale;

    pd : DPoint;
    hidden : BOOLEAN;
    drawflag : BOOLEAN;
    org : Grawin.Coord;
    i, winnum: CARDINAL;
    pt : PtoHead;
    pt2 : PtoStObj;

BEGIN
  IF worldmode THEN
    pt:=sceneptable[objnum].ptohead;
    hidden:=sceneptable[objnum].hidden;
  ELSE
    pt:=objptable[objnum].ptohead;
    hidden:=objptable[objnum].hidden;
  END;
  IF pt=NIL THEN
    FOR winnum:=xywin TO dddwin DO
      pp:=Grawin.GetWinOrg(winnum);
      IF worldmode THEN
        Grawin.CreateObj(winnum,objnum,0.5,0.5,pp.x,pp.y);
      ELSE
        Grawin.CreateObj(winnum,objnum,1.0,1.0,pp.x,pp.y);
      END;
    END;
  ELSE
    sc:=pt^.scale;
    org:=pt^.orgxy;
    Grawin.CreateObj(xywin,objnum,sc.scx,sc.scy,org.x,org.y);
    org:=pt^.orgxz;
    Grawin.CreateObj(xzwin,objnum,sc.scx,sc.scz,org.x,org.y);
    org:=pt^.orgyz;
    Grawin.CreateObj(yzwin,objnum,sc.scy,sc.scz,org.x,org.y);
    pp:=Grawin.GetWinOrg(dddwin);
    Grawin.CreateObj(dddwin,objnum,1.0,1.0,pp.x,pp.y);

    pt2:=pt^.ptoobj;

    IF hidden THEN
      FOR i:=xywin TO dddwin DO
        Grawin.DontDrawObj(i,objnum);
      END;
    END;

    IF pt2<>NIL THEN
      REPEAT
        FOR i:=1 TO pt2^.size DO
          pd:=pt2^.coords[i];
          pd.col:=pt2^.coords[i].col;
          Grawin.AddObjPoint(xywin,objnum,pd.x,pd.y,pd.col,pd.drawline);
          Grawin.AddObjPoint(xzwin,objnum,pd.x,pd.z,pd.col,pd.drawline);
          Grawin.AddObjPoint(yzwin,objnum,pd.y,pd.z,pd.col,pd.drawline);
        END;
        pt2:=pt2^.nextobj;
        UNTIL pt2=NIL;
      END;
      DestroyStObj(objnum);
    END;
    AddNewDCoords(objnum);
END RestoreStObj;

PROCEDURE AddStObjPoint(objnum  : CARDINAL; pd : DPoint);
VAR pt, npt : PtoStObj;
    co : CARDINAL;
BEGIN
  IF worldmode THEN
    pt:=sceneptable[objnum].ptohead^.ptoobj;
  ELSE
    pt:=objptable[objnum].ptohead^.ptoobj;
  END;

  REPEAT
    npt:=pt;
    pt:=pt^.nextobj;
  UNTIL pt=NIL;

  IF npt^.size>=maxpoints THEN
    IF NOT(Available(SIZE(StoredObj))) THEN
      IntMen.Error;
      RETURN;
    ELSE
      ALLOCATE(pt,SIZE(StoredObj));
    END;
    npt^.nextobj:=pt;
    npt^.size:=maxpoints;
    pt^.nextobj:=NIL;
    pt^.size:=0;
    npt:=pt;
    co:=1;
  END;

  INC(npt^.size);

  npt^.coords[npt^.size]:=pd;

END AddStObjPoint;

PROCEDURE GetStObjPoint(objnum, pointnum : CARDINAL):DPoint;

VAR co, i : CARDINAL;
    pt : PtoStObj; c : CHAR;
BEGIN
  co:=0;
  IF worldmode THEN
    pt:=sceneptable[objnum].ptohead^.ptoobj;
  ELSE
    pt:=objptable[objnum].ptohead^.ptoobj;
  END;
  FOR i:=1 TO ((pointnum-1) DIV maxpoints) DO
    pt:=pt^.nextobj;
  END;
  RETURN(pt^.coords[(pointnum-1) MOD maxpoints +1]);
END GetStObjPoint;

PROCEDURE GetStObjSize(objnum : CARDINAL):CARDINAL;

VAR co, size, i : CARDINAL;
    pt : PtoStObj; c: CHAR;
BEGIN
  co:=0;
  IF worldmode THEN
    IF sceneptable[objnum].ptohead=NIL THEN
      RETURN(0);
    ELSE
      pt:= sceneptable[objnum].ptohead^.ptoobj;
    END;
  ELSE
    IF objptable[objnum].ptohead=NIL THEN
      RETURN(0);
    ELSE
      pt:= objptable[objnum].ptohead^.ptoobj;
    END;
  END;
  WHILE pt<>NIL DO
    size:=pt^.size;
    pt:=pt^.nextobj;
    INC(co);
  END;
  IF co=0 THEN RETURN(0) ELSE
    RETURN(size + maxpoints*(co-1));
  END;
END GetStObjSize;

PROCEDURE GetStObjScale(objnum : CARDINAL):DScale;

BEGIN
  IF worldmode THEN
    IF sceneptable[objnum].ptohead<>NIL THEN
      RETURN(sceneptable[objnum].ptohead^.scale);
    ELSE
      RETURN(noscale);
    END;
  ELSE
    IF objptable[objnum].ptohead<>NIL THEN
      RETURN(objptable[objnum].ptohead^.scale);
    ELSE
      RETURN(noscale);
    END;
  END;
END GetStObjScale;

PROCEDURE SetStObjScale(objnum : CARDINAL; sc : DScale);

BEGIN
  IF worldmode THEN
    IF sceneptable[objnum].ptohead<>NIL THEN
      sceneptable[objnum].ptohead^.scale:=sc;
    END;
  ELSE
    IF objptable[objnum].ptohead<>NIL THEN
      objptable[objnum].ptohead^.scale:=sc;
    END;
  END;
END SetStObjScale;

PROCEDURE IsStObjDrawable(objnum : CARDINAL):BOOLEAN;
BEGIN
  IF worldmode THEN
    RETURN(NOT(sceneptable[objnum].hidden));
  ELSE
    RETURN(NOT(objptable[objnum].hidden));
  END;
END IsStObjDrawable;

PROCEDURE DontDrawStObj(objnum : CARDINAL);
BEGIN
  IF worldmode THEN
    sceneptable[objnum].hidden:=TRUE;
  ELSE
    objptable[objnum].hidden:=TRUE;
  END;
END DontDrawStObj;

PROCEDURE AllowDrawStObj(objnum : CARDINAL);
BEGIN
  IF worldmode THEN
    sceneptable[objnum].hidden:=FALSE;
  ELSE
    objptable[objnum].hidden:=FALSE;
  END;
END AllowDrawStObj;

PROCEDURE GetStObjXYOrg(objnum : CARDINAL):Grawin.Coord;

BEGIN
  IF worldmode THEN
    IF sceneptable[objnum].ptohead<>NIL THEN
      RETURN(sceneptable[objnum].ptohead^.orgxy);
    ELSE
      RETURN(nocoord);
    END;
  ELSE
    IF objptable[objnum].ptohead<>NIL THEN
      RETURN(objptable[objnum].ptohead^.orgxy);
    ELSE
      RETURN(nocoord);
    END;
  END;
END GetStObjXYOrg;

PROCEDURE GetStObjXZOrg(objnum : CARDINAL):Grawin.Coord;
BEGIN
  IF worldmode THEN
    IF sceneptable[objnum].ptohead<>NIL THEN
      RETURN(sceneptable[objnum].ptohead^.orgxz);
    ELSE
      RETURN(nocoord);
    END;
  ELSE
    IF objptable[objnum].ptohead<>NIL THEN
      RETURN(objptable[objnum].ptohead^.orgxz);
    ELSE
      RETURN(nocoord);
    END;
  END;
END GetStObjXZOrg;

PROCEDURE GetStObjYZOrg(objnum : CARDINAL):Grawin.Coord;
BEGIN
  IF worldmode THEN
    IF sceneptable[objnum].ptohead<>NIL THEN
      RETURN(sceneptable[objnum].ptohead^.orgyz);
    ELSE
      RETURN(nocoord);
    END;
  ELSE
    IF objptable[objnum].ptohead<>NIL THEN
      RETURN(objptable[objnum].ptohead^.orgyz);
    ELSE
      RETURN(nocoord);
    END;
  END;
END GetStObjYZOrg;

PROCEDURE SetStObjXYOrg(objnum : CARDINAL; org : Grawin.Coord);
BEGIN
  IF worldmode THEN
    sceneptable[objnum].ptohead^.orgxy:=org;
  ELSE
    objptable[objnum].ptohead^.orgxy:=org;
  END;
END SetStObjXYOrg;


PROCEDURE SetStObjXZOrg(objnum : CARDINAL; org :Grawin.Coord);
BEGIN
  IF worldmode THEN
    sceneptable[objnum].ptohead^.orgxz:=org;
  ELSE
    objptable[objnum].ptohead^.orgxz:=org;
  END;
END SetStObjXZOrg;


PROCEDURE SetStObjYZOrg(objnum : CARDINAL; org : Grawin.Coord);
BEGIN
  IF worldmode THEN
    sceneptable[objnum].ptohead^.orgyz:=org;
  ELSE
    objptable[objnum].ptohead^.orgyz:=org;
  END;
END SetStObjYZOrg;

PROCEDURE DrawStObjsToWin(winnum : CARDINAL);
VAR pp, org, orgxy, orgyz, crorgxy, crorgyz, crogddd : Grawin.Coord;
    worg, wul, wbr : Grawin.Coord;
    dsc : DScale;
    pt : PtoHead;
    pt2 : PtoStObj;
    obj : CARDINAL;
    col : Grawin.RGB;
    ax, ay, x, y : INTEGER;
    flag, hidden : BOOLEAN;
    drawflag : BOOLEAN;
    pd : DCoord;
    sc : REAL;
BEGIN
  NewTerm.xormode:=FALSE;
  wul:=Grawin.GetWinUpperLeft(winnum);
  wbr:=Grawin.GetWinBottomRight(winnum);
  worg:=Grawin.GetWinOrg(winnum);
  sc:=Grawin.GetWinZoom(winnum);

  NewTerm.SetWindow(wul.x,wul.y,wbr.x,wbr.y);
  FOR obj:=firstobj TO lastobj DO
    IF worldmode THEN
      pt:=sceneptable[obj].ptohead;
      hidden:=sceneptable[obj].hidden;
    ELSE
      pt:=objptable[obj].ptohead;
      hidden:=objptable[obj].hidden;
    END;
    IF pt<>NIL THEN
      flag:=FALSE;
      IF winnum=xywin THEN org:=pt^.orgxy; END;
      IF winnum=xzwin THEN org:=pt^.orgxz; END;
      IF winnum=yzwin THEN org:=pt^.orgyz; END;
      IF winnum=dddwin THEN
        orgxy:=pt^.orgxy;
        orgyz:=pt^.orgyz;
        crorgxy:=Grawin.GetWinOrg(xywin);
        crorgyz:=Grawin.GetWinOrg(yzwin);
        org:=Grawin.GetWinOrg(dddwin);
      END;

      dsc:=pt^.scale;

      IF winnum<>dddwin THEN
        dsc.scx:=dsc.scx*sc;
        dsc.scy:=dsc.scy*sc;
        dsc.scz:=dsc.scz*sc;
      END;

      org.x:=ROUND(FLOAT(worg.x+wul.x)+FLOAT(org.x-worg.x)*sc);
      org.y:=ROUND(FLOAT(worg.y+wul.y)+FLOAT(org.y-worg.y)*sc);

      IF NOT(hidden) THEN
        pt2:=pt^.ptoobj;
        IF pt2<>NIL THEN
          flag:=FALSE;
          REPEAT
            WITH pt2^ DO
              FOR i:=1 TO size DO
                col:=coords[i].col;
                drawflag:=coords[i].drawline;
                IF winnum=xywin THEN
                  x:=ROUND(FLOAT(coords[i].x)*dsc.scx)+org.x;
                  y:=ROUND(FLOAT(coords[i].y)*dsc.scy)+org.y;
                END;
                IF winnum=xzwin THEN
                  x:=ROUND(FLOAT(coords[i].x)*dsc.scx)+org.x;
                  y:=ROUND(FLOAT(coords[i].z)*dsc.scz)+org.y;
                END;
                IF winnum=yzwin THEN
                  x:=ROUND(FLOAT(coords[i].y)*dsc.scy)+org.x;
                  y:=ROUND(FLOAT(coords[i].z)*dsc.scz)+org.y;
                END;
                IF winnum=dddwin THEN
                  pd.x:=ROUND(FLOAT(coords[i].x)*dsc.scx)+orgxy.x-crorgxy.x;
                  pd.y:=ROUND(FLOAT(coords[i].y)*dsc.scy)+orgxy.y-crorgxy.y;
                  pd.z:=ROUND(FLOAT(coords[i].z)*dsc.scz)+orgyz.y-crorgyz.y;
                  pp:=Project(pd);

                  x:=ROUND(FLOAT(pp.x)*sc)+org.x;
                  y:=ROUND(FLOAT(pp.y)*sc)+org.y;

                END;
                IF flag AND drawflag THEN
                  NewTerm.SetFGColor(col.r,col.g,col.b);
                  NewTerm.Line(ax,ay,x,y);
                END;
                flag:=TRUE;
                ax:=x;
                ay:=y;
              END;
            END;
            pt2:=pt2^.nextobj;
          UNTIL pt2=NIL;
        END;
      END;
    END;
  END;
END DrawStObjsToWin;

PROCEDURE AllignAllStObjs(winnum : CARDINAL; originx, originy : INTEGER);
VAR pt : PtoHead;
    dx, dy : INTEGER;
    obj : CARDINAL;
    worg : Grawin.Coord;
BEGIN
  worg:=Grawin.GetWinOrg(winnum);
  dx:=worg.x-originx;
  dy:=worg.y-originy;

  FOR obj:=firstobj TO lastobj DO
    IF worldmode THEN
      pt:=sceneptable[obj].ptohead;
    ELSE
      pt:=objptable[obj].ptohead;
    END;
    IF pt<>NIL THEN
      WITH pt^ DO
        IF winnum=xywin THEN
          orgxy.x:=orgxy.x-dx;
          orgxy.y:=orgxy.y-dy;
        ELSIF winnum=xzwin THEN
          orgxz.x:=orgxz.x-dx;
          orgxz.y:=orgxz.y-dy;
        ELSIF winnum=yzwin THEN
          orgyz.x:=orgyz.x-dx;
          orgyz.y:=orgyz.y-dy;
        END;
      END;
    END;
  END;
  (* Grawin.SetWinOrg(winnum,originx,originy); *)
END AllignAllStObjs;

(* on how many objects does capsle work? *)

PROCEDURE GetMaxNumOfObjs():CARDINAL;
BEGIN
  RETURN(lastobj-firstobj+1);
END GetMaxNumOfObjs;

(* determine the position of the eye for 3d-operations *)

PROCEDURE SetEyePos(pd : DCoord);
BEGIN
  eyepos:=pd;
  ex:=FLOAT(pd.x);
  ey:=FLOAT(pd.y);
  ez:=FLOAT(pd.z);
END SetEyePos;

(* determine the distance of the projection-plane for 3d-operations *)

PROCEDURE SetPlaneDist(r : INTEGER);
BEGIN
  fz:=r;
  rfz:=FLOAT(r);
END SetPlaneDist;

PROCEDURE GetEyePos():DCoord;
BEGIN
  RETURN(eyepos);
END GetEyePos;

PROCEDURE GetPlaneDist():INTEGER;
BEGIN
  RETURN(fz);
END GetPlaneDist;

(* tell capsle on wich windows to work; windows are identified by numbers *)

PROCEDURE GetXYWin():CARDINAL;
BEGIN
  RETURN(xywin);
END GetXYWin;

PROCEDURE GetXZWin():CARDINAL;
BEGIN
  RETURN(xzwin);
END GetXZWin;

PROCEDURE GetYZWin():CARDINAL;
BEGIN
  RETURN(yzwin);
END GetYZWin;

PROCEDURE GetDDDWin():CARDINAL;
BEGIN
  RETURN(dddwin);
END GetDDDWin;

PROCEDURE GetCrossObj():CARDINAL;
BEGIN
  RETURN(crossobj);
END GetCrossObj;

(* tell capsle wether it is used by world-editor or scene-editor *)
(* temporary objects are used for yanking, copying etc. *)

PROCEDURE SetWorldMode();
BEGIN
  worldmode:=TRUE;
  firstobj:=WorldEd.firstscene;
  lastobj:=WorldEd.lastscene;
  backupscene:=WorldEd.lastscene+1;

  xywin:=WorldEd.xywin;
  xzwin:=WorldEd.xzwin;
  yzwin:=WorldEd.yzwin;
  dddwin:=WorldEd.dddwin;
  crossobj:=WorldEd.crossobj;
  whereframeobj:=WorldEd.whereframeobj;
END SetWorldMode;

PROCEDURE SetSceneMode();
BEGIN
  worldmode:=FALSE;
  firstobj:=SceneEd.firstobj;
  lastobj:=SceneEd.lastobj;
  backupobj:=SceneEd.lastobj+1;

  xywin:=SceneEd.xywin;
  xzwin:=SceneEd.xzwin;
  yzwin:=SceneEd.yzwin;
  dddwin:=SceneEd.dddwin;
  crossobj:=SceneEd.crossobj;
  whereframeobj:=SceneEd.whereframeobj;
END SetSceneMode;

PROCEDURE GetFirstObjnum():CARDINAL;
BEGIN
  RETURN(firstobj);
END GetFirstObjnum;

PROCEDURE GetLastObjnum():CARDINAL;
BEGIN
  RETURN(lastobj);
END GetLastObjnum;

PROCEDURE InWorldMode():BOOLEAN;
BEGIN
  RETURN(worldmode);
END InWorldMode;

PROCEDURE InSceneMode():BOOLEAN;
BEGIN
  RETURN(NOT(worldmode));
END InSceneMode;

(* planecol is the color of the projection-plane shown on editor-screens *)

PROCEDURE SetPlaneCol(col : Grawin.RGB);
BEGIN
  planecol:=col;
END SetPlaneCol;

(* eyecol is the color of the eye-marker shown on editor-screens *)

PROCEDURE SetEyeCol(col : Grawin.RGB);
BEGIN
  eyecol:=col;
END SetEyeCol;

(* crosscol is the color of the origin-cross shown on editor-screens *)

PROCEDURE SetCrossCol(col : Grawin.RGB);
BEGIN
  crocol:=col;
END SetCrossCol;

(* wherecol is the color of the where-frame-objects showing object-boundaries *)

PROCEDURE SetWhereCol(col : Grawin.RGB);
BEGIN
  wherecol:=col;
END SetWhereCol;

(* orgcol is the color of the origin-marker showing objects origin *)

PROCEDURE SetOrgCol(col : Grawin.RGB);
BEGIN
  orgcol:=col;
END SetOrgCol;

PROCEDURE GetPlaneCol():Grawin.RGB;
BEGIN
  RETURN(planecol);
END GetPlaneCol;

PROCEDURE GetEyeCol():Grawin.RGB;
BEGIN
  RETURN(eyecol);
END GetEyeCol;

PROCEDURE GetCrossCol():Grawin.RGB;
BEGIN
  RETURN(crocol);
END GetCrossCol;

PROCEDURE GetWhereCol():Grawin.RGB;
BEGIN
  RETURN(wherecol);
END GetWhereCol;

PROCEDURE GetOrgCol():Grawin.RGB;
BEGIN
  RETURN(orgcol);
END GetOrgCol;

(* turn whereframeobj, showing the boundarie-rectangle, on *)

PROCEDURE TurnWhereOn;
BEGIN
  whereflag:=TRUE;
END TurnWhereOn;

PROCEDURE TurnWhereOff;
BEGIN
  whereflag:=FALSE;
END TurnWhereOff;

PROCEDURE ToggleWhere;
BEGIN
  whereflag:=NOT(whereflag);
END ToggleWhere;

PROCEDURE IsWhereOn():BOOLEAN;
BEGIN
  RETURN(whereflag);
END IsWhereOn;

(* eye-marker can be switched on or off *)

PROCEDURE TurnEyeOn;
BEGIN
  eyeflag:=TRUE;
END TurnEyeOn;

PROCEDURE TurnEyeOff;
BEGIN
  eyeflag:=FALSE;
END TurnEyeOff;

PROCEDURE ToggleEye;
BEGIN
  eyeflag:=NOT(eyeflag);
END ToggleEye;

PROCEDURE IsEyeOn():BOOLEAN;
BEGIN
  RETURN(eyeflag);
END IsEyeOn;

(* plane-bar can be switched off or on *)

PROCEDURE TurnPlaneOn;
BEGIN
  planeflag:=TRUE;
END TurnPlaneOn;

PROCEDURE TurnPlaneOff;
BEGIN
  planeflag:=FALSE;
END TurnPlaneOff;

PROCEDURE TogglePlane;
BEGIN
  planeflag:=NOT(planeflag);
END TogglePlane;

PROCEDURE IsPlaneOn():BOOLEAN;
BEGIN
  RETURN(planeflag);
END IsPlaneOn;

(* show origin of the object can be switched on or off *)

PROCEDURE TurnOrgOn;
BEGIN
  orgflag:=TRUE;
END TurnOrgOn;

PROCEDURE TurnOrgOff;
BEGIN
  orgflag:=FALSE;
END TurnOrgOff;

PROCEDURE ToggleOrg;
BEGIN
  orgflag:=NOT(orgflag);
END ToggleOrg;

PROCEDURE IsOrgOn():BOOLEAN;
BEGIN
  RETURN(orgflag);
END IsOrgOn;

(* localnames are used for creating, destroying, editing objectnames etc. *)

PROCEDURE CopyToLocalNames(sourcenames : ARRAY OF Grawin.LString);
VAR i : CARDINAL;
BEGIN
  FOR i:=0 TO HIGH(sourcenames) DO
    IF worldmode THEN
      sceneptable[i+firstobj].name:=sourcenames[i];
    ELSE
      objptable[i+firstobj].name:=sourcenames[i];
    END;
  END;
END CopyToLocalNames;

PROCEDURE CopyFromLocalNames(VAR destnames : ARRAY OF Grawin.LString);
VAR i : CARDINAL;
BEGIN
  FOR i:=0 TO HIGH(destnames) DO
    IF worldmode THEN
      destnames[i]:=sceneptable[i+firstobj].name;
    ELSE
      destnames[i]:=objptable[i+firstobj].name;
    END;
  END;
END CopyFromLocalNames;

PROCEDURE ClearDDDObj(objnum : CARDINAL);
VAR pp : Grawin.Coord;
   sc : Grawin.Scale;
BEGIN
   pp:=Grawin.GetObjOrg(dddwin,objnum);
   sc:=Grawin.GetObjScale(dddwin,objnum);
   Grawin.DeleteObj(dddwin,objnum);
   Grawin.CreateObj(dddwin,objnum,sc.scx,sc.scy,pp.x,pp.y);
END ClearDDDObj;

(* obj-copy-function *)

PROCEDURE CopyObjFromTo(sourceobj, destobj : CARDINAL);
VAR pp : Grawin.Coord;
    sc : Grawin.Scale;
    poi : Grawin.Point;
    winnum,i  : CARDINAL;

BEGIN
  FOR winnum:=xywin TO yzwin DO
    pp:=Grawin.GetObjOrg(winnum,sourceobj);
    sc:=Grawin.GetObjScale(winnum,sourceobj);
    Grawin.CreateObj(winnum,destobj,sc.scx,sc.scy,pp.x,pp.y);
    Grawin.SetObjPointNumber(winnum,sourceobj,1);
    FOR i:=1 TO Grawin.GetObjMaxPointNumber(winnum,sourceobj) DO
      poi:=Grawin.GetObjPoint(winnum,sourceobj);
      Grawin.AddObjPoint(winnum,destobj,poi.x,poi.y,poi.col,poi.drawline);
    END;
  END;
END CopyObjFromTo;

PROCEDURE MakeObjBackup(objnum :CARDINAL);
VAR pp : Grawin.Coord;
    poi : Grawin.Point;
    sc : Grawin.Scale;
    max, i, co : CARDINAL;
    pt : PtoHead;
    pt2, pt3 : PtoStObj;
    flag : BOOLEAN;

BEGIN
  IF worldmode THEN
    DestroyStObj(backupscene);
  ELSE
    DestroyStObj(backupobj);
  END;

  IF NOT(Available(SIZE(ObjHead))) THEN
    IntMen.Error;
    RETURN;
  ELSE
    ALLOCATE(pt,SIZE(ObjHead));
  END;

  IF worldmode THEN
    sceneptable[backupscene].ptohead:=pt;
    sceneptable[backupscene].hidden:=NOT(Grawin.IsObjDrawable(xywin,objnum));
    backupedscene:=objnum;
  ELSE
    objptable[backupobj].ptohead:=pt;
    objptable[backupobj].hidden:=NOT(Grawin.IsObjDrawable(xywin,objnum));
    backupedobj:=objnum;
  END;

  pp:=Grawin.GetObjOrg(xywin,objnum);
  pt^.orgxy:=pp;

  pp:=Grawin.GetObjOrg(xzwin,objnum);
  pt^.orgxz:=pp;

  pp:=Grawin.GetObjOrg(yzwin,objnum);
  pt^.orgyz:=pp;

  sc:=Grawin.GetObjScale(xywin,objnum);
  pt^.scale.scx:=sc.scx;
  pt^.scale.scy:=sc.scy;
  sc:=Grawin.GetObjScale(xzwin,objnum);
  pt^.scale.scz:=sc.scy;

  max:=Grawin.GetObjMaxPointNumber(xywin,objnum);
  flag:=TRUE;
  co:=0;

  Grawin.SetObjPointNumber(xywin,objnum,1);
  Grawin.SetObjPointNumber(xzwin,objnum,1);

  IF max>0 THEN
    FOR i:=1 TO max DO
      INC(co);
      IF (co>maxpoints) OR flag THEN
        IF NOT(Available(SIZE(StoredObj))) THEN
          IntMen.Error;
          RETURN;
        ELSE
          ALLOCATE(pt3,SIZE(StoredObj));
        END;
        IF flag THEN
          pt^.ptoobj:=pt3;
          pt3^.size:=0;
          pt3^.nextobj:=NIL;
          flag:=FALSE;
        ELSE
          pt2^.nextobj:=pt3;
        END;
        pt2:=pt3;
        co:=1;
      END;
      pt2^.size:=co;
      poi:=Grawin.GetObjPoint(xywin,objnum);
      pt2^.coords[co].x:=poi.x;
      pt2^.coords[co].y:=poi.y;
      poi:=Grawin.GetObjPoint(xzwin,objnum);
      pt2^.coords[co].z:=poi.y;
      pt2^.coords[co].col:=poi.col;
      pt2^.coords[co].drawline:=poi.drawline;

    END;
    pt2^.nextobj:=NIL;
  ELSE
    pt^.ptoobj:=NIL;
  END;

END MakeObjBackup;

PROCEDURE RestoreStObjBackup();
VAR pp : Grawin.Coord;
    poi : Grawin.Point;
    sc : DScale;
    pd : DPoint;
    hidden : BOOLEAN;
    drawflag : BOOLEAN;
    org : Grawin.Coord;
    objnum, i, winnum: CARDINAL;
    pt : PtoHead;
    pt2 : PtoStObj;

BEGIN

  IF worldmode THEN
    objnum:=backupedscene;
    pt:=sceneptable[backupscene].ptohead;
    hidden:=sceneptable[backupscene].hidden;
  ELSE
    objnum:=backupedobj;
    pt:=objptable[backupobj].ptohead;
    hidden:=objptable[backupobj].hidden;
  END;

  IF pt=NIL THEN RETURN; END;

  sc:=pt^.scale;
  org:=pt^.orgxy;
  Grawin.DeleteObj(xywin,objnum);
  Grawin.CreateObj(xywin,objnum,sc.scx,sc.scy,org.x,org.y);
  org:=pt^.orgxz;
  Grawin.DeleteObj(xzwin,objnum);
  Grawin.CreateObj(xzwin,objnum,sc.scx,sc.scz,org.x,org.y);
  org:=pt^.orgyz;
  Grawin.DeleteObj(yzwin,objnum);
  Grawin.CreateObj(yzwin,objnum,sc.scy,sc.scz,org.x,org.y);
  pp:=Grawin.GetWinOrg(dddwin);
  Grawin.DeleteObj(dddwin,objnum);
  Grawin.CreateObj(dddwin,objnum,1.0,1.0,pp.x,pp.y);

  pt2:=pt^.ptoobj;

  IF hidden THEN
    FOR i:=xywin TO dddwin DO
      Grawin.DontDrawObj(i,objnum);
    END;
  END;

  IF pt2<>NIL THEN
    REPEAT
      FOR i:=1 TO pt2^.size DO
        pd:=pt2^.coords[i];
        pd.col:=pt2^.coords[i].col;
        Grawin.AddObjPoint(xywin,objnum,pd.x,pd.y,pd.col,pd.drawline);
        Grawin.AddObjPoint(xzwin,objnum,pd.x,pd.z,pd.col,pd.drawline);
        Grawin.AddObjPoint(yzwin,objnum,pd.y,pd.z,pd.col,pd.drawline);
      END;
      pt2:=pt2^.nextobj;
    UNTIL pt2=NIL;
  END;
  IF worldmode THEN
    DestroyStObj(backupscene);
  ELSE
    DestroyStObj(backupobj);
  END;
  AddNewDCoords(objnum);
END RestoreStObjBackup;

PROCEDURE DestroyStObjBackup();
BEGIN
  IF worldmode THEN
    DestroyStObj(backupscene);
  ELSE
    DestroyStObj(backupobj);
  END;
END DestroyStObjBackup;

PROCEDURE ClearXYZObj(objnum : CARDINAL);
VAR winnum : CARDINAL;
    org : Grawin.Coord;
    scale  : Grawin.Scale;

BEGIN
  FOR winnum:=xywin TO yzwin DO
    scale:=Grawin.GetObjScale(winnum,objnum);
    org:=Grawin.GetObjOrg(winnum,objnum);
    Grawin.DeleteObj(winnum,objnum);
    Grawin.CreateObj(winnum,objnum,scale.scx,scale.scy,org.x,org.y);
  END;
END ClearXYZObj;

PROCEDURE CreateXYZObj(objnum : CARDINAL);
VAR winnum : CARDINAL;
    org : Grawin.Coord;

BEGIN
  FOR winnum:=xywin TO yzwin DO
    org:=Grawin.GetWinOrg(winnum);
    Grawin.CreateObj(winnum,objnum,1.0,1.0,org.x,org.y);
  END;
END CreateXYZObj;

PROCEDURE CreateDDDObj(objnum : CARDINAL);
VAR pp : Grawin.Coord;
BEGIN
   pp:=Grawin.GetWinOrg(dddwin);
   Grawin.CreateObj(dddwin,objnum,1.0,1.0,pp.x,pp.y);
END CreateDDDObj;

PROCEDURE DestroyDDDObj(objnum : CARDINAL);
BEGIN
   Grawin.DeleteObj(dddwin,objnum);
END DestroyDDDObj;

PROCEDURE DestroyXYZObj(objnum : CARDINAL);
VAR winnum : CARDINAL;

BEGIN
  FOR winnum:=xywin TO yzwin DO
    Grawin.DeleteObj(winnum,objnum);
  END;
END DestroyXYZObj;

PROCEDURE AddXYZObjPoint(objnum : CARDINAL; pd: DPoint);
BEGIN
  Grawin.AddObjPoint(xywin,objnum,pd.x,pd.y,pd.col,pd.drawline);
  Grawin.AddObjPoint(xzwin,objnum,pd.x,pd.z,pd.col,pd.drawline);
  Grawin.AddObjPoint(yzwin,objnum,pd.y,pd.z,pd.col,pd.drawline);
END AddXYZObjPoint;


PROCEDURE ChangeXYZObjPoint(objnum : CARDINAL; pd: DPoint);
BEGIN
  Grawin.ChangeObjPoint(xywin,objnum,pd.x,pd.y,pd.col);
  Grawin.ChangeObjPoint(xzwin,objnum,pd.x,pd.z,pd.col);
  Grawin.ChangeObjPoint(yzwin,objnum,pd.y,pd.z,pd.col);
END ChangeXYZObjPoint;

PROCEDURE DeleteXYZObjPoint(objnum : CARDINAL);
BEGIN
  Grawin.DeleteObjPoint(xywin,objnum);
  Grawin.DeleteObjPoint(xzwin,objnum);
  Grawin.DeleteObjPoint(yzwin,objnum);
END DeleteXYZObjPoint;

PROCEDURE GetXYZObjPoint(objnum : CARDINAL):DPoint;
VAR pp : Grawin.Point;
    pd : DPoint;
BEGIN
  pp:=Grawin.GetObjPoint(xywin,objnum);
  pd.x:=pp.x;
  pd.y:=pp.y;
  pd.col:=pp.col;
  pp:=Grawin.GetObjPoint(yzwin,objnum);
  pd.z:=pp.y;
  pd.drawline:=pp.drawline;
  RETURN(pd);
END GetXYZObjPoint;

PROCEDURE ReadXYZObjPoint(objnum : CARDINAL):DPoint;
VAR pp : Grawin.Point;
    pd : DPoint;
BEGIN
  pp:=Grawin.ReadObjPoint(xywin,objnum);
  pd.x:=pp.x;
  pd.y:=pp.y;
  pd.col:=pp.col;
  pp:=Grawin.ReadObjPoint(yzwin,objnum);
  pd.z:=pp.y;
  pd.drawline:=pp.drawline;
  RETURN(pd);
END ReadXYZObjPoint;

PROCEDURE SetXYZObjPointNumber(objnum, pointnum : CARDINAL);
BEGIN
  Grawin.SetObjPointNumber(xywin,objnum,pointnum);
  Grawin.SetObjPointNumber(xzwin,objnum,pointnum);
  Grawin.SetObjPointNumber(yzwin,objnum,pointnum);
END SetXYZObjPointNumber;

PROCEDURE SetXYZObjScale(objnum : CARDINAL; sc : DScale);
BEGIN
  Grawin.SetObjScale(xywin,objnum,sc.scx,sc.scy);
  Grawin.SetObjScale(xzwin,objnum,sc.scx,sc.scz);
  Grawin.SetObjScale(yzwin,objnum,sc.scy,sc.scz);
END SetXYZObjScale;

PROCEDURE SetXYZObjOrg(objnum : CARDINAL; org : DCoord);
BEGIN
  Grawin.SetObjOrg(xywin,objnum,org.x,org.y);
  Grawin.SetObjOrg(xzwin,objnum,org.x,org.z);
  Grawin.SetObjOrg(yzwin,objnum,org.y,org.z);
END SetXYZObjOrg;

PROCEDURE GetXYZObjScale(objnum : CARDINAL):DScale;
VAR sc : Grawin.Scale;
    dsc : DScale;
BEGIN
  sc:=Grawin.GetObjScale(xywin,objnum);
  dsc.scx:=sc.scx;
  dsc.scy:=sc.scy;
  sc:=Grawin.GetObjScale(yzwin,objnum);
  dsc.scz:=sc.scy;
  RETURN(dsc);
END GetXYZObjScale;

PROCEDURE DontDrawXYZObj(objnum : CARDINAL);
BEGIN
  Grawin.DontDrawObj(xywin,objnum);
  Grawin.DontDrawObj(xzwin,objnum);
  Grawin.DontDrawObj(yzwin,objnum);
END DontDrawXYZObj;

PROCEDURE AllowDrawXYZObj(objnum : CARDINAL);
BEGIN
  Grawin.AllowDrawObj(xywin,objnum);
  Grawin.AllowDrawObj(xzwin,objnum);
  Grawin.AllowDrawObj(yzwin,objnum);
END AllowDrawXYZObj;

PROCEDURE GetXYZObjOrg(objnum : CARDINAL):DCoord;
VAR pp : Grawin.Coord;
    pd : DCoord;
BEGIN
  pp:=Grawin.GetObjOrg(xywin,objnum);
  pd.x:=pp.x;
  pd.y:=pp.y;
  pp:=Grawin.GetObjOrg(yzwin,objnum);
  pd.z:=pp.y;
  RETURN(pd);
END GetXYZObjOrg;

(* create 2d-origin-cross *)

PROCEDURE CreateCross(winnum, objnum, gridstep: CARDINAL);
BEGIN
  Grawin.AddObjPoint(winnum,objnum,-INTEGER(2*gridstep),0,crocol,FALSE);
  Grawin.AddObjPoint(winnum,objnum,2*gridstep,0,crocol,TRUE);
  Grawin.AddObjPoint(winnum,objnum,0,-INTEGER(2*gridstep),crocol,FALSE);
  Grawin.AddObjPoint(winnum,objnum,0,2*gridstep,crocol,TRUE);
END CreateCross;

(* create 3d-origin-cross *)

PROCEDURE AdjustDCross(winnum, objnum, gridstep: CARDINAL);
VAR pd : DCoord;
    pp : Grawin.Coord;

BEGIN

  Grawin.SetObjPointNumber(winnum,objnum,1);
  pd.x:=-INTEGER(2*gridstep);
  pd.y:=0;
  pd.z:=0;
  pp:=GiveXY(winnum,pd);
  Grawin.ChangeObjPoint(winnum,objnum,pp.x,pp.y,crocol);

  pd.x:=2*gridstep;
  pd.y:=0;
  pd.z:=0;
  pp:=GiveXY(winnum,pd);
  Grawin.ChangeObjPoint(winnum,objnum,pp.x,pp.y,crocol);

  pd.x:=0;
  pd.y:=-INTEGER(2*gridstep);
  pd.z:=0;
  pp:=GiveXY(winnum,pd);
  Grawin.ChangeObjPoint(winnum,objnum,pp.x,pp.y,crocol);

  pd.x:=0;
  pd.y:=2*gridstep;
  pd.z:=0;
  pp:=GiveXY(winnum,pd);
  Grawin.ChangeObjPoint(winnum,objnum,pp.x,pp.y,crocol);

  pd.x:=0;
  pd.y:=0;
  pd.z:=-INTEGER(2*gridstep);
  pp:=GiveXY(winnum,pd);
  Grawin.ChangeObjPoint(winnum,objnum,pp.x,pp.y,crocol);

  pd.x:=0;
  pd.y:=0;
  pd.z:=2*gridstep;
  pp:=GiveXY(winnum,pd);
  Grawin.ChangeObjPoint(winnum,objnum,pp.x,pp.y,crocol);

END AdjustDCross;

PROCEDURE CreateDCross(winnum, objnum, gridstep: CARDINAL);
VAR pd : DCoord;
    pp : Grawin.Coord;

BEGIN

  Grawin.SetObjPointNumber(winnum,objnum,1);
  pd.x:=-INTEGER(2*gridstep);
  pd.y:=0;
  pd.z:=0;
  pp:=GiveXY(winnum,pd);
  Grawin.AddObjPoint(winnum,objnum,pp.x,pp.y,crocol,FALSE);

  pd.x:=2*gridstep;
  pd.y:=0;
  pd.z:=0;
  pp:=GiveXY(winnum,pd);
  Grawin.AddObjPoint(winnum,objnum,pp.x,pp.y,crocol,TRUE);

  pd.x:=0;
  pd.y:=-INTEGER(2*gridstep);
  pd.z:=0;
  pp:=GiveXY(winnum,pd);
  Grawin.AddObjPoint(winnum,objnum,pp.x,pp.y,crocol,FALSE);

  pd.x:=0;
  pd.y:=2*gridstep;
  pd.z:=0;
  pp:=GiveXY(winnum,pd);
  Grawin.AddObjPoint(winnum,objnum,pp.x,pp.y,crocol,TRUE);

  pd.x:=0;
  pd.y:=0;
  pd.z:=-INTEGER(2*gridstep);
  pp:=GiveXY(winnum,pd);
  Grawin.AddObjPoint(winnum,objnum,pp.x,pp.y,crocol,FALSE);

  pd.x:=0;
  pd.y:=0;
  pd.z:=2*gridstep;
  pp:=GiveXY(winnum,pd);
  Grawin.AddObjPoint(winnum,objnum,pp.x,pp.y,crocol,TRUE);

END CreateDCross;


(* show eye-marker on window *)

PROCEDURE ShowEye(winnum : CARDINAL);
VAR p : DCoord;
    x, y, off : INTEGER;
    r : REAL;

BEGIN
  IF winnum=dddwin THEN RETURN; END;
  IF eyeflag THEN
    p:=eyepos;

    IF winnum=xywin THEN p.x:=p.x; p.y:=p.y; END;
    IF winnum=xzwin THEN p.x:=p.x; p.y:=p.z; END;
    IF winnum=yzwin THEN p.x:=p.y; p.y:=p.z; END;

    r:=Grawin.GetWinZoom(winnum);
    Grawin.SetWinZoom(winnum,1.0);

    x:=ROUND(FLOAT(p.x)*r);
    y:=ROUND(FLOAT(p.y)*r);

    Grawin.SetMarker1(winnum,x-2,y,eyecol);
    Grawin.SetMarker1(winnum,x+2,y,eyecol);
    Grawin.SetMarker1(winnum,x,y-2,eyecol);
    Grawin.SetMarker1(winnum,x,y+2,eyecol);

    Grawin.SetWinZoom(winnum,r);

  END;
END ShowEye;

(* show plane-bar on window *)

PROCEDURE ShowPlane(winnum : CARDINAL);
VAR afz : INTEGER;
BEGIN
  IF planeflag THEN
    afz:=ROUND(REAL(fz)*Grawin.GetWinZoom(winnum));
    IF (winnum=xzwin) OR (winnum=yzwin) THEN
      Grawin.DrawRegardless(winnum, -30,afz-2, 30,afz-2,planecol);
      Grawin.DrawRegardless(winnum, -30,afz ,  30,afz  ,planecol);
      Grawin.DrawRegardless(winnum, -30,afz+2, 30,afz+2,planecol);
    END;
  END;
END ShowPlane;

(* show where-object on window *)

PROCEDURE ShowWhere(objnum : CARDINAL);
VAR winnum, i  : CARDINAL;
    xmin, ymin, xmax, ymax : INTEGER;
    poi : Grawin.Point;
    pp, ppp: Grawin.Coord;
    sc : Grawin.Scale;

BEGIN
  IF whereflag AND (Grawin.GetObjMaxPointNumber(xywin,objnum)>1) THEN
    FOR winnum:=xywin TO yzwin DO
      Grawin.SetObjPointNumber(winnum,objnum,1);
      poi:=Grawin.GetObjPoint(winnum,objnum);
      xmin:=poi.x;
      ymin:=poi.y;
      xmax:=poi.x;
      ymax:=poi.y;
      FOR i:=2 TO Grawin.GetObjMaxPointNumber(winnum,objnum) DO
        poi:=Grawin.GetObjPoint(winnum,objnum);
        IF poi.x<xmin THEN xmin:=poi.x; END;
        IF poi.y<ymin THEN ymin:=poi.y; END;
        IF poi.x>xmax THEN xmax:=poi.x; END;
        IF poi.y>ymax THEN ymax:=poi.y; END;
      END;

      pp:=Grawin.GetObjOrg(winnum,objnum);
      ppp:=Grawin.GetObjOrg(winnum,whereframeobj);
      sc:=Grawin.GetObjScale(winnum,objnum);
      xmin:=ROUND(FLOAT(xmin)*sc.scx)+pp.x-ppp.x;
      ymin:=ROUND(FLOAT(ymin)*sc.scy)+pp.y-ppp.y;
      xmax:=ROUND(FLOAT(xmax)*sc.scx)+pp.x-ppp.x;
      ymax:=ROUND(FLOAT(ymax)*sc.scy)+pp.y-ppp.y;
      pp:=Grawin.GetWinOrg(winnum);
      Grawin.DeleteObj(winnum,whereframeobj);
      Grawin.CreateObj(winnum,whereframeobj,1.0,1.0,pp.x,pp.y);

      Grawin.AddObjPoint(winnum,whereframeobj,xmin,ymin,wherecol,FALSE);
      Grawin.AddObjPoint(winnum,whereframeobj,xmin,ymax,wherecol,TRUE);
      Grawin.AddObjPoint(winnum,whereframeobj,xmax,ymax,wherecol,TRUE);
      Grawin.AddObjPoint(winnum,whereframeobj,xmax,ymin,wherecol,TRUE);
      Grawin.AddObjPoint(winnum,whereframeobj,xmin,ymin,wherecol,TRUE);
    END;
  ELSE
    FOR winnum:=xywin TO yzwin DO
      pp:=Grawin.GetWinOrg(winnum);
      Grawin.DeleteObj(winnum,whereframeobj);
      Grawin.CreateObj(winnum,whereframeobj,1.0,1.0,pp.x,pp.y);
    END;
  END;
END ShowWhere;

(* show origin with marker typ 2 *)

PROCEDURE ShowOrg(winnum, objnum : CARDINAL);
VAR pp , ppp : Grawin.Coord;
BEGIN
  IF winnum<>dddwin THEN
    pp:=Grawin.GetObjOrg(winnum,objnum);
    ppp:=Grawin.GetWinOrg(winnum);
    IF orgflag THEN Grawin.SetMarker2(winnum,pp.x-ppp.x,pp.y-ppp.y,orgcol); END;
  END;
END ShowOrg;

(* transform 3d-point to 2d-point with coordinates for given window *)

PROCEDURE GiveXY(winnum : CARDINAL; pd : DCoord):Grawin.Coord;
VAR pproj : Grawin.Coord;

BEGIN
  IF winnum=xywin THEN
    pproj.x:=pd.x; pproj.y:=pd.y;
  END;
  IF winnum=xzwin THEN
    pproj.x:=pd.x; pproj.y:=pd.z;
  END;
  IF winnum=yzwin THEN
    pproj.x:=pd.y; pproj.y:=pd.z;
  END;
  IF winnum=dddwin THEN
    pproj:=Project(pd);
  END;
  RETURN(pproj);
END GiveXY;

(* calculate new 3d-coords, add or change if possible *)

PROCEDURE CalcNewDCoords(objnum : CARDINAL);
VAR pp, orgxy, orgyz, org : Grawin.Coord;
    poi :Grawin.Point;
    p : DCoord;
    m, n, i : CARDINAL;
    sc: Grawin.Scale;

BEGIN
  orgxy:=Grawin.GetWinOrg(xywin);
  orgyz:=Grawin.GetWinOrg(yzwin);
  Grawin.SetObjPointNumber(xywin,objnum,1);
  Grawin.SetObjPointNumber(yzwin,objnum,1);
  m:=Grawin.GetObjMaxPointNumber(dddwin,objnum);
  n:=Grawin.GetObjMaxPointNumber(xywin,objnum);

  FOR i:=1 TO n DO
    org:=Grawin.GetObjOrg(xywin,objnum);
    sc:=Grawin.GetObjScale(xywin,objnum);
    poi:=Grawin.GetObjPoint(xywin,objnum);

    p.x:=ROUND(FLOAT(poi.x)*sc.scx)+org.x-orgxy.x;
    p.y:=ROUND(FLOAT(poi.y)*sc.scy)+org.y-orgxy.y;

    org:=Grawin.GetObjOrg(yzwin,objnum);
    sc:=Grawin.GetObjScale(yzwin,objnum);
    poi:=Grawin.GetObjPoint(yzwin,objnum);
    p.z:=ROUND(FLOAT(poi.y)*sc.scy)+org.y-orgyz.y;

    pp:=Project(p);

    IF i<=m THEN
      Grawin.SetObjPointNumber(dddwin,objnum,i);
      Grawin.ChangeObjPoint(dddwin,objnum,pp.x,pp.y,poi.col);
    ELSE
      Grawin.AddObjPoint(dddwin,objnum,pp.x,pp.y,poi.col,poi.drawline);
    END;
  END;

  Grawin.Reset;
END CalcNewDCoords;

(* add new 3d-coords to existing 3d-coords *)

PROCEDURE AddNewDCoords(objnum : CARDINAL);
VAR pp, orgxy, orgyz, org : Grawin.Coord;
    poi : Grawin.Point;
    p : DCoord;
    i : CARDINAL;
    sc: Grawin.Scale;

BEGIN
  orgxy:=Grawin.GetWinOrg(xywin);
  orgyz:=Grawin.GetWinOrg(yzwin);
  Grawin.SetObjPointNumber(xywin,objnum,1);
  Grawin.SetObjPointNumber(yzwin,objnum,1);

  FOR i:=1 TO Grawin.GetObjMaxPointNumber(xywin,objnum) DO
    org:=Grawin.GetObjOrg(xywin,objnum);
    sc:=Grawin.GetObjScale(xywin,objnum);
    poi:=Grawin.GetObjPoint(xywin,objnum);
    p.x:=ROUND(FLOAT(poi.x)*sc.scx)+org.x-orgxy.x;
    p.y:=ROUND(FLOAT(poi.y)*sc.scy)+org.y-orgxy.y;

    org:=Grawin.GetObjOrg(yzwin,objnum);
    sc:=Grawin.GetObjScale(yzwin,objnum);
    poi:=Grawin.GetObjPoint(yzwin,objnum);
    p.z:=ROUND(FLOAT(poi.y)*sc.scy)+org.y-orgyz.y;
    pp:=Project(p);

    Grawin.AddObjPoint(dddwin,objnum,pp.x,pp.y,poi.col,poi.drawline);
  END;
  Grawin.Reset;
END AddNewDCoords;

(* give back smallest point of object *)

PROCEDURE GiveObjMin(objnum : CARDINAL):DCoord;
VAR i  : CARDINAL;
    poi : Grawin.Point;
    min : DCoord;

BEGIN
  Grawin.SetObjPointNumber(xywin,objnum,1);
  Grawin.SetObjPointNumber(xzwin,objnum,1);

  poi:=Grawin.GetObjPoint(xywin,objnum);
  min.x:=poi.x;
  min.y:=poi.y;
  poi:=Grawin.GetObjPoint(xzwin,objnum);
  min.z:=poi.y;
  FOR i:=2 TO Grawin.GetObjMaxPointNumber(xywin,objnum) DO
    poi:=Grawin.GetObjPoint(xywin,objnum);
    IF poi.x<min.x THEN min.x:=poi.x; END;
    IF poi.y<min.y THEN min.y:=poi.y; END;
    poi:=Grawin.GetObjPoint(xzwin,objnum);
    IF poi.y<min.z THEN min.z:=poi.y; END;
  END;
  RETURN(min);
END GiveObjMin;

(* give back biggest point of object *)

PROCEDURE GiveObjMax(objnum : CARDINAL):DCoord;
VAR i  : CARDINAL;
    poi : Grawin.Point;
    max : DCoord;

BEGIN
  Grawin.SetObjPointNumber(xywin,objnum,1);
  Grawin.SetObjPointNumber(xzwin,objnum,1);

  poi:=Grawin.GetObjPoint(xywin,objnum);
  max.x:=poi.x;
  max.y:=poi.y;
  poi:=Grawin.GetObjPoint(xzwin,objnum);
  max.z:=poi.y;
  FOR i:=2 TO Grawin.GetObjMaxPointNumber(xywin,objnum) DO
    poi:=Grawin.GetObjPoint(xywin,objnum);
    IF poi.x>max.x THEN max.x:=poi.x; END;
    IF poi.y>max.y THEN max.y:=poi.y; END;
    poi:=Grawin.GetObjPoint(xzwin,objnum);
    IF poi.y>max.z THEN max.z:=poi.y; END;
  END;
  RETURN(max);
END GiveObjMax;

(* init of generic rotation around axis (a1,a2,a3) - (v1,v2,v3) and angle alpha *)

PROCEDURE InitGenRot(a1, a2, a3, v1, v2, v3 : INTEGER; alpha : REAL);
VAR rho, theta, cal, sal, cph, sph, cth, sth, cph2, sph2, cth2, sth2, pi, cal1 : REAL;

BEGIN
  cal:=cos(alpha);
  sal:=sin(alpha);
  rho:=sqrt(FLOAT(v1)*FLOAT(v1)+FLOAT(v2)*FLOAT(v2)+FLOAT(v3)*FLOAT(v3));
  IF (rho=0.0) THEN
    theta:=0.0;
    cph:=1.0;
    sph:=0.0;
  ELSE
    IF v1=0 THEN
      IF v2>=0 THEN theta:=0.5*PI; ELSE theta:=1.5*PI; END;
    ELSE
      theta:=arctan(FLOAT(v2)/FLOAT(v1));
      IF v1<0 THEN theta:=theta+PI; END;
    END;
    cph:=FLOAT(v3)/rho;
    sph:=sqrt(1.0-cph*cph);
  END;
  cth:=cos(theta); sth:=sin(theta);
  cph2:=cph*cph; sph2:=1.0-cph2;
  cth2:=cth*cth; sth2:=1.0-cth2;
  r11:=(cal*cph2+sph2)*cth2+cal*sth2;
  r12:=sal*cph+cal1*sph2*cth*sth;
  r13:=sph*(cph*cth*cal1-sal*sth);
  r21:=sph2*cth*sth*cal1-sal*cph;
  r22:=sth2*(cal*cph2+sph2)+cal*cth2;
  r23:=sph*(cph*sth*cal1+sal*cth);
  r31:=sph*(cph*cth*cal1+sal*sth);
  r32:=sph*(cph*sth*cal1-sal*cth);
  r33:=cal*sph2+cph2;
  r41:=FLOAT(a1)-FLOAT(a1)*r11-FLOAT(a2)*r21-FLOAT(a3)*r31;
  r42:=FLOAT(a2)-FLOAT(a1)*r12-FLOAT(a2)*r22-FLOAT(a3)*r32;
  r43:=FLOAT(a3)-FLOAT(a1)*r13-FLOAT(a2)*r23-FLOAT(a3)*r33;
END InitGenRot;

(* perform generic rotation, (x,y,z) -> (px1, py1, pz1) *)

PROCEDURE GenRot(xi, yi, zi: INTEGER; VAR xo, yo, zo : INTEGER);

BEGIN
  xo:=ROUND(FLOAT(xi)*r11+FLOAT(yi)*r21+FLOAT(zi)*r31+r41);
  yo:=ROUND(FLOAT(xi)*r12+FLOAT(yi)*r22+FLOAT(zi)*r32+r42);
  zo:=ROUND(FLOAT(xi)*r13+FLOAT(yi)*r23+FLOAT(zi)*r33+r43);
END GenRot;

(* used by point-rotations around x, y or z-axis *)

PROCEDURE InitPointRot(angle : REAL);
BEGIN
  angle:=angle/360.0*2.0*PI;
  r11:=cos(angle);
  r12:=sin(angle);
END InitPointRot;

(* rotate point around x-axis *)

PROCEDURE PointXRot(pi : DCoord):DCoord;
VAR po : DCoord;

BEGIN
  po.x:=pi.x;
  po.y:=ROUND(FLOAT(pi.y)*r11)-ROUND(FLOAT(pi.z)*r12);
  po.z:=ROUND(FLOAT(pi.y)*r12)+ROUND(FLOAT(pi.z)*r11);
  RETURN(po);
END PointXRot;

(* rotate point around y-axis *)

PROCEDURE PointYRot(pi : DCoord):DCoord;
VAR po : DCoord;

BEGIN
  po.x:=ROUND(FLOAT(pi.z)*r12)+ROUND(FLOAT(pi.x)*r11);
  po.y:=pi.y;
  po.z:=ROUND(FLOAT(pi.z)*r11)-ROUND(FLOAT(pi.x)*r12);
  RETURN(po);
END PointYRot;

(* rotate point around z-axis *)

PROCEDURE PointZRot(pi : DCoord):DCoord;
VAR po : DCoord;

BEGIN
  po.x:=ROUND(FLOAT(pi.x)*r11)-ROUND(FLOAT(pi.y)*r12);
  po.y:=ROUND(FLOAT(pi.x)*r12)+ROUND(FLOAT(pi.y)*r11);
  po.z:=pi.z;
  RETURN(po);
END PointZRot;

(* rotate whole object around x-axis *)

PROCEDURE XRot(angle : REAL; objnum : CARDINAL);
VAR p : Grawin.Point;
    i, max : CARDINAL;
    pi : DPoint;
    poo, po : DCoord;

BEGIN
  max:=Grawin.GetObjMaxPointNumber(xywin,objnum);
  InitPointRot(angle);

  SetXYZObjPointNumber(objnum,1);
  FOR i:=1 TO max DO
    pi:=ReadXYZObjPoint(objnum);
    poo.x:=pi.x; poo.y:=pi.y; poo.z:=pi.z;
    po:=PointXRot(poo);
    pi.x:=po.x; pi.y:=po.y; pi.z:=po.z;
    ChangeXYZObjPoint(objnum,pi);
  END;
  Grawin.Reset;
END XRot;

(* rotate whole object around y-axis *)

PROCEDURE YRot(angle : REAL; objnum : CARDINAL);
VAR p : Grawin.Point;
    i, max : CARDINAL;
    pi : DPoint;
    poo, po : DCoord;

BEGIN
  max:=Grawin.GetObjMaxPointNumber(xywin,objnum);
  InitPointRot(angle);

  SetXYZObjPointNumber(objnum,1);
  FOR i:=1 TO max DO
    pi:=ReadXYZObjPoint(objnum);
    poo.x:=pi.x; poo.y:=pi.y; poo.z:=pi.z;
    po:=PointYRot(poo);
    pi.x:=po.x; pi.y:=po.y; pi.z:=po.z;
    ChangeXYZObjPoint(objnum,pi);
  END;
  Grawin.Reset;
END YRot;

(* rotate whole object around z-axis *)

PROCEDURE ZRot(angle : REAL; objnum : CARDINAL);
VAR p : Grawin.Point;
    i, max : CARDINAL;
    pi : DPoint;
    poo, po : DCoord;

BEGIN
  max:=Grawin.GetObjMaxPointNumber(xywin,objnum);
  InitPointRot(angle);

  SetXYZObjPointNumber(objnum,1);
  FOR i:=1 TO max DO
    pi:=ReadXYZObjPoint(objnum);
    poo.x:=pi.x; poo.y:=pi.y; poo.z:=pi.z;
    po:=PointZRot(poo);
    pi.x:=po.x; pi.y:=po.y; pi.z:=po.z;
    ChangeXYZObjPoint(objnum,pi);
  END;
  Grawin.Reset;
END ZRot;

(* init of capsles internal data and variables *)

BEGIN

  eyepos.x:=0;
  eyepos.y:=0;
  eyepos.z:=0;
  fz:=0;

  xywin:=0;
  xzwin:=0;
  yzwin:=0;
  dddwin:=0;

  noscale.scx:=0.0;
  noscale.scy:=0.0;
  noscale.scz:=0.0;
  nocoord.x:=0;
  nocoord.y:=0;

  whereflag:=FALSE;
  planeflag:=FALSE;
  eyeflag:=FALSE;
  orgflag:=FALSE;

  whereframeobj:=0;
  crossobj:=0;

  firstobj:=0;
  lastobj:=0;

  FOR i:=SceneEd.firstobj TO SceneEd.lastobj+1 DO
    WITH objptable[i] DO
      ptohead:=NIL;
      hidden:=FALSE;
      name:=noname;
    END;
  END;

  FOR i:=WorldEd.firstscene TO WorldEd.lastscene+1 DO
    WITH objptable[i] DO
      ptohead:=NIL;
      hidden:=FALSE;
      name:=noname;
    END;
  END;

END DDD.
