IMPLEMENTATION MODULE IntMen;

IMPORT DDD;
IMPORT Guitool;
IMPORT WorldEd;
IMPORT SceneEd;
IMPORT NewTerm;
IMPORT Colors;
IMPORT NewIO;
IMPORT InOut;
IMPORT Handler;

FROM Mouse    IMPORT WaitForAnything, StoreMousePos, WaitForEvents, InitMouse, StoreMousePos,
                     GetMouseState, StoreMousePos, SetMousePos, LeftAndMove, RestoreMousePos;

FROM Storage  IMPORT ALLOCATE, DEALLOCATE, Available;

FROM MathLib0 IMPORT cos, sin, sqrt, arctan;

FROM KeyConst IMPORT CurRight, CurLeft, CurUp, CurDown,
                     AltCurRight, AltCurLeft, AltCurUp, AltCurDown,
                     StrgCurRight, StrgCurLeft, StrgCurUp, StrgCurDown,
                     PgUp, PgDown, AltPgUp, AltPgDown,
                     Tab, Enter, Esc, Space, Home, FunKey1, FunKey2;

FROM Strings  IMPORT Concat, Length;

FROM Str      IMPORT Match, CardToStr;

FROM NewTerm IMPORT xormode;

TYPE yn = (escape, yes, no);

VAR wherecol, eyecol, orgcol, planecol, crocol : Grawin.RGB;  (* colors of special-objects and markers *)
    worldmode : BOOLEAN;
    i, xywin, xzwin, yzwin, dddwin : CARDINAL;
    crossobj, firstobj, lastobj : CARDINAL;
    tempobj1, tempobj2 : CARDINAL;

    alltrue      : ARRAY[1..WorldEd.absolutmaxobjs] OF BOOLEAN;
    nokeys       : ARRAY[1..WorldEd.absolutmaxobjs] OF CHAR;

    lastdx, lastdy : INTEGER;

    table : ARRAY[1..WorldEd.absolutmaxobjs] OF Grawin.CTLine;
    selab : ARRAY[1..WorldEd.absolutmaxobjs] OF BOOLEAN;

    yesno                                      : ARRAY[1..2] OF Grawin.CTLine;  (* simple YES-NO-text for confirmations *)
    errortext                                  : ARRAY[1..3] OF Grawin.CTLine;

    text : ARRAY[1..12] OF Grawin.CTLine;
    itext : Grawin.CTLine;

PROCEDURE Error();
VAR c : CHAR;
BEGIN
  Grawin.ShowInfoBox(10,10,3,errortext);
  WaitForAnything;
  Grawin.RefreshAllWins(TRUE);
END Error;

PROCEDURE GetNumbers();
BEGIN
  xywin:=DDD.GetXYWin();
  xzwin:=DDD.GetXZWin();
  yzwin:=DDD.GetYZWin();
  dddwin:=DDD.GetDDDWin();
  firstobj:=DDD.GetFirstObjnum();
  lastobj:=DDD.GetLastObjnum();
  crossobj:=DDD.GetCrossObj();
  tempobj1:=lastobj+1;
  tempobj2:=lastobj+2;
  wherecol:=DDD.GetWhereCol();
  eyecol:=DDD.GetEyeCol();
  orgcol:=DDD.GetOrgCol();
  planecol:=DDD.GetPlaneCol();
  worldmode:=DDD.InWorldMode();
END GetNumbers;

(* interactive menu-functions: *)
(* zoom-menu, zoom one or all objects/scenes *)

PROCEDURE ScaleInt(objnum : CARDINAL);
VAR
    winnum, ii, i, obj : CARDINAL;
    sc : REAL;
    pp : Grawin.Coord;
    oldscale : Grawin.Scale;
    dsc : DDD.DScale;

BEGIN


  IF Grawin.InErrorState() THEN Error; RETURN; END;

  GetNumbers();

  NewIO.UseDefault;
  text[1].t:='Scale-Function.';
  text[1].c:=Colors.ititlecol;
  text[2].t:='Please choose:';
  text[2].c:=Colors.ipromptcol;
  Grawin.ShowInfoBox(31,14,2,text);

  IF worldmode THEN
    text[1].t:='Scale actual scene...';
    text[2].t:='Scale all scenes.....';
  ELSE
    text[1].t:='Scale actual object...';
    text[2].t:='Scale all objects.....';
  END;
  text[1].c:=Colors.itextcol;
  text[2].c:=Colors.itextcol;
  ii:=Grawin.ShowChooseBox(33,18,2,text,nokeys,alltrue,1);

  IF ii=0 THEN RETURN; END;

  text[1].t:='All-Scale...';
  text[2].t:='X-Scale.....';
  text[3].t:='Y-Scale.....';
  text[4].t:='Z-Scale.....';

  Grawin.ColFromTo(text,Colors.itextcol,1,4);
  i:=Grawin.ShowChooseBox(35,22,4,text,nokeys,alltrue,1);

  text[1].c:=Colors.ipromptcol;
  text[2].c:=Colors.ipromptcol;
  text[3].c:=Colors.ientercol;

  IF i=0 THEN RETURN; END;

  CASE i OF
    1 : text[1].t:='All-Scale. Enter';
        text[2].t:='scale-factor: '; |
    2 : text[1].t:='X-Scale. Enter';
        text[2].t:='scale-factor: '; |
    3 : text[1].t:='Y-Scale. Enter';
        text[2].t:='scale-factor: '; |
    4 : text[1].t:='Z-Scale. Enter';
        text[2].t:='scale-factor: ';
  END;

  text[3].t:="";
  REPEAT
    Grawin.ShowInfoBox(52,22,3,text);
    NewIO.ReadReal(sc,"1.0");
  UNTIL NewIO.Done;

   CASE i OF
     1 : Grawin.SetObjScale(xywin,objnum,sc,sc);
         Grawin.SetObjScale(xzwin,objnum,sc,sc);
         Grawin.SetObjScale(yzwin,objnum,sc,sc); |

     2 : oldscale:=Grawin.GetObjScale(xywin,objnum);
         Grawin.SetObjScale(xywin,objnum,sc,oldscale.scy);
         oldscale:=Grawin.GetObjScale(xzwin,objnum);
         Grawin.SetObjScale(xzwin,objnum,sc,oldscale.scy); |
     3 : oldscale:=Grawin.GetObjScale(xywin,objnum);
         Grawin.SetObjScale(xywin,objnum,oldscale.scx,sc);
         oldscale:=Grawin.GetObjScale(yzwin,objnum);
         Grawin.SetObjScale(yzwin,objnum,sc,oldscale.scy); |
     4 : oldscale:=Grawin.GetObjScale(xzwin,objnum);
         Grawin.SetObjScale(xzwin,objnum,oldscale.scx,sc);
         oldscale:=Grawin.GetObjScale(yzwin,objnum);
         Grawin.SetObjScale(yzwin,objnum,oldscale.scx,sc);
   END;

   DDD.CalcNewDCoords(objnum);

   IF ii=2 THEN
     FOR obj:=firstobj TO lastobj DO
       IF DDD.DoesObjExists(obj) THEN
         dsc:=DDD.GetStObjScale(obj);
         CASE i OF
            1 : dsc.scx:=sc;
                dsc.scy:=sc;
                dsc.scz:=sc; |
            2 : dsc.scx:=sc; |
            3 : dsc.scy:=sc; |
            4 : dsc.scz:=sc; |
         END;
         DDD.SetStObjScale(obj,dsc);
       END;
     END;
   END;
  DDD.ShowWhere(objnum);
END ScaleInt;

(* rotate-menu *)

PROCEDURE RotateInt(objnum : CARDINAL);
VAR
    winnum, obj, ii, i : CARDINAL;
    winkel : REAL;
    orgxy, orgxz, orgyz, corgxy, corgxz, corgyz : Grawin.Coord;

BEGIN


  IF Grawin.InErrorState() THEN IntMen.Error; RETURN; END;

  GetNumbers;

  NewIO.UseDefault;
  text[1].t:='Rotate-Function.';
  text[2].t:='Please choose:';
  text[1].c:=Colors.ititlecol;
  text[2].c:=Colors.ipromptcol;

  Grawin.ShowInfoBox(31,14,2,text);

  text[1].t:='X-Rotate...';
  text[2].t:='Y-Rotate...';
  text[3].t:='Z-Rotate...';

  Grawin.ColFromTo(text,Colors.itextcol,1,3);

  i:=Grawin.ShowChooseBox(33,18,3,text,nokeys,alltrue,1);

  text[1].c:=Colors.itextcol;
  text[2].c:=Colors.itextcol;
  text[3].c:=Colors.ipromptcol;

  IF i=0 THEN RETURN; END;

  CASE i OF
    1 : text[1].t:='X-Rotate. Enter';
        text[2].t:='rotate-angle: '; |

    2 : text[1].t:='Y-Rotate. Enter';
        text[2].t:='rotate-angle: ';  |

    3 : text[1].t:='Z-Rotate. Enter';
        text[2].t:='rotate-angle: ';
  END;

  text[3].t:="";
  REPEAT
    Grawin.ShowInfoBox(49,18,3,text);
    NewIO.ReadReal(winkel,"10.0");
  UNTIL NewIO.Done;

  CASE i OF
    1 : DDD.XRot(winkel,objnum); |
    2 : DDD.YRot(winkel,objnum); |
    3 : DDD.ZRot(winkel,objnum);
  END;

  DDD.CalcNewDCoords(objnum);
  DDD.ShowWhere(objnum);

END RotateInt;

(* objects can be hidden for decreasing complexity *)
(* interactive selecter: hide or show objects *)

PROCEDURE HideShowObjInt(objnum : CARDINAL);

TYPE objects = (nil, done, cross, plane, eye, org, start);

VAR
    winnum, ai, ii, i : CARDINAL;
    c : CHAR;
    name : Grawin.LString;

BEGIN

  IF Grawin.InErrorState() THEN IntMen.Error; RETURN; END;

  GetNumbers();

  FOR i:=ORD(done) TO ORD(org) DO
    selab[i]:=TRUE;
  END;
  table[ORD(done)].t:=' DONE! ';
  table[ORD(done)].c:=Colors.iexitcol;
  ii:=1;
  ai:=objnum-firstobj+6;


  REPEAT
    table[ORD(cross)].t:=' Cross';
    table[ORD(plane)].t:=' Plane';
    table[ORD(eye)].t:=' Eye';
    table[ORD(org)].t:=' Org';

    IF Grawin.IsObjDrawable(xywin,crossobj) THEN
      Concat('YES :   ',table[ORD(cross)].t,table[ORD(cross)].t);
      table[ORD(cross)].c:=Colors.iselcol;
    ELSE
      Concat('NO  :   ',table[ORD(cross)].t,table[ORD(cross)].t);
      table[ORD(cross)].c:=Colors.iunselcol;
    END;
    IF DDD.IsPlaneOn() THEN
      Concat('YES :   ',table[ORD(plane)].t,table[ORD(plane)].t);
      table[ORD(plane)].c:=Colors.iselcol;
    ELSE
      Concat('NO  :   ',table[ORD(plane)].t,table[ORD(plane)].t);
      table[ORD(plane)].c:=Colors.iunselcol;
    END;
    IF DDD.IsEyeOn() THEN
      Concat('YES :   ',table[ORD(eye)].t,table[ORD(eye)].t);
      table[ORD(eye)].c:=Colors.iselcol;
    ELSE
      Concat('NO  :   ',table[ORD(eye)].t,table[ORD(eye)].t);
      table[ORD(eye)].c:=Colors.iunselcol;
    END;
    IF DDD.IsOrgOn() THEN
      Concat('YES :   ',table[ORD(org)].t,table[ORD(org)].t);
      table[ORD(org)].c:=Colors.iselcol;
    ELSE
      Concat('NO  :   ',table[ORD(org)].t,table[ORD(org)].t);
      table[ORD(org)].c:=Colors.iunselcol;
    END;

    FOR i:=firstobj TO lastobj DO
      selab[i-firstobj+ORD(start)]:=DDD.DoesObjExists(i);
      DDD.GetObjName(name,i);

      IF ((i<>objnum) AND DDD.IsStObjDrawable(i) AND DDD.DoesObjExists(i)) OR
         ((i=objnum) AND Grawin.IsObjDrawable(xywin,i)) THEN
        Concat('YES : ',name,table[i-firstobj+ORD(start)].t);
        table[i-firstobj+ORD(start)].c:=Colors.iselcol;
      ELSE
        Concat('NO  : ',name,table[i-firstobj+ORD(start)].t);
        table[i-firstobj+ORD(start)].c:=Colors.iunselcol;
      END;

    END;

    REPEAT
      ii:=Grawin.ShowChooseBox(30,4,lastobj-firstobj+ORD(start),table,nokeys,selab,ai);
    UNTIL ii<>0;
    ai:=ii;

    IF ii=ORD(plane) THEN DDD.TogglePlane(); END;
    IF ii=ORD(eye) THEN DDD.ToggleEye(); END;
    IF ii=ORD(org) THEN DDD.ToggleOrg(); END;

    IF (ai>1) AND NOT((ai=ORD(plane)) OR (ai=ORD(eye)) OR (ai=ORD(org))) THEN
     IF ii=ORD(cross) THEN
       ii:=crossobj;
     ELSE
       ii:=ii-ORD(start)+firstobj;
     END;

     IF (ii=objnum) OR (ii=crossobj) THEN
         IF Grawin.IsObjDrawable(xywin,ii) THEN
           FOR winnum:=xywin TO dddwin DO
             Grawin.DontDrawObj(winnum,ii);
           END;
         ELSE
           FOR winnum:=xywin TO dddwin DO
             Grawin.AllowDrawObj(winnum,ii);
           END;
         END;
     ELSE
       IF DDD.DoesObjExists(ii) THEN
         IF DDD.IsStObjDrawable(ii) THEN
           DDD.DontDrawStObj(ii);
         ELSE
           DDD.AllowDrawStObj(ii);
         END;
       END;
    END;
  END;
  UNTIL ai=ORD(done);
END HideShowObjInt;

(* create-menu *)

PROCEDURE CreateObjInt(objnum : CARDINAL; store : BOOLEAN):CARDINAL;
VAR name : Grawin.LString;
    obj, i, winnum : CARDINAL;
    pp : Grawin.Coord;
    c : CHAR;

BEGIN

  IF Grawin.InErrorState() THEN Error; RETURN(objnum); END;

  GetNumbers();

  Grawin.ColFromTo(text,Colors.ialacol,1,3);

  IF worldmode THEN
    text[1].t:="A new scene will be created.";
    text[3].t:="be independed from other scenes.";
  ELSE
    text[1].t:="A new object will be created.";
    text[3].t:="be independed from other objects.";
  END;
  text[2].t:="All operations on it will";

  text[4].t:="Please give it a name: ";
  text[4].c:=Colors.ientercol;

  obj:=firstobj-1;

  REPEAT
    INC(obj);
  UNTIL (obj>lastobj) OR NOT(DDD.DoesObjExists(obj));

  IF obj<=lastobj THEN
    REPEAT
      Grawin.ShowInfoBox(12,14,4,text);
      NewIO.ReadString(name);
    UNTIL NewIO.Done AND (name[1]<>"_");

    IF store THEN DDD.StoreObj(objnum); END;

    DDD.NameObj(name,obj);

    Grawin.SelectObj(obj);
    FOR winnum:=xywin TO dddwin DO
      pp:=Grawin.GetWinOrg(winnum);
      DDD.AllowDrawStObj(obj);
      IF worldmode THEN
        Grawin.CreateObj(winnum,obj,0.5,0.5,pp.x,pp.y);
      ELSE
        Grawin.CreateObj(winnum,obj,1.0,1.0,pp.x,pp.y);
      END;
      IF NOT(Grawin.IsObjDrawable(winnum,obj)) THEN

        NewIO.WriteString("Error: ");
        NewIO.WriteInt(obj,3);

        Grawin.ErrorTick();
      END;
    END;
  ELSE
    Grawin.ColFromTo(text,Colors.ialacol,1,3);
    text[1].t:=" Sorry, no more space left ";
    IF worldmode THEN
      text[2].t:=" for new scenes. Destroy ";
      text[3].t:=" some scenes first. ";
    ELSE
      text[2].t:=" for new objects. Destroy ";
      text[3].t:=" some objects first. ";
    END;
    Grawin.ShowInfoBox(12,14,3,text);
    WaitForAnything;
    obj:=objnum;
  END;
  DDD.ShowWhere(obj);
  RETURN(obj);

END CreateObjInt;

(* select object/scene as actual to work on it *)

PROCEDURE SelectObjInt(objnum : CARDINAL):CARDINAL;
VAR actobj, i :CARDINAL;

BEGIN

  IF Grawin.InErrorState() THEN Error; RETURN(objnum); END;
  GetNumbers();

  FOR i:=firstobj TO lastobj DO
    selab[i-firstobj+1]:=DDD.DoesObjExists(i);
    DDD.GetObjName(table[i-firstobj+1].t,i);

    IF DDD.DoesObjExists(i) THEN
      table[i-firstobj+1].c:=Colors.iselcol;
    ELSE
      table[i-firstobj+1].c:=Colors.iunselcol;
    END;
  END;

  DDD.StoreObj(objnum);

  REPEAT
    actobj:=Grawin.ShowChooseBox(30,4,lastobj-firstobj+1,table,nokeys,selab,objnum-firstobj+1)+firstobj-1;
  UNTIL (actobj>=firstobj) AND DDD.DoesObjExists(actobj);

  DDD.RestoreStObj(actobj);

  Grawin.SelectObj(actobj);
  DDD.ShowWhere(actobj);
  RETURN(actobj);
END SelectObjInt;

(* destroy-menu *)

PROCEDURE DestroyObjInt(objnum : CARDINAL):CARDINAL;
VAR i, winnum : CARDINAL;
    flag : BOOLEAN;
    c : CHAR;
    name : Grawin.LString;

BEGIN

  IF Grawin.InErrorState() THEN Error; RETURN(objnum); END;

  GetNumbers();

  IF worldmode THEN
    text[1].t:="Actual scene will be destroyed!!! ";
  ELSE
    text[1].t:="Actual object will be destroyed!!! ";
  END;
  text[2].t:="Do you really want to do so? ";
  text[1].c:=Colors.ialacol;
  text[2].c:=Colors.ialacol;

  Grawin.ShowInfoBox(10,10,2,text);
  i:=Grawin.ShowChooseBox(12,14,2,yesno,nokeys,alltrue,1);
  IF i=ORD(escape) THEN RETURN(objnum); END;

  IF i=ORD(yes) THEN

    DDD.NameObj(DDD.noname,objnum);

    FOR i:=xywin TO dddwin DO
      Grawin.DeleteObj(i,objnum);
    END;

    DDD.AllowDrawStObj(objnum);
    DDD.DestroyStObj(objnum);

    flag:=TRUE;
    FOR i:=firstobj TO lastobj DO
      flag:=flag AND NOT(DDD.DoesObjExists(i));
    END;

    IF NOT(flag) THEN
      FOR i:=firstobj TO lastobj DO
         selab[i-firstobj+1]:=DDD.DoesObjExists(i);
         DDD.GetObjName(table[i-firstobj+1].t,i);

         IF DDD.DoesObjExists(i) THEN
           table[i-firstobj+1].c:=Colors.iselcol;
         ELSE
           table[i-firstobj+1].c:=Colors.iunselcol;
         END;
      END;

      REPEAT
        objnum:=Grawin.ShowChooseBox(30,4,lastobj-firstobj+1,table,nokeys,selab,objnum-firstobj+1)+firstobj-1;
      UNTIL (objnum>=firstobj) AND DDD.DoesObjExists(objnum);

      DDD.RestoreStObj(objnum);
      Grawin.SelectObj(objnum);

    ELSE  (* i = ORD(no) *)

      objnum:=CreateObjInt(objnum,FALSE);

    END;
  END;
  DDD.ShowWhere(objnum);
  RETURN(objnum);
END DestroyObjInt;

(* copy-menu *)

PROCEDURE CopyObjInt(objnum: CARDINAL):CARDINAL;
VAR i, newobj, winnum, obj :CARDINAL;
   poi : Grawin.Point;
   sc : Grawin.Scale;
   pp : Grawin.Coord;
   name : Grawin.LString;
   c : CHAR;

BEGIN

  IF Grawin.InErrorState() THEN Error; RETURN(objnum); END;

  GetNumbers();

  IF worldmode THEN
    text[1].t:="A new scene will be created. ";
    text[3].t:="be independed from other scenes.";
  ELSE
    text[1].t:="A new object will be created. ";
    text[3].t:="be independed from other objects.";
  END;
  text[2].t:="All operations on it will ";
  text[4].t:="Please give it a name: ";

  Grawin.ColFromTo(text,Colors.ialacol,1,3);
  text[4].c:=Colors.ientercol;

  obj:=firstobj-1;

  REPEAT
    INC(obj);
  UNTIL (obj>lastobj) OR NOT(DDD.DoesObjExists(obj));

  IF obj<=lastobj THEN
    REPEAT
      Grawin.ShowInfoBox(12,14,4,text);
      NewIO.ReadString(name);
    UNTIL NewIO.Done AND (name[1]<>"_");

    DDD.NameObj(name,obj);

    Grawin.SelectObj(obj);
    FOR winnum:=xywin TO dddwin DO
      sc:=Grawin.GetObjScale(winnum,objnum);
      pp:=Grawin.GetWinOrg(winnum);
      Grawin.CreateObj(winnum,obj,sc.scx,sc.scy,pp.x,pp.y);
    END;
  ELSE

    Grawin.ColFromTo(text,Colors.ialacol,1,3);
    text[1].t:=" Sorry, no more space left ";
    IF worldmode THEN
      text[2].t:=" for new scenes. Destroy ";
      text[3].t:=" some scenes first. ";
    ELSE
      text[2].t:=" for new objects. Destroy ";
      text[3].t:=" some objects first. ";
    END;
    Grawin.ShowInfoBox(12,14,3,text);

    WaitForAnything;
    obj:=objnum;
  END;

  FOR winnum:=xywin TO yzwin DO
    Grawin.SetObjPointNumber(winnum,objnum,1);
    FOR i:=1 TO Grawin.GetObjMaxPointNumber(winnum,objnum) DO
      poi:=Grawin.GetObjPoint(winnum,objnum);
      Grawin.AddObjPoint(winnum,obj,poi.x,poi.y,poi.col,poi.drawline);
    END;
  END;

  DDD.StoreObj(objnum);
  DDD.AddNewDCoords(obj);
  DDD.ShowWhere(obj);
  RETURN(obj);
END CopyObjInt;

(* actual object/scene will be moved to actual cursor-position a *)

PROCEDURE MoveObj(objnum : CARDINAL; a : ARRAY OF Grawin.Coord);
VAR winnum : CARDINAL;
    pp : Grawin.Coord;

BEGIN
  GetNumbers();
  FOR winnum:=xywin TO yzwin DO
    pp:=Grawin.GetWinOrg(winnum);
    Grawin.SetObjOrg(winnum,objnum,a[winnum-xywin].x+pp.x,
                                     a[winnum-xywin].y+pp.y);
  END;
  DDD.CalcNewDCoords(objnum);
  DDD.ShowWhere(objnum);
END MoveObj;

PROCEDURE ResetWin(actwin, objnum : CARDINAL);
VAR pp : Grawin.Coord;
    zoom : REAL;
BEGIN
  Grawin.SetWinZoom(actwin,1.0);
  zoom:=1.0;
  pp:=Grawin.GetWinGraphicSize(actwin);
  pp.x:=pp.x DIV 2;
  pp.y:=pp.y DIV 2;
  Grawin.AllignAllObjs(actwin,pp.x,pp.y);
  DDD.AllignAllStObjs(actwin,pp.x,pp.y);
  Grawin.SetWinOrg(actwin,pp.x,pp.y);
  Grawin.DrawWin(actwin,TRUE,TRUE);
END ResetWin;

PROCEDURE ZoomInt(actwin : CARDINAL; zoom : REAL; dy : INTEGER);
VAR (* zoom : REAL; *)
    pp : Grawin.Coord;

BEGIN

  IF dy<>lastdy THEN
    zoom:=zoom+REAL(dy)*0.2;
    IF zoom>10.0 THEN zoom:=10.0; END;
    IF zoom<0.2  THEN zoom:=0.2; END;
    Grawin.SetWinZoom(actwin,zoom);
    Grawin.DrawWin(actwin,TRUE,FALSE);
    itext.t:=' Zoom: ';
    IF worldmode THEN
      itext.c:=Colors.wwintitlecol;
    ELSE
      itext.c:=Colors.swintitlecol;
    END;
    Grawin.TextToWinTitle(actwin,itext);
    NewIO.WriteReal(zoom,6,2,FALSE);
  END;
  lastdy:=dy;
END ZoomInt;

PROCEDURE ScrollInt(actwin : CARDINAL; pp : Grawin.Coord; dx, dy : INTEGER);
BEGIN
  IF (dx<>lastdx) OR (dy<>lastdy) THEN
    pp.x:=pp.x+INTEGER(Grawin.GetCursorstep())*dx;
    pp.y:=pp.y+INTEGER(Grawin.GetCursorstep())*dy;

    Grawin.AllignAllObjs(actwin,pp.x,pp.y);
    DDD.AllignAllStObjs(actwin,pp.x,pp.y);
    Grawin.SetWinOrg(actwin,pp.x,pp.y);
    Grawin.DrawWin(actwin,TRUE,FALSE);
    itext.t:=' Scroll: cur.keys, mouse';
    IF worldmode THEN
      itext.c:=Colors.wwintitlecol;
    ELSE
      itext.c:=Colors.swintitlecol;
    END;
    Grawin.TextToWinTitle(actwin,itext);
  END;
  lastdx:=dx;
  lastdy:=dy;
END ScrollInt;

(* free memory and space in nametable by copying objects/scenes together *)
(* select interactivly which objects can by yanked together *)

PROCEDURE YankObjsTogetherInt(objnum : CARDINAL):CARDINAL;
VAR
  newobj, winnum, first, ai, ii, i : CARDINAL;
  c : CHAR;
  together : BITSET;
  ppp, pp, orgxy, orgxz, orgyz : Grawin.Coord;
  poi : Grawin.Point;
  pd : DDD.DPoint;
  dsc : DDD.DScale;
  name : Grawin.LString;

BEGIN

  IF Grawin.InErrorState() THEN Error; RETURN(objnum); END;

  GetNumbers();

  together:={};
  table[1].t:=' DONE! ';
  table[1].c:=Colors.iexitcol;
  ii:=1;
  ai:=objnum-firstobj+2;

  selab[1]:=TRUE;
  FOR i:=firstobj TO lastobj DO
    selab[i-firstobj+2]:=DDD.DoesObjExists(i);
  END;

  REPEAT

    FOR i:=firstobj TO lastobj DO
      DDD.GetObjName(name,i);
      IF (i-firstobj+1) IN together THEN
        table[i-firstobj+2].c:=Colors.iselcol;
        Concat('YES : ',name,table[i-firstobj+2].t);
      ELSE
        table[i-firstobj+2].c:=Colors.iunselcol;
        Concat('NO  : ',name,table[i-firstobj+2].t);
      END;
    END;

    REPEAT
      ii:=Grawin.ShowChooseBox(30,4,lastobj-firstobj+2,table,nokeys,selab,ai);
    UNTIL ii<>0;
    ai:=ii;
    IF ii>1 THEN
      IF (ii-1 IN together) THEN
        EXCL(together,ii-1);
      ELSE
        IF DDD.DoesObjExists(firstobj+ii-2) THEN INCL(together,ii-1); END;
      END;
    END;
  UNTIL ii=1;

  IF together<>{} THEN
    newobj:=CreateObjInt(objnum,TRUE);
    IF newobj=objnum THEN RETURN(objnum); END;

    orgxy.x:=0;
    orgxy.y:=0;

    orgxz.x:=0;
    orgxz.y:=0;

    orgyz.x:=0;
    orgyz.y:=0;

    i:=0;

    FOR ii:=1 TO lastobj-firstobj+1 DO
      IF ii IN together THEN
        INC(i);
        pp:=DDD.GetStObjXYOrg(ii+firstobj-1);

        orgxy.x:=orgxy.x+pp.x;
        orgxy.y:=orgxy.y+pp.y;

        pp:=DDD.GetStObjXZOrg(ii+firstobj-1);

        orgxz.x:=orgxz.x+pp.x;
        orgxz.y:=orgxz.y+pp.y;

        pp:=DDD.GetStObjYZOrg(ii+firstobj-1);

        orgyz.x:=orgyz.x+pp.x;
        orgyz.y:=orgyz.y+pp.y;

      END;
    END;

    orgxy.x:=orgxy.x DIV INTEGER(i);
    orgxy.y:=orgxy.y DIV INTEGER(i);
    orgxz.x:=orgxz.x DIV INTEGER(i);
    orgxz.y:=orgxz.y DIV INTEGER(i);

    orgyz.x:=orgyz.x DIV INTEGER(i);
    orgyz.y:=orgyz.y DIV INTEGER(i);

    Grawin.SetObjOrg(xywin,newobj,orgxy.x,orgxy.y);
    Grawin.SetObjOrg(xzwin,newobj,orgxz.x,orgxz.y);
    Grawin.SetObjOrg(yzwin,newobj,orgyz.x,orgyz.y);

    FOR winnum:=xywin TO yzwin DO
      FOR ii:=1 TO lastobj-firstobj+1 DO
        IF ii IN together THEN
          dsc:=DDD.GetStObjScale(ii+firstobj-1);
          IF winnum=xywin THEN
            pp:=DDD.GetStObjXYOrg(ii+firstobj-1);

          END;
          IF winnum=xzwin THEN
            pp:=DDD.GetStObjXZOrg(ii+firstobj-1);

          END;
          IF winnum=yzwin THEN
            pp:=DDD.GetStObjYZOrg(ii+firstobj-1);
          END;

          FOR i:=1 TO DDD.GetStObjSize(ii+firstobj-1) DO
            pd:=DDD.GetStObjPoint(ii+firstobj-1,i);
            IF winnum=xywin THEN
              poi.x:=DDD.ROUND((FLOAT(pd.x)*dsc.scx + FLOAT(pp.x-orgxy.x)));
              poi.y:=DDD.ROUND((FLOAT(pd.y)*dsc.scy + FLOAT(pp.y-orgxy.y)));
            END;
            IF winnum=xzwin THEN
              poi.x:=DDD.ROUND((FLOAT(pd.x)*dsc.scx + FLOAT(pp.x-orgxz.x)));
              poi.y:=DDD.ROUND((FLOAT(pd.z)*dsc.scz + FLOAT(pp.y-orgxz.y)));
            END;
            IF winnum=yzwin THEN
              poi.x:=DDD.ROUND((FLOAT(pd.y)*dsc.scy + FLOAT(pp.x-orgyz.x)));
              poi.y:=DDD.ROUND((FLOAT(pd.z)*dsc.scz + FLOAT(pp.y-orgyz.y)));
            END;

            IF i=1 THEN
              Grawin.AddObjPoint(winnum,newobj,poi.x,poi.y,pd.col,FALSE);
            ELSE
              Grawin.AddObjPoint(winnum,newobj,poi.x,poi.y,pd.col,pd.drawline);
            END;
          END;
        END;
      END;
    END;
  ELSE
    newobj:=objnum;
  END;

  DDD.ShowWhere(newobj);
  RETURN(newobj);

END YankObjsTogetherInt;


PROCEDURE RenameObjInt(objnum : CARDINAL);
VAR name : Grawin.LString;
BEGIN

  IF Grawin.InErrorState() THEN Error; RETURN; END;

  GetNumbers();

  text[1].t:="Rename.";
  text[2].t:="Name is: ";
  text[1].c:=Colors.ititlecol;
  text[2].c:=Colors.itextcol;
  text[4].c:=Colors.ientercol;

  DDD.GetObjName(name,objnum);
  Concat(text[2].t,name,text[2].t);

  text[3].t:="Enter new name: ";
  text[3].c:=Colors.ipromptcol;
  text[4].t:="";
  name:="";

  REPEAT
    Grawin.ShowInfoBox(12,14,4,text);
    NewIO.ReadString(name);
  UNTIL NewIO.Done AND (name[1]<>"_");
  DDD.NameObj(name,objnum);
END RenameObjInt;

PROCEDURE ShowTestWin(rgb : Grawin.RGB; state : CARDINAL);
VAR poi : Grawin.Point;
    i, x, y : INTEGER;
    winnum : CARDINAL;
    backcol : Grawin.RGB;

BEGIN
  IF worldmode THEN
    backcol:=Colors.wwinbackcol;
    CASE state OF
      1 : Colors.wwinbackcol:=rgb; |
      2 : Colors.wwinborcol:=rgb; |
      3 : Colors.wwinhicol:=rgb; |
      4 : Colors.wwintitlecol:=rgb; |
      5 : Colors.wscreenbackcol:=rgb; |
      6 : Colors.wcurcol:=Grawin.XorRGBs(rgb,backcol);
    END;
  ELSE
    backcol:=Colors.swinbackcol;
    CASE state OF
      1 : Colors.swinbackcol:=rgb; |
      2 : Colors.swinborcol:=rgb; |
      3 : Colors.swinhicol:=rgb; |
      4 : Colors.swintitlecol:=rgb; |
      5 : Colors.sscreenbackcol:=rgb; |
      6 : Colors.scurcol:=Grawin.XorRGBs(rgb,backcol);
    END;
  END;

  IF state=3 THEN Grawin.SetWinActCol(rgb); END;
  IF state=5 THEN Grawin.SetScreenBackCol(rgb); END;

  FOR winnum:=xywin TO dddwin+1 DO
    CASE state OF
      1 : Grawin.SetWinBackCol(winnum,rgb); |
      2 : Grawin.SetWinBorCol(winnum,rgb); |
      4 : Grawin.SetWinTitleCol(winnum,rgb);
    END;
  END;

  FOR i:=1 TO 5 DO
    Grawin.SetObjPointNumber(dddwin+1,tempobj2,i);
    poi:=Grawin.ReadObjPoint(dddwin+1,tempobj2);
    Grawin.ChangeObjPoint(dddwin+1,tempobj2,poi.x,poi.y,wherecol);
  END;

  IF state=3 THEN
    Grawin.DrawWin(dddwin+1,TRUE,TRUE);
  ELSE
    Grawin.DrawWin(dddwin+1,FALSE,TRUE);
  END;

  Grawin.DrawWinObjlist(dddwin+1,FALSE);

  IF worldmode THEN
    Grawin.ShowCursor(dddwin+1,Colors.wcurcol,TRUE);
  ELSE
    Grawin.ShowCursor(dddwin+1,Colors.scurcol,TRUE);
  END;

  xormode:=FALSE;
  Grawin.DrawRegardless(dddwin+1, - 30,-48, 30,-48,planecol);
  Grawin.DrawRegardless(dddwin+1, - 30,-50, 30,-50,planecol);
  Grawin.DrawRegardless(dddwin+1, - 30,-52, 30,-52,planecol);

  x:=-50; y:=-70;
  xormode:=FALSE;
  Grawin.SetMarker1(dddwin+1,x-2,y,eyecol);
  Grawin.SetMarker1(dddwin+1,x+2,y,eyecol);
  Grawin.SetMarker1(dddwin+1,x,y-2,eyecol);
  Grawin.SetMarker1(dddwin+1,x,y+2,eyecol);

  x:=40; y:=-20;
  xormode:=TRUE;
  IF worldmode THEN
    Grawin.SetMarker1(dddwin+1,x,y,Colors.wmarcol);
  ELSE
    Grawin.SetMarker1(dddwin+1,x,y,Colors.smarcol);
  END;
  xormode:=FALSE;
  DDD.ShowOrg(dddwin+1,tempobj1);

END ShowTestWin;

PROCEDURE ShowTestWin2(rgb : Grawin.RGB; state : CARDINAL);
VAR p : Grawin.Coord;
    poi : Grawin.Point;
    i, x, y : INTEGER;
    winnum : CARDINAL;
    backcol : Grawin.RGB;

BEGIN

  IF worldmode THEN
    backcol:=Colors.wwinbackcol;
    CASE state OF
      1 : Colors.weyecol:=rgb; |
      2 : Colors.wplanecol:=rgb; |
      3 : Colors.wcrosscol:=rgb; |
      4 : Colors.wgridcol:=rgb; |
      5 : Colors.wwherecol:=rgb; |
      6 : Colors.worgcol:=rgb; |
      7 : Colors.wmarcol:=Grawin.XorRGBs(rgb,backcol);
    END;
  ELSE
    backcol:=Colors.swinbackcol;
    CASE state OF
      1 : Colors.seyecol:=rgb; |
      2 : Colors.splanecol:=rgb; |
      3 : Colors.scrosscol:=rgb; |
      4 : Colors.sgridcol:=rgb; |
      5 : Colors.swherecol:=rgb; |
      6 : Colors.sorgcol:=rgb; |
      7 : Colors.smarcol:=Grawin.XorRGBs(rgb,backcol);
    END;
  END;

  CASE state OF
    8 : Colors.stereored:=rgb; |
    9 : Colors.stereogreen:=rgb; |
    10 : Colors.viewscreencol:=rgb; |
    11 : Colors.viewinfocol:=rgb;
  END;

  CASE state OF
    1 : eyecol:=rgb;
        DDD.SetEyeCol(eyecol); |
    2 : planecol:=rgb;
        DDD.SetPlaneCol(planecol); |
    3 : crocol:=rgb;
        DDD.SetCrossCol(crocol); |
    5 : wherecol:=rgb;
        DDD.SetWhereCol(wherecol); |
    6 : orgcol:=rgb;
        DDD.SetOrgCol(orgcol);
  END;

  IF state=3 THEN
    Grawin.DeleteObj(dddwin+1,crossobj);
    p:=Grawin.GetWinOrg(dddwin+1);
    Grawin.CreateObj(dddwin+1,crossobj,1.0,1.0,p.x,p.y);
    DDD.CreateCross(dddwin+1,crossobj,Grawin.GetCursorstep());
    FOR winnum:=xywin TO yzwin DO
      Grawin.DeleteObj(winnum,crossobj);
      p:=Grawin.GetWinOrg(winnum);
      Grawin.CreateObj(winnum,crossobj,1.0,1.0,p.x,p.y);
      DDD.CreateCross(winnum,crossobj,Grawin.GetCursorstep());
    END;
    DDD.AdjustDCross(dddwin,crossobj,Grawin.GetCursorstep());
  END;

  FOR i:=1 TO 5 DO
    Grawin.SetObjPointNumber(dddwin+1,tempobj2,i);
    poi:=Grawin.ReadObjPoint(dddwin+1,tempobj2);
    Grawin.ChangeObjPoint(dddwin+1,tempobj2,poi.x,poi.y,wherecol);
  END;

  xormode:=FALSE;
  Grawin.SolidBox(42,2,3,4,Colors.stereored,Colors.iborcol1,Colors.iborcol2);
  Grawin.SolidBox(47,2,3,4,Colors.stereogreen,Colors.iborcol1,Colors.iborcol2);

  Grawin.SolidBox(10,2,17,5,Colors.viewscreencol,Colors.iborcol1,Colors.iborcol2);
  NewTerm.GotoXY(11*8,3*16);
  NewTerm.SetFGColor(Colors.viewinfocol.r,Colors.viewinfocol.g,Colors.viewinfocol.b);
  NewTerm.SetBGColor(Colors.viewscreencol.r,Colors.viewscreencol.g,Colors.viewscreencol.b);
  NewIO.WriteString("Objviewer ready.");

  Grawin.DrawWin(dddwin+1,FALSE,TRUE);

  xormode:=FALSE;
  IF state=4 THEN
    Grawin.LayGrid(dddwin+1,8,8,0,0,rgb);
  END;

  Grawin.DrawWinObjlist(dddwin+1,FALSE);

  IF worldmode THEN
    Grawin.ShowCursor(dddwin+1,Colors.wcurcol,TRUE);
  ELSE
    Grawin.ShowCursor(dddwin+1,Colors.scurcol,TRUE);
  END;

  xormode:=FALSE;
  Grawin.DrawRegardless(dddwin+1,-30,-48,30,-48,planecol);
  Grawin.DrawRegardless(dddwin+1,-30,-50,30,-50,planecol);
  Grawin.DrawRegardless(dddwin+1,-30,-52,30,-52,planecol);

  x:=-50; y:=-70;
  xormode:=FALSE;
  Grawin.SetMarker1(dddwin+1,x-2,y,eyecol);
  Grawin.SetMarker1(dddwin+1,x+2,y,eyecol);
  Grawin.SetMarker1(dddwin+1,x,y-2,eyecol);
  Grawin.SetMarker1(dddwin+1,x,y+2,eyecol);

  x:=40; y:=-20;
  xormode:=TRUE;
  IF worldmode THEN
    Grawin.SetMarker1(dddwin+1,x,y,Colors.wmarcol);
  ELSE
    Grawin.SetMarker1(dddwin+1,x,y,Colors.smarcol);
  END;

  xormode:=FALSE;
  DDD.ShowOrg(dddwin+1,tempobj1);
END ShowTestWin2;

PROCEDURE ShowTestMenu(rgb : Grawin.RGB; state : CARDINAL);
BEGIN
  IF worldmode THEN
    CASE state OF
      1 : Colors.wmenbackcol:=rgb; |
      2 : Colors.wmenitemcol:=rgb; |
      3 : Colors.wmenhiitemcol:=rgb; |
      4 : Colors.wmentitlecol:=rgb; |
      5 : Colors.wmenborcol1:=rgb; |
      6 : Colors.wmenborcol2:=rgb;
    END;
  ELSE
    CASE state OF
      1 : Colors.smenbackcol:=rgb; |
      2 : Colors.smenitemcol:=rgb; |
      3 : Colors.smenhiitemcol:=rgb; |
      4 : Colors.smentitlecol:=rgb; |
      5 : Colors.smenborcol1:=rgb; |
      6 : Colors.smenborcol2:=rgb;
    END;
  END;
  CASE state OF
    1 : Grawin.SetMenuBackCol(rgb); |
    2 : Grawin.SetMenuItemCol(rgb); |
    3 : Grawin.SetMenuActItemCol(rgb); |
    4 : Grawin.SetMenuTitleCol(rgb); |
    5 : Grawin.SetMenuBorCol1(rgb); |
    6 : Grawin.SetMenuBorCol2(rgb);
  END;
  Grawin.OpenMenu(dddwin+1);
END ShowTestMenu;

PROCEDURE ShowTestBox(rgb : Grawin.RGB; state : CARDINAL);
VAR
BEGIN

  CASE state OF
    1 : IF worldmode THEN
          Colors.winfobackcol:=rgb;
        ELSE
          Colors.sinfobackcol:=rgb;
        END; |
    2 : Colors.iborcol1:=rgb; |
    3 : Colors.iborcol2:=rgb; |
    4 : Colors.ititlecol:=rgb; |
    5 : Colors.itextcol:=rgb; |
    6 : Colors.ihitextcol:=rgb; |
    7 : Colors.ialacol:=rgb; |
    8 : Colors.ipromptcol:=rgb; |
    9 : Colors.ientercol:=rgb; |
    10 : Colors.iselcol:=rgb; |
    11 : Colors.iunselcol:=rgb; |
    12 : Colors.iexitcol:=rgb;
  END;

  CASE state OF
    1 : Grawin.SetInfoBackCol(rgb); |
    2 : Grawin.SetInfoBorCol1(rgb); |
    3 : Grawin.SetInfoBorCol2(rgb); |
    5 : Grawin.SetInfoCol(rgb); |
    6 : Grawin.SetInfoActCol(rgb); |
    8 : yesno[1].c:=Colors.ipromptcol;
        yesno[2].c:=Colors.ipromptcol; |
  END;

  text[1].t:=" This is a title-line ";
  text[1].c:=Colors.ititlecol;
  text[2].t:=" -------------------  ";
  text[2].c:=Colors.ititlecol;
  text[3].t:=" !! ATTENTION !!      ";
  text[3].c:=Colors.ialacol;
  text[4].t:=" Inactive textline    ";
  text[4].c:=Colors.itextcol;
  text[5].t:=">Active textline      ";
  text[5].c:=Colors.ihitextcol;
  text[6].t:=" Enter a value:       ";
  text[6].c:=Colors.ipromptcol;
  text[7].t:=" 10.0                  ";
  text[7].c:=Colors.ientercol;
  text[8].t:=" YES : selected  icon  ";
  text[8].c:=Colors.iselcol;
  text[9].t:=" NO : unselected icon  ";
  text[9].c:=Colors.iunselcol;
  text[10].t:=" DONE: exit-item       ";
  text[10].c:=Colors.iexitcol;

  Grawin.ShowInfoBox(46,10,10,text);
END ShowTestBox;

PROCEDURE ChangeColorsInt(actwin, objnum : CARDINAL);
VAR x, y, horwin : REAL;
    step1, winnum : CARDINAL;
    i, ii : INTEGER;
    rgb : Grawin.RGB;
    poi : Grawin.Point;
    pp : Grawin.Coord;
    p1, p2 : DDD.DCoord;

BEGIN

  IF Grawin.InErrorState() THEN Error; RETURN; END;

  GetNumbers();

  IF worldmode THEN
    Grawin.CreateWin("Test",dddwin+1,42,10,75,25,Colors.wwinborcol,
                     Colors.wwinbackcol,Colors.wwintitlecol,TRUE,FALSE);
    Grawin.SetCursor(dddwin+1,-40,-70,Colors.wcurcol,TRUE);
  ELSE
    Grawin.CreateWin("Test",dddwin+1,42,10,75,25,Colors.swinborcol,
                     Colors.swinbackcol,Colors.swintitlecol,TRUE,FALSE);
    Grawin.SetCursor(dddwin+1,-40,-70,Colors.scurcol,TRUE);
  END;

  pp:=Grawin.GetWinGraphicSize(dddwin+1);
  pp.x:=pp.x DIV 2;
  pp.y:=pp.y DIV 2;
  Grawin.SetWinOrg(dddwin+1,pp.x,pp.y);
  NewTerm.SetWindow(40*8,10*16,60*8,25*16);

  Grawin.CreateObj(dddwin+1,crossobj,1.0,1.0,pp.x,pp.y);
  DDD.CreateCross(dddwin+1,crossobj,Grawin.GetCursorstep());
  Grawin.CreateObj(dddwin+1,tempobj1,1.0,1.0,pp.x+15,pp.y+20);
  Grawin.CreateObj(dddwin+1,tempobj2,1.0,1.0,pp.x+15,pp.y+20);

  step1:=360 DIV 15;
  i:=0;
  REPEAT
    horwin:=FLOAT(i)/180.0*DDD.PI;
    x:=sin(horwin)*50.0;
    y:=cos(horwin)*40.0;
    Grawin.AddObjPoint(dddwin+1,tempobj1,DDD.ROUND(x),DDD.ROUND(y),Colors.white,i<>0);
    i:=i+INTEGER(step1);
  UNTIL i=INTEGER(360+step1);

  IF worldmode THEN
    rgb:=Colors.wcurcol;
  ELSE
    rgb:=Colors.scurcol;
  END;

  Grawin.AddObjPoint(dddwin+1,tempobj2,-50,-40,rgb,FALSE);
  Grawin.AddObjPoint(dddwin+1,tempobj2, 50,-40,rgb,TRUE);
  Grawin.AddObjPoint(dddwin+1,tempobj2, 50, 40,rgb,TRUE);
  Grawin.AddObjPoint(dddwin+1,tempobj2,-50, 40,rgb,TRUE);
  Grawin.AddObjPoint(dddwin+1,tempobj2,-50,-40,rgb,TRUE);
  Grawin.DrawWin(dddwin+1,FALSE,TRUE);

  REPEAT

    text[1].t:="Change Colors.";
    text[2].t:="Please choose:";
    text[1].c:=Colors.ititlecol;
    text[2].c:=Colors.ipromptcol;
    Grawin.ShowInfoBox(10,10,2,text);

    text[1].t:="Window Colors....";
    text[2].t:="Menu Colors......";
    text[3].t:="Infobox Colors...";
    text[4].t:="Other Colors.....";

    Grawin.ColFromTo(text,Colors.itextcol,1,4);
    i:=Grawin.ShowChooseBox(12,14,4,text,nokeys,alltrue,1);

    CASE i OF
      1 : ShowTestWin(rgb,0);
          text[1].t:="Windowplane............";
          text[2].t:="Inactive windowframe...";
          text[3].t:="Active windowframe.....";
          text[4].t:="Windowtitle............";
          text[5].t:="Screenplane............";
          text[6].t:="Cursor.................";
          Grawin.ColFromTo(text,Colors.itextcol,1,6);

          REPEAT
            ii:=Grawin.ShowChooseBox(12,14,6,text,nokeys,alltrue,1);
            IF ii<>0 THEN Guitool.ColorPicker(10,10,ii,ShowTestWin); END;
          UNTIL ii=0; |

      2 : ShowTestWin(rgb,0);
          text[1].t:="Menuplane...........";
          text[2].t:="Inactive menuitem...";
          text[3].t:="Selected menuitem...";
          text[4].t:="Menutitle...........";
          text[5].t:="Menuborder outside..";
          text[6].t:="Menuborder inside...";
          Grawin.ColFromTo(text,Colors.itextcol,1,6);
          Grawin.OpenMenu(dddwin+1);

          REPEAT
            ii:=Grawin.ShowChooseBox(12,14,6,text,nokeys,alltrue,1);
            IF ii<>0 THEN Guitool.ColorPicker(10,10,ii,ShowTestMenu); END;
          UNTIL ii=0; |

      3 : REPEAT
            ShowTestBox(Colors.ititlecol,4);
            text[1].t:="Infoboxplane................";
            text[2].t:="Infoboxframe outside........";
            text[3].t:="Infoboxframe inside.........";
            text[4].t:="Infoboxtitle................";
            text[5].t:="Infobox inactive textline...";
            text[6].t:="Infobox active textline.....";
            text[7].t:="Infobox attention-line......";
            text[8].t:="Infobox prompt-line.........";
            text[9].t:="Infobox enter-line..........";
            text[10].t:="Infobox selected item.......";
            text[11].t:="Infobox unselected item.....";
            text[12].t:="Infobox exit-item...........";

            Grawin.ColFromTo(text,Colors.itextcol,1,12);

            ii:=Grawin.ShowChooseBox(12,14,12,text,nokeys,alltrue,1);
            IF ii<>0 THEN Guitool.ColorPicker(10,10,ii,ShowTestBox); END;
          UNTIL ii=0; |

      4 : rgb:=eyecol;
          ShowTestWin2(rgb,0);
          text[1].t:="Eye................";
          text[2].t:="Projection-plane...";
          text[3].t:="Cross..............";
          text[4].t:="Grid...............";
          text[5].t:="Whereframe.........";
          text[6].t:="Origin.............";
          text[7].t:="Marker.............";
          text[8].t:="Stereo-green.......";
          text[9].t:="Stereo-red.........";
          text[10].t:="Viewer screencol...";
          text[11].t:="Viewer infoline....";

          Grawin.ColFromTo(text,Colors.itextcol,1,11);

          REPEAT
            ii:=Grawin.ShowChooseBox(12,14,11,text,nokeys,alltrue,1);
            IF ii<>0 THEN Guitool.ColorPicker(10,10,ii,ShowTestWin2); END;
          UNTIL ii=0;|
    END;
  UNTIL i=0;
  Grawin.DestroyWin(dddwin+1);
END ChangeColorsInt;

PROCEDURE About();
VAR
    i : CARDINAL;
BEGIN

  text[1].c:=Colors.ititlecol;
  text[2].c:=Colors.ialacol;
  Grawin.ColFromTo(text,Colors.itextcol,3,8);
  text[9].c:=Colors.ipromptcol;
  text[10].c:=Colors.ipromptcol;

  text[1].t:=" `Poor World' V. 1.0, a -CAD-Program ";
  text[2].t:="______________________________________";
  text[3].t:=" Created by Michael Wessel, MW-Soft,  ";
  text[4].t:=" 3/94-1/95.  Very, very many thanks   ";
  text[5].t:="  have to go to Thomas Schubert, who  ";
  text[6].t:="   created fastest graphic-routines   ";
  text[7].t:="  ever seen on TopSpeed-Modula-2 and  ";
  text[8].t:="  wrote many other helpfull routines  ";
  text[9].t:=" `Poor World' is PUBLIC DOMAIN. Give  ";
  text[10].t:="       it to anyone you want.         ";
  Grawin.ShowInfoBox(18,10,10,text);
  WaitForAnything;
END About;

BEGIN

  yesno[1].t:=" Yes, of course! ";
  yesno[2].t:=" Oh no, don't do so! ";
  yesno[1].c:=Colors.ipromptcol;
  yesno[2].c:=Colors.ipromptcol;

  errortext[1].t:="***** FATAL ERROR *****";
  errortext[2].t:=" Sorry, no more MEMORY ";
  errortext[3].t:=" left. Save your work! ";
  errortext[1].c:=Colors.ititlecol;
  errortext[2].c:=Colors.ialacol;
  errortext[3].c:=Colors.ialacol;

  FOR i:=1 TO WorldEd.absolutmaxobjs DO
    alltrue[i]:=TRUE;
    nokeys[i]:="";
  END;

END IntMen.
