IMPLEMENTATION MODULE Mouse;

FROM NewTerm IMPORT Scan, Read, Write;
IMPORT Info;
IMPORT Lib;
IMPORT SYSTEM;

TYPE stackelem = RECORD
                   fromx, fromy, tox, toy : CARDINAL;
                   mousex, mousey : CARDINAL;
                 END;

VAR stack : ARRAY[1..10] OF stackelem;
    stackpointer : CARDINAL;

    r : SYSTEM.Registers;
    lastbutton, afromx, afromy, atox, atoy, mousex, mousey : CARDINAL;
    flag, leftpressed, rightpressed, poschanged, butleft, butright : BOOLEAN;

    waitmax : CARDINAL;
    waitproc : PROC;
    enabled : BOOLEAN;

PROCEDURE NilProc();
BEGIN
END NilProc;

PROCEDURE SetWaitProc(wproc : PROC);
BEGIN
  waitproc:=wproc;
END SetWaitProc;

PROCEDURE EnableWaitProcEvery(time : CARDINAL);
BEGIN
  waitmax:=time;
  enabled:=TRUE;
END EnableWaitProcEvery;

PROCEDURE DisableWaitProc();
BEGIN
  enabled:=FALSE;
END DisableWaitProc;

PROCEDURE WaitForEvents(VAR left, right, pos, key, strgkey : BOOLEAN;
                        VAR keycode : CHAR);

VAR button, x, y : CARDINAL;
    dt, h, m, s, s100, as, as100 : INTEGER;

BEGIN
  IF enabled THEN
    Info.GetTime(h,m,s,s100);
    as:=s;
    as100:=s100;
  END;

  REPEAT

    r.AX:=3;
    Lib.Intr(r,51);
    x:=r.CX;
    y:=r.DX;
    button:=r.BX;
    Scan(keycode,key);

    IF enabled THEN
      Info.GetTime(h,m,s,s100);
      dt:=ABS(s-as)*100+(s100-as100);

      IF dt>INTEGER(waitmax) THEN
        waitproc;
        Info.GetTime(h,m,as,as100);
      END;
    END;

  UNTIL (x<>mousex) OR (y<>mousey) OR (lastbutton<>button) OR key;

  leftpressed:=(button=1);
  rightpressed:=(button=2);

  poschanged:=(x<>mousex) OR (y<>mousey);
  pos:=poschanged;

  IF button=0 THEN
    butleft:=(lastbutton=1);
    butright:=(lastbutton=2);
    left:=butleft;
    right:=butright;
  ELSE
    left:=FALSE;
    right:=FALSE;
  END;

  IF key AND (keycode=CHR(0)) THEN
    strgkey:=TRUE;
    Read(keycode);
  ELSE
    strgkey:=FALSE;
  END;

  lastbutton:=button;
  mousex:=x;
  mousey:=y;

END WaitForEvents;

PROCEDURE WaitForAnything();
VAR m : CARDINAL;
    keycode : CHAR;
    key : BOOLEAN;

BEGIN
  r.AX:=3;
  Lib.Intr(r,51);
  m:=r.BX;
  REPEAT
    r.AX:=3;
    Lib.Intr(r,51);
    Scan(keycode,key);
  UNTIL (m<>r.BX) OR key;

  REPEAT
    r.AX:=3;
    Lib.Intr(r,51);
  UNTIL (r.BX=0);
END WaitForAnything;

PROCEDURE GetMouseState(VAR x,y : CARDINAL; VAR left, right : BOOLEAN);
VAR button : CARDINAL;
BEGIN
  r.AX:=3;
  Lib.Intr(r,51);
  x:=r.CX;
  y:=r.DX;
  button:=r.BX;

  left:=button=1;
  right:=button=2;

  mousex:=x;
  mousey:=y;

END GetMouseState;

PROCEDURE InitMouse(xmin, ymin, xmax, ymax : CARDINAL);
BEGIN
  butleft:=FALSE;
  butright:=FALSE;
  poschanged:=FALSE;

  r.AX:=7;
  r.CX:=xmin;
  r.DX:=xmax;
  Lib.Intr(r,33H);

  r.AX:=8;
  r.CX:=ymin;
  r.DX:=ymax;

  afromx:=xmin;
  afromy:=ymin;
  atox:=xmax;
  atoy:=ymax;
  Lib.Intr(r,33H);

  r.AX:=15;
  r.CX:=8;
  r.DX:=16;
  Lib.Intr(r,33H);

  r.AX:=2; (*Mouse-Cursor off*)
  Lib.Intr(r,51);

  r.AX:=04;
  r.CX:=0;
  r.DX:=0;
  Lib.Intr(r,51);
  mousex:=0;
  mousey:=0;
END InitMouse;

PROCEDURE SetMousePos(xpos, ypos : CARDINAL);
BEGIN
  r.AX:=04;
  r.CX:=xpos;
  r.DX:=ypos;
  Lib.Intr(r,51);
  mousex:=xpos;
  mousey:=ypos;
  butleft:=FALSE;
  butright:=FALSE;
  poschanged:=FALSE;
END SetMousePos;

PROCEDURE StoreMousePos();
BEGIN
  INC(stackpointer);
  (* r.AX:=3;
  Lib.Intr(r,51);
  mousex:=r.CX;
  mousey:=r.DX; *)

  stack[stackpointer].mousex:=r.CX;
  stack[stackpointer].mousey:=r.DX;

  stack[stackpointer].fromx:=afromx;
  stack[stackpointer].fromy:=afromy;
  stack[stackpointer].tox:=atox;
  stack[stackpointer].toy:=atoy;
END StoreMousePos;

PROCEDURE RestoreMousePos();
BEGIN
  mousex:=stack[stackpointer].mousex;
  mousey:=stack[stackpointer].mousey;
  WITH stack[stackpointer] DO

    r.AX:=7;
    r.CX:=fromx;
    r.DX:=tox;
    Lib.Intr(r,33H);

    r.AX:=8;
    r.CX:=fromy;
    r.DX:=toy;
    Lib.Intr(r,33H);

    afromx:=fromx;
    afromy:=fromy;
    atox:=tox;
    atoy:=toy;

  END;
  r.AX:=04;
  r.CX:=mousex;
  r.DX:=mousey;
  Lib.Intr(r,51);
  DEC(stackpointer);
END RestoreMousePos;

PROCEDURE LeftButtonPressed():BOOLEAN;
BEGIN
  RETURN(butleft);
END LeftButtonPressed;

PROCEDURE RightButtonPressed():BOOLEAN;
BEGIN
  RETURN(butright);
END RightButtonPressed;

PROCEDURE PosChanged():BOOLEAN;
BEGIN
  RETURN(poschanged);
END PosChanged;

PROCEDURE ConfLeftButton();
BEGIN
  lastbutton:=0;
  butleft:=FALSE;
  leftpressed:=FALSE;
END ConfLeftButton;

PROCEDURE ConfRightButton();
BEGIN
  lastbutton:=0;
  butright:=FALSE;
  rightpressed:=FALSE;
END ConfRightButton;

PROCEDURE ConfPosChanged();
BEGIN
  poschanged:=FALSE;
  r.AX:=3;
  Lib.Intr(r,51);
  mousex:=r.CX;
  mousey:=r.DX;
  lastbutton:=r.BX;
END ConfPosChanged;

PROCEDURE LeftAndMove():BOOLEAN;
BEGIN
  RETURN(leftpressed);
END LeftAndMove;

PROCEDURE RightAndMove():BOOLEAN;
BEGIN
  RETURN(rightpressed);
END RightAndMove;

BEGIN
  stackpointer:=0;
  waitproc:=NilProc;
  enabled:=FALSE;
END Mouse.




