IMPLEMENTATION MODULE NewTerm;




TYPE
    InfoType     = RECORD
        Width         : INTEGER;
        Height        : INTEGER;

        WinSize       : LONGINT;
        NextBank      : INTEGER;
        LineLength    : LONGINT;
        BitsPerPixel  : INTEGER;

        RedSize       : INTEGER;
        RedPos        : INTEGER;
        GreenSize     : INTEGER;
        GreenPos      : INTEGER;
        BlueSize      : INTEGER;
        BluePos       : INTEGER;
    END;





VAR
    VESA                 : InfoType;

    SetFGColorPROC,
    SetBGColorPROC       : PROCEDURE( CARDINAL, CARDINAL, CARDINAL );
    PutBitmapPROC        : PROCEDURE( VAR ARRAY OF BITSET,
                                      INTEGER, INTEGER,
                                      INTEGER, INTEGER );
    PutFontPROC          : PROCEDURE( VAR ARRAY OF BITSET,
                                      INTEGER, INTEGER,
                                      INTEGER, INTEGER );
    ScrollPROC           : PROCEDURE( INTEGER, INTEGER, INTEGER, INTEGER, INTEGER );
    WritePROC            : PROCEDURE( CHAR );
    GotoXYPROC           : PROCEDURE( INTEGER, INTEGER );
    GetXYPROC            : PROCEDURE( VAR INTEGER, VAR INTEGER );

    GetLinePROC          : PROCEDURE( VAR ARRAY OF SHORTCARD,
                                      INTEGER, INTEGER, INTEGER, INTEGER );
    PutLinePROC          : PROCEDURE( VAR ARRAY OF SHORTCARD,
                                      INTEGER, INTEGER, INTEGER, INTEGER );
    PlotPROC             : PROCEDURE( INTEGER, INTEGER );
    PointPROC            : PROCEDURE( INTEGER, INTEGER,
                                      VAR CARDINAL, VAR CARDINAL, VAR CARDINAL );
    LineHorizontal       : PROCEDURE( INTEGER, INTEGER, INTEGER, INTEGER );
    LineLE45, LineGE45   : PROCEDURE( INTEGER, INTEGER, INTEGER, INTEGER,
                                      LONGINT, LONGINT, LONGINT );

    columns[0040H:004AH] : INTEGER;
    rows[0040:0084H]     : SHORTCARD;

    maxwin, win          : RECORD
                               xmin : INTEGER;
                               ymin : INTEGER;
                               xmax : INTEGER;
                               ymax : INTEGER;
                           END;
    cursor_state         : BOOLEAN;
    fg_color, bg_color   : LONGCARD;
    bank                 : INTEGER;




PROCEDURE Beep(  );
BEGIN
    IO.WrChar( CHR( 7 ) );
END Beep;








MODULE TEXT;
(*
==============================================================================
BEGIN: 40*25 and 80*25 and MONO
==============================================================================
*)




IMPORT SYSTEM;
IMPORT AsmLib;
IMPORT NewTerm;




VAR
    cursor             : RECORD
                             x : INTEGER;
                             y : INTEGER;
                         END;



PROCEDURE SetFGColor( red, green, blue : CARDINAL );
BEGIN
    NewTerm.fg_color := LONGCARD( red );
END SetFGColor;




PROCEDURE SetBGColor( red, green, blue : CARDINAL );
BEGIN
    NewTerm.bg_color := LONGCARD( red );
END SetBGColor;




PROCEDURE GotoXY( x, y : INTEGER );
VAR
    r : SYSTEM.Registers;
BEGIN
    IF  ( x >= NewTerm.win.xmin )
        AND
        ( x <= NewTerm.win.xmax )
        AND
        ( y >= NewTerm.win.ymin )
        AND
        ( y <= NewTerm.win.ymax )
        THEN
            cursor.x := x;
            cursor.y := y;
            r.AH := 02H;
            r.BH := 00H;
            r.DH := SHORTCARD( y );
            r.DL := SHORTCARD( x );
            AsmLib.Intr( r, 10H );
    END;
END GotoXY;




PROCEDURE GetXY( VAR x, y : INTEGER );
BEGIN
    x := cursor.x;
    y := cursor.y;
END GetXY;




PROCEDURE Cursor ( state : BOOLEAN );
VAR
    r : SYSTEM.Registers;
BEGIN
    r.AH   := 03H; (* Get Cursor-Shape *)
    AsmLib.Intr ( r , 10H );
    r.CH := r.CH MOD 32; (* Tunr On *)
    r.CL := r.CL MOD 32;

    IF  NOT ( state ) (* Turn off *)
        THEN
            INC ( r.CH , 32 );
    END;

    r.AH := 01H; (* Set Cursor-Shape *)
    AsmLib.Intr ( r , 10H );
END Cursor;




PROCEDURE Scroll( x1, y1, x2, y2, times : INTEGER );
VAR
    r : SYSTEM.Registers;
BEGIN
    IF  ( times >= 0 )
        THEN
            r.AH := 06H; (* scroll up *)
        ELSE
            r.AH := 07H; (* scroll down *)
    END;
    r.AL := SHORTCARD( ABS( times ) );
    r.BH := SHORTCARD( NewTerm.bg_color ) << 4;
    r.CL := SHORTCARD( x1 );
    r.CH := SHORTCARD( y1 );
    r.DL := SHORTCARD( x2 );
    r.DH := SHORTCARD( y2 );
    AsmLib.Intr( r, 10H );
END Scroll;




PROCEDURE Write( ch : CHAR );
VAR
    r    : SYSTEM.Registers;
    x, y : INTEGER;
    i    : INTEGER;
    map  : BITSET;
BEGIN
    GetXY( x, y );

    CASE ch OF
        CHR( 7 )  : r.AH := 0EH;
                    r.AL := SHORTCARD( ch );
                    AsmLib.Intr( r, 10H );
                    RETURN;
                    |
        CHR( 8 )  : IF  ( x >= NewTerm.win.xmin )
                        THEN
                            DEC( x );
                    END;
                    GotoXY( x, y );
                    RETURN;
                    |
        CHR( 10 ) : IF  ( (y+1) > NewTerm.win.ymax )
                        THEN
                            NewTerm.Scroll( 1 );
                        ELSE
                            INC( y );
                    END;
                    GotoXY( x, y );
                    RETURN;
                    |
        CHR( 13 ) : x := NewTerm.win.xmin;
                    GotoXY( x, y );
                    RETURN;
                    |
    END;
(*
    IF  ( ORD( ch ) < 32 )
        THEN
            RETURN;
    END;
*)
    r.AH := 09H;
    r.AL := SHORTCARD( ch );
    r.BH := 00H;
    r.BL := SHORTCARD( NewTerm.fg_color )
            + SHORTCARD( NewTerm.bg_color ) << 4;
    r.CX := 01H;
    AsmLib.Intr( r, 10H );

    INC( x );
    IF  ( x > NewTerm.win.xmax )
        THEN
            x := NewTerm.win.xmin;
            INC( y );
    END;
    IF  ( y > NewTerm.win.ymax )
        THEN
            NewTerm.Scroll( 1 );
            DEC( y );
    END;

    GotoXY( x, y );
END Write;




PROCEDURE Plot( x, y : INTEGER );
BEGIN
END Plot;




PROCEDURE Point( x, y : INTEGER;
                 red, green, blue : INTEGER );
BEGIN
    red   := 0;
    green := 0;
    blue  := 0;
END Point;




PROCEDURE LineHorizontal( x1, y1, x2, y2 : INTEGER );

BEGIN
END LineHorizontal;




PROCEDURE Line( x1, y1, x2, y2 : INTEGER;
                dx, dy, next   : LONGINT );
BEGIN
END Line;




(*
==============================================================================
END: TEXT 40*25 and 80*25 and MONO
==============================================================================
*)
END TEXT;








MODULE GRAPH;
(*
==============================================================================
BEGIN: PROCEDURES FOR ALL GRAPH-MODES
==============================================================================
*)




IMPORT NewTerm;
IMPORT AsmLib;
IMPORT SYSTEM;




VAR
    cursor : RECORD
                 x : INTEGER;
                 y : INTEGER;
             END;
    save   : ARRAY[0..5119]OF SHORTCARD;




PROCEDURE Scroll( x1, y1, x2, y2, times : INTEGER );
VAR
    tmp     : INTEGER;
    xormode : BOOLEAN;
    savecol : LONGCARD;
BEGIN
    savecol          := NewTerm.fg_color;
    NewTerm.fg_color := NewTerm.bg_color;
    xormode          := NewTerm.xormode;
    NewTerm.xormode  := FALSE;
    IF      ( times = 0 )
            THEN
                WHILE
                    ( y1 <= y2 )
                DO
                    NewTerm.LineHorizontal( x1, y1, x2, y1 );
                    INC( y1 );
                END;
    ELSIF   ( times > 0 )
            THEN
                tmp := y2 - times;
                WHILE
                    ( y1 <= tmp )
                DO
                    NewTerm.GetLinePROC( save, x1, y1+times, 0, 1+x2-x1 );
                    NewTerm.PutLinePROC( save, x1, y1, 0, 1+x2-x1 );
                    INC( y1 );
                END;
                WHILE
                    ( y1 <= y2 )
                DO
                    NewTerm.LineHorizontal( x1, y1, x2, y1 );
                    INC( y1 );
                END;
    ELSIF   ( times < 0 )
            THEN
                tmp := y1 - times;
                WHILE
                    ( y2 >= tmp )
                DO
                    NewTerm.GetLinePROC( save, x1, y2 + times, 0, 1+x2-x1 );
                    NewTerm.PutLinePROC( save, x1, y2, 0, 1+x2-x1 );
                    DEC( y2 );
                END;
                WHILE
                    ( y2 >= y1 )
                DO
                    NewTerm.LineHorizontal( x1, y2, x2, y2 );
                    DEC( y2 );
                END;
    END;
    NewTerm.fg_color := savecol;
    NewTerm.xormode  := xormode;
END Scroll;




PROCEDURE GetXY( VAR x, y : INTEGER );
BEGIN
    x := cursor.x;
    y := cursor.y;
END GetXY;




PROCEDURE GotoXY( x, y : INTEGER );
BEGIN
    IF  ( x >= NewTerm.win.xmin )
        AND
        ( ( x+NewTerm.font_width-1 ) <= ( NewTerm.win.xmax ) )
        AND
        ( y >= NewTerm.win.ymin )
        AND
        ( ( y + NewTerm.font_height-1 ) <= ( NewTerm.win.ymax ) )
        THEN
            cursor.x := x;
            cursor.y := y;
    END;
END GotoXY;




PROCEDURE Write( ch : CHAR );
VAR
    r    : SYSTEM.Registers;
    x, y : INTEGER;
    i    : INTEGER;
    map  : BITSET;
BEGIN
    GetXY( x, y );

    CASE ch OF
        CHR( 7 )  : r.AH := 0EH;
                    r.AL := SHORTCARD( ch );
                    AsmLib.Intr( r, 10H );
                    RETURN;
                    |
        CHR( 8 )  : IF  ( ( x - NewTerm.font_width ) >= NewTerm.win.xmin )
                        THEN
                            DEC( x, NewTerm.font_width );
                    END;
                    GotoXY( x, y );
                    RETURN;
                    |
        CHR( 10 ) : IF  ( (y+2*NewTerm.font_height-1) > NewTerm.win.ymax )
                        THEN
                            NewTerm.Scroll( NewTerm.font_height );
                        ELSE
                            INC( y, NewTerm.font_height );
                    END;
                    GotoXY( x, y );
                    RETURN;
                    |
        CHR( 13 ) : x := NewTerm.win.xmin;
                    GotoXY( x, y );
                    RETURN;
                    |
    END;
(*
    IF  ( ORD( ch ) < 32 )
        THEN
            RETURN;
    END;
*)

    FOR i := 0 TO NewTerm.font_height-1 DO
(*
        map := NewTerm.FONT[ ORD(ch), i ];
        NewTerm.PutBitmapPROC( map, 0, 8, x, y+i, NewTerm.text_fg_color );
        map := BITSET( 0FFFFH ) - map;
        NewTerm.PutBitmapPROC( map, 0, 8, x, y+i, NewTerm.text_bg_color );
*)
        NewTerm.PutFontPROC( NewTerm.FONT[ORD(ch),i],
                              0, NewTerm.font_width,
                              x, y+i );
    END;

    INC( x, NewTerm.font_width );
    IF  ( (x+NewTerm.font_width) > (NewTerm.win.xmax+1) )
        THEN
            x := NewTerm.win.xmin;
            INC( y, NewTerm.font_height );
    END;
    IF  ( (y+NewTerm.font_height) > (NewTerm.win.ymax+1) )
        THEN
            NewTerm.Scroll( NewTerm.font_height );
            DEC( y, NewTerm.font_height );
    END;

    GotoXY( x, y );
END Write;




(*
==============================================================================
END: PROCEDURES FOR ALL GRAPH-MODES
==============================================================================
*)
END GRAPH;







MODULE C256;
(*
==============================================================================
BEGIN: VGA 640*480*256
==============================================================================
*)


IMPORT SYSTEM;
IMPORT AsmLib;
IMPORT NewTerm;
FROM NewTerm IMPORT VESA;


VAR
    Screen[0A000H:0000H] : ARRAY[0..65535]OF SHORTCARD;
    save                 : ARRAY[0..1279]OF SHORTCARD;




PROCEDURE SetFGColor( red, green, blue : CARDINAL );
BEGIN
    NewTerm.fg_color := LONGCARD( ( red >> 13 ) << 5 );
    NewTerm.fg_color := NewTerm.fg_color + LONGCARD( ( green >> 13 ) << 2 );
    NewTerm.fg_color := NewTerm.fg_color + LONGCARD( blue >> 14 );
END SetFGColor;




PROCEDURE SetBGColor( red, green, blue : CARDINAL );
BEGIN
    NewTerm.bg_color := LONGCARD( ( red >> 13 ) << 5 );
    NewTerm.bg_color := NewTerm.bg_color + LONGCARD( ( green >> 13 ) << 2 );
    NewTerm.bg_color := NewTerm.bg_color + LONGCARD( blue >> 14 );
END SetBGColor;




PROCEDURE SetBank( bank : INTEGER );
VAR
    r : SYSTEM.Registers;
BEGIN
    IF  ( bank <> NewTerm.bank )
        THEN
            r.AX := 4F05H;
            r.BX := 0000H;
            NewTerm.bank := bank;
            r.DX := bank;
			AsmLib.Intr ( r , 10H );
    END;
END SetBank;




PROCEDURE Plot ( x , y : INTEGER );
TYPE
    TINYSET = SET OF [0..7];
VAR
    index   : LONGINT;
    newbank : INTEGER;
BEGIN
    index := LONGINT( y ) * VESA.LineLength + LONGINT( x );
    newbank := INTEGER( index >> 16 ) * VESA.NextBank;
    SetBank( newbank );
    IF  ( NewTerm.xormode )
        THEN
            Screen[CARDINAL(index)] := SHORTCARD(TINYSET( Screen[CARDINAL(index)] )
                                       / TINYSET(NewTerm.fg_color));
        ELSE
            Screen[CARDINAL(index)] := SHORTCARD(NewTerm.fg_color);
    END;
END Plot;




PROCEDURE Point ( x , y : INTEGER;
                  VAR red, green, blue : CARDINAL );
VAR
    index   : LONGINT;
    newbank : INTEGER;
    point   : CARDINAL;
BEGIN
    index := LONGINT( y ) * VESA.LineLength + LONGINT( x );
    newbank := INTEGER( index >> 16 ) * VESA.NextBank;
    SetBank( newbank );
    point := CARDINAL( Screen[CARDINAL(index)] );
(*
    red   := 8191 + ( point >> 5 ) << 13;
    green := 8191 + ( point >> 2 ) << 13;
    blue  := 8191 + ( point << 14;
*)
    red   := ( point >> 5 ) << 13;
    green := ( point >> 2 ) << 13;
    blue  := ( point << 14 );
END Point;




PROCEDURE LineHorizontal ( x1 , y1 , x2 , y2 : INTEGER );
TYPE
    TINYSET = SET OF [0..7];
VAR
    index1, index2     : LONGINT;
    newbank1, newbank2 : INTEGER;
BEGIN
    index1   := LONGINT( y1 ) * VESA.LineLength + LONGINT( x1 );
    newbank1 := INTEGER( index1 >> 16 ) * VESA.NextBank;
    index1   := LONGINT( CARDINAL( index1 ) );
    index2   := LONGINT( y2 ) * VESA.LineLength + LONGINT( x2 );
    newbank2 := INTEGER( index2 >> 16 ) * VESA.NextBank;
    index2   := LONGINT( CARDINAL( index2 ) );
    SetBank( newbank1 );

    IF      ( NewTerm.xormode )
            AND
            ( newbank1 = newbank2 )
            THEN
                WHILE
                    ( index1 <= index2 )
                DO
                    Screen[CARDINAL(index1)] := SHORTCARD( TINYSET(Screen[CARDINAL(index1)])
                                                           / TINYSET(NewTerm.fg_color) );
                    INC( index1 );
                END;
    ELSIF   ( NewTerm.xormode )
            AND
            ( newbank1 <> newbank2 )
            THEN
                WHILE
                    ( index1 < 65536 )
                DO
                    Screen[CARDINAL(index1)] := SHORTCARD( TINYSET(Screen[CARDINAL(index1)])
                                                           / TINYSET(NewTerm.fg_color) );
                    INC( index1 );
                END;
                index1 := 0;
                SetBank( newbank2 );
                WHILE
                    ( index1 <= index2 )
                DO
                    Screen[CARDINAL(index1)] := SHORTCARD( TINYSET(Screen[CARDINAL(index1)])
                                                           / TINYSET(NewTerm.fg_color) );
                    INC( index1 );
                END;
    ELSIF   ( newbank1 = newbank2 )
            THEN
                AsmLib.Fill( [0A000H:CARDINAL(index1)],
                             1 + x2 - x1,
                             SHORTCARD(NewTerm.fg_color) );
    ELSIF   ( newbank1 <> newbank2 )
            THEN
                AsmLib.Fill( [0A000H:CARDINAL(index1)],
                             CARDINAL(65536-index1),
                             SHORTCARD(NewTerm.fg_color) );
                SetBank( newbank2 );
                AsmLib.Fill( [0A000H:0H],
                             CARDINAL( 1 + index2 ),
                             SHORTCARD(NewTerm.fg_color) );
    END;
END LineHorizontal;




PROCEDURE LineLE45 ( x1 , y1 , x2 , y2 : INTEGER;
                     dx, dy, next      : LONGINT );
TYPE
    TINYSET = SET OF [0..7];
VAR
    tmp     : INTEGER;
    add     : LONGINT;
    index   : LONGINT;
    newbank : INTEGER;
BEGIN
    IF  ( dy >= 0 )
        THEN
            add := VESA.LineLength;
        ELSE
            add := -VESA.LineLength;
            dy  := ABS( dy );
    END;
    index := LONGINT( y1 ) * VESA.LineLength + LONGINT( x1 );
    newbank := INTEGER( index >> 16) * VESA.NextBank;
    index := LONGINT( CARDINAL( index ) );
    SetBank( newbank );

    IF  ( NewTerm.xormode )
        THEN
            FOR tmp := x1 TO x2 DO
                Screen[CARDINAL(index)] := SHORTCARD(TINYSET(Screen[CARDINAL(index)])
                                                             / TINYSET(NewTerm.fg_color));
                INC( index );
                next := next - dy;
                IF  ( next < 0 )
                    THEN
                        next := next + dx;
                        INC ( index , add );
                END;
                IF      ( index >= 65536 )
                        THEN
                            DEC( index, 65536 );
                            INC( newbank, VESA.NextBank );
                            SetBank( newbank );
                ELSIF   ( index <= 0 )
                        AND
                        ( NewTerm.bank > 0 )
                        THEN
                            INC( index, 65536 );
                            DEC( newbank, VESA.NextBank );
                            SetBank( newbank );
                 END;
            END;
        ELSE
            FOR tmp := x1 TO x2 DO
                Screen[CARDINAL(index)] := SHORTCARD(NewTerm.fg_color);
                INC( index );
                next := next - dy;
                IF  ( next < 0 )
                    THEN
                        next := next + dx;
                        INC ( index , add );
                END;
                IF      ( index >= 65536 )
                        THEN
                            DEC( index, 65536 );
                            INC( newbank, VESA.NextBank );
                            SetBank( newbank );
                ELSIF   ( index < 0 )
                        AND
                        ( NewTerm.bank > 0 )
                        THEN
                            INC( index, 65536 );
                            DEC( newbank, VESA.NextBank );
                            SetBank( newbank );
                END;
            END;
    END;
END LineLE45;




PROCEDURE LineGE45 ( x1 , y1 , x2 , y2 : INTEGER;
                     dx, dy, next      : LONGINT );
TYPE
    TINYSET = SET OF [0..7];
VAR
    tmp     : INTEGER;
    add     : LONGINT;
    index   : LONGINT;
    newbank : INTEGER;
BEGIN
    index := LONGINT( y1 ) * VESA.LineLength + LONGINT( x1 );
    newbank := INTEGER( index >> 16) * VESA.NextBank;
    index := LONGINT( CARDINAL( index ) );
    SetBank( newbank );
    IF  ( dy >= 0 )
        THEN
            add := VESA.LineLength;
        ELSE
            add := -VESA.LineLength;
            dy  := ABS( dy );
            tmp := y1;
            y1  := y2;
            y2  := tmp;
    END;

    IF  ( NewTerm.xormode )
        THEN
            FOR tmp := y1 TO y2 DO
                Screen[CARDINAL(index)] := SHORTCARD(TINYSET(Screen[CARDINAL(index)])
                                                     / TINYSET(NewTerm.fg_color));
                INC( index , add );
                next := next - dx;
                IF  ( next < 0 )
                    THEN
                        next := next + dy;
                        INC ( index );
                END;
                IF      ( index >= 65536 )
                        THEN
                            DEC( index, 65536 );
                            INC( newbank, VESA.NextBank );
                            SetBank( newbank );
                ELSIF   ( index < 0 )
                        AND
                        ( NewTerm.bank > 0 )
                        THEN
                            INC( index, 65536 );
                            DEC( newbank, VESA.NextBank );
                            SetBank( newbank );
                END;
            END;
        ELSE
            FOR tmp := y1 TO y2 DO
                Screen[CARDINAL(index)] := SHORTCARD(NewTerm.fg_color);
                INC( index , add );
                next := next - dx;
                IF  ( next < 0 )
                    THEN
                        next := next + dy;
                        INC ( index );
                END;
                IF      ( index >= 65536 )
                        THEN
                            DEC( index, 65536 );
                            INC( newbank, VESA.NextBank );
                            SetBank( newbank );
                ELSIF   ( index < 0 )
                        AND
                        ( NewTerm.bank > 0 )
                        THEN
                            INC( index, 65536 );
                            DEC( newbank, VESA.NextBank );
                            SetBank( newbank );
                END;
            END;
    END;
END LineGE45;




PROCEDURE GetLine( VAR save         : ARRAY OF SHORTCARD;
                   x, y, start, end : INTEGER );
VAR
    index1, index2 : LONGINT;
    tmp            : CARDINAL;
    bank1, bank2   : INTEGER;
BEGIN
    index1 := LONGINT( y ) * VESA.LineLength + LONGINT( x + start );
    index2 := index1 + LONGINT( end );
    bank1  := INTEGER( index1 >> 16 ) * VESA.NextBank;
    bank2  := INTEGER( index2 >> 16 ) * VESA.NextBank;

    IF  ( bank1 = bank2 )
        THEN
            SetBank( bank1 );
            AsmLib.Move( [0A000H:CARDINAL( index1 )],
                         [SYSTEM.Seg(save):SYSTEM.Ofs(save)+CARDINAL(start)],
                         end );
        ELSE
            tmp := CARDINAL( 65536 - LONGINT( CARDINAL( index1 ) ) );
            SetBank( bank1 );
            AsmLib.Move( [0A000H:CARDINAL( index1 )],
                         [SYSTEM.Seg(save):SYSTEM.Ofs(save)+CARDINAL(start)],
                         tmp );
            SetBank( bank2 );
            AsmLib.Move( [0A000H:0H],
                         [SYSTEM.Seg(save):SYSTEM.Ofs(save)+tmp],
                         CARDINAL( index2 ) );

    END;
END GetLine;




PROCEDURE PutLine( VAR save         : ARRAY OF SHORTCARD;
                   x, y, start, end : INTEGER );
VAR
    index1, index2 : LONGINT;
    tmp            : CARDINAL;
    bank1, bank2   : INTEGER;
BEGIN
    index1 := LONGINT( y ) * VESA.LineLength + LONGINT( x + start );
    index2 := index1 + LONGINT( end );
    bank1  := INTEGER( index1 >> 16 ) * VESA.NextBank;
    bank2  := INTEGER( index2 >> 16 ) * VESA.NextBank;

    IF  ( bank1 = bank2 )
        THEN
            SetBank( bank1 );
            AsmLib.Move( [SYSTEM.Seg(save):SYSTEM.Ofs(save)+CARDINAL(start)],
                         [0A000H:CARDINAL(index1)],
                         end );
        ELSE
            tmp := CARDINAL( 65536 - LONGINT( CARDINAL( index1 ) ) );
            SetBank( bank1 );
            AsmLib.Move( [SYSTEM.Seg(save):SYSTEM.Ofs(save)+CARDINAL(start)],
                         [0A000H:CARDINAL( index1 )],
                         tmp );
            SetBank( bank2 );
            AsmLib.Move( [SYSTEM.Seg(save):SYSTEM.Ofs(save)+tmp],
                         [0A000H:0H],
                         CARDINAL( index2 ) );

    END;
END PutLine;




PROCEDURE PutBitmap( VAR bitmap    : ARRAY OF BITSET;
                     start, end    : INTEGER;
                     x, y          : INTEGER );
TYPE
    BIT     = [0..15];
    TINYSET = SET OF [0..7];
VAR
    bit    : INTEGER;
    index  : LONGINT;
    bank   : INTEGER;
BEGIN
    index := LONGINT( y ) * VESA.LineLength + LONGINT( x );
    bank  := INTEGER( index >> 16 ) * VESA.NextBank;
    index := LONGINT( CARDINAL( index ) );
    SetBank( bank );
    bit := 15 - ( start MOD 16 );

    IF  ( NewTerm.xormode )
        THEN
            WHILE
                ( start < end )
            DO
                IF  ( BIT( bit ) IN bitmap[start >> 4] )
                    THEN
                        Screen[CARDINAL(index)] := SHORTCARD( TINYSET(Screen[CARDINAL(index)])
                                                              / TINYSET( NewTerm.fg_color ) );
                END;
                INC( start );
                DEC( bit );
                IF  ( bit < 0 )
                    THEN
                        bit := 15;
                END;
                INC( index );
                IF      ( index >= 65536 )
                        THEN
                            DEC( index, 65536 );
                            INC( bank, VESA.NextBank );
                            SetBank( bank );
                END;
           END;
       ELSE
            WHILE
                ( start < end )
            DO
                IF  ( BIT( bit ) IN bitmap[start >> 4] )
                    THEN
                        Screen[CARDINAL(index)] := SHORTCARD( NewTerm.fg_color );
                END;
                INC( start );
                DEC( bit );
                IF  ( bit < 0 )
                    THEN
                        bit := 15;
                END;
                INC( index );
                IF      ( index >= 65536 )
                        THEN
                            DEC( index, 65536 );
                            INC( bank, VESA.NextBank );
                            SetBank( bank );
                END;
           END;
    END;
END PutBitmap;




PROCEDURE PutFont( VAR bitmap         : ARRAY OF BITSET;
                   start, end         : INTEGER;
                   x, y               : INTEGER );
TYPE
    BIT     = [0..15];
    TINYSET = SET OF [0..7];
VAR
    bit    : INTEGER;
    index  : LONGINT;
    bank   : INTEGER;
BEGIN
    index := LONGINT( y ) * VESA.LineLength + LONGINT( x );
    bank  := INTEGER( index >> 16 ) * VESA.NextBank;
    index := LONGINT( CARDINAL( index ) );
    SetBank( bank );
    bit := 15 - ( start MOD 16 );

    IF  ( NewTerm.xormode )
        THEN
            WHILE
                ( start < end )
            DO
                IF  ( BIT( bit ) IN bitmap[start >> 4] )
                    THEN
                        Screen[CARDINAL(index)] := SHORTCARD( TINYSET(Screen[CARDINAL(index)])
                                                              / TINYSET( NewTerm.fg_color ) );
                    ELSE
                        Screen[CARDINAL(index)] := SHORTCARD( TINYSET(Screen[CARDINAL(index)])
                                                                      / TINYSET( NewTerm.bg_color ) );
                END;
                INC( start );
                DEC( bit );
                IF  ( bit < 0 )
                    THEN
                        bit := 15;
                END;
                INC( index );
                IF      ( index >= 65536 )
                        THEN
                            DEC( index, 65536 );
                            INC( bank, VESA.NextBank );
                            SetBank( bank );
                END;
           END;
       ELSE
            WHILE
                ( start < end )
            DO
                IF  ( BIT( bit ) IN bitmap[start >> 4] )
                    THEN
                        Screen[CARDINAL(index)] := SHORTCARD( NewTerm.fg_color );
                    ELSE
                        Screen[CARDINAL(index)] := SHORTCARD( NewTerm.bg_color );
                END;
                INC( start );
                DEC( bit );
                IF  ( bit < 0 )
                    THEN
                        bit := 15;
                END;
                INC( index );
                IF      ( index >= 65536 )
                        THEN
                            DEC( index, 65536 );
                            INC( bank, VESA.NextBank );
                            SetBank( bank );
                END;
           END;
    END;
END PutFont;




(*
==============================================================================
END: VGA 640*480*256
==============================================================================
*)
END C256;




PROCEDURE SetMode ( mode : INTEGER );
VAR
    r : SYSTEM.Registers;
BEGIN
    bank := 0;
    IF  ( mode < 100H )
        THEN
            r.AH := 00H;
            r.AL := SHORTCARD ( mode );
            AsmLib.Intr ( r , 10H );
        ELSE
            r.AX := 4F02H;
            r.BX := mode;
            AsmLib.Intr ( r , 10H );
    END;
END SetMode;




PROCEDURE GetMode (  ) : INTEGER;
VAR
    r    : SYSTEM.Registers;
    mode : INTEGER;
BEGIN
    (* GET VESA MODE *)
    r.AX := 4F03H;
    AsmLib.Intr ( r , 10H );
    mode := r.BX;

    (* NOT IN VESA MODE ? *)
    IF  ( r.AH <> 00H )
        THEN
            (* GET NORMAL MODE *)
            r.AX := 0F00H;
            AsmLib.Intr ( r , 10H );
            mode := INTEGER ( r.AL );
    END;
    RETURN mode;
END GetMode;




PROCEDURE SetWindow ( xmin , ymin , xmax , ymax : INTEGER );
BEGIN
    IF  ( xmin >= maxwin.xmin )
        AND
        ( xmax <= maxwin.xmax )
        AND
        ( xmin <= xmax )
        AND
        ( ymin >= maxwin.ymin )
        AND
        ( ymax <= maxwin.ymax )
        AND
        ( ymin <= ymax )
        THEN
            win.xmin := xmin;
            win.ymin := ymin;
            win.xmax := xmax;
            win.ymax := ymax;
            GotoXY( win.xmin, win.ymin );
    END;
END SetWindow;




PROCEDURE GetMaxWindow ( VAR xmin , ymin , xmax , ymax : INTEGER );
BEGIN
    xmin := maxwin.xmin;
    ymin := maxwin.ymin;
    xmax := maxwin.xmax;
    ymax := maxwin.ymax;
END GetMaxWindow;




PROCEDURE GetBytesPerPixel(  ) : INTEGER;
BEGIN
    RETURN ( ( 7+VESA.BitsPerPixel ) DIV 8 );
END  GetBytesPerPixel;




PROCEDURE GetLine( VAR save     : ARRAY OF SHORTCARD;
                   x ,y, length : INTEGER );
BEGIN
(*
    AsmLib.Fill( [SYSTEM.Seg(save):SYSTEM.Ofs(save)],
                 HIGH( save ) + 1,
                 0 );
*)
    IF  ( CARDINAL( length - 1 ) <= HIGH( save ) )
        AND
        ( length > 0 )
        THEN
            GetLinePROC( save, x, y, 0, length );
    END;
END GetLine;




PROCEDURE PutLine( VAR save     : ARRAY OF SHORTCARD;
                   x ,y, length : INTEGER );
BEGIN
    IF  ( CARDINAL( length - 1 ) <= HIGH( save ) )
        AND
        ( length > 0 )
        THEN
            PutLinePROC( save, x, y, 0, length );
    END;
END PutLine;




PROCEDURE Plot ( x , y : INTEGER );
BEGIN
    IF  ( x >= win.xmin )
        AND
        ( x <= win.xmax )
        AND
        ( y >= win.ymin )
        AND
        ( y <= win.ymax )
        THEN
            PlotPROC ( x , y );
    END;
END Plot;




PROCEDURE Point ( x , y : INTEGER;
                  VAR red, green, blue : CARDINAL );
BEGIN
    IF  ( x >= win.xmin )
        AND
        ( x <= win.xmax )
        AND
        ( y >= win.ymin )
        AND
        ( y <= win.ymax )
        THEN
            PointPROC ( x , y, red, green, blue );
        ELSE
            red   := 0;
            green := 0;
            blue  := 0;
    END;
END Point;




PROCEDURE Line ( x1 , y1 , x2 , y2  : INTEGER );
VAR
    tmp            : INTEGER;
    dx , dy , next : LONGINT;


    PROCEDURE CalcIntersections( VAR x1, y1, x2, y2                 : INTEGER;
                                 x1_axis, x2_axis, y1_axis, y2_axis : INTEGER;
                                 dx, dy                             : LONGINT;
                                 VAR next                           : LONGINT ) : BOOLEAN;
    (*
        TopSpeed Modula-2 V1.17 has a bug in the function
        INTEGER := TRUNC( LONGREAL );
        when LONGREAL is negative!
        So I use INTEGER instead, becauce this function does,
        what TRUNC should!
    *)
    VAR
        tmp1, tmp2                     : LONGREAL;
        old_x1, old_y1, old_x2, old_y2 : INTEGER;


        PROCEDURE DiffFirstAxisP1( axis, p1, p2 : INTEGER ) : LONGREAL;
        BEGIN
            IF  ( p1 >= p2 )
                THEN
                    RETURN ( LONGREAL( axis ) - LONGREAL( p1 ) + 0.5 );
                ELSE
                    RETURN ( LONGREAL( axis ) - LONGREAL( p1 ) - 0.5 );
            END;
        END DiffFirstAxisP1;


        PROCEDURE DiffSecondAxisP1( axis, p1, p2 : INTEGER ) : LONGREAL;
        BEGIN
            IF  ( p1 >= p2 )
                THEN
                    RETURN ( LONGREAL( axis ) - LONGREAL( p1 ) - 0.5 );
                ELSE
                    RETURN ( LONGREAL( axis ) - LONGREAL( p1 ) + 0.5 );
            END;
        END DiffSecondAxisP1;


        PROCEDURE RoundFirst( value : LONGREAL ) : LONGINT;
        BEGIN
            IF      ( value > 0.0 )
                    THEN
                        RETURN ( LONGINT( value ) + 1 );
            ELSIF   ( value < 0.0 )
                    THEN
                        RETURN ( LONGINT( value ) - 1 );
            ELSIF   ( value = 0.0 )
                    THEN
                        SetMode( 03H );
                        AsmLib.FatalError('NewTerm.RoundFirst: wrong input' );
            END;
        END RoundFirst;


        PROCEDURE RoundSecond( value : LONGREAL ) : LONGINT;
        BEGIN
            RETURN LONGINT( value );
        END RoundSecond;


        PROCEDURE RoundThird( value : LONGREAL ) : LONGINT;
        VAR
            tmp : LONGREAL;
        BEGIN
            tmp := value - LONGREAL( INTEGER( value ) );
            IF      ( tmp > 0.5 )
                    THEN
                        RETURN LONGINT( value + 0.5 );
            ELSIF   ( tmp < -0.5 )
                    THEN
                        RETURN LONGINT( value - 0.5 );
            END;
            RETURN LONGINT( value );
        END RoundThird;


    BEGIN
        old_x1 := x1;
        old_y1 := y1;
        old_x2 := x2;
        old_y2 := y2;

        IF      ( ( x1 < x1_axis ) AND ( x2 >= x1_axis ) )
                OR
                ( ( x1 > x1_axis ) AND ( x2 <= x1_axis ) )
                THEN
                     tmp1 := ( LONGREAL( x1_axis ) - LONGREAL( old_x1 ) )
                             *
                             ( LONGREAL( dy ) / LONGREAL( dx ) );
                     y1   := INTEGER( LONGINT( old_y1 ) + RoundThird( tmp1 ) );
                     x1   := x1_axis;
        END;

        IF       ( ( x2 > x2_axis ) AND ( x1 <= x2_axis ) )
                 OR
                 ( ( x2 < x2_axis ) AND ( x1 >= x2_axis ) )
                 THEN
                     tmp1 := ( LONGREAL( x2_axis ) - LONGREAL( old_x1 ) )
                             *
                             ( LONGREAL( dy ) / LONGREAL( dx ) );
                     y2   := INTEGER( LONGINT( old_y1 ) + RoundThird( tmp1 ) );
                     x2   := x2_axis;
        END;

        IF       ( ( y1 < y1_axis ) AND ( y2 >= y1_axis ) )
                 OR
                 ( ( y1 > y1_axis ) AND ( y2 <= y1_axis ) )
                 THEN
                     tmp1 := DiffFirstAxisP1( y1_axis, old_y1, old_y2 )
                             /
                             ( LONGREAL( dy ) / LONGREAL( dx ) );
                     x1   := INTEGER( LONGINT( old_x1 ) + RoundFirst( tmp1 ) );
                     y1   := y1_axis;
        END;

        IF       ( ( y2 > y2_axis ) AND ( y1 <= y2_axis ) )
                 OR
                 ( ( y2 < y2_axis ) AND ( y1 >= y2_axis ) )
                 THEN
                     tmp1 := DiffSecondAxisP1( y2_axis, old_y1, old_y2 )
                             /
                             ( LONGREAL( dy ) / LONGREAL( dx ) );
                     x2   := INTEGER( LONGINT( old_x1 ) + RoundSecond( tmp1 ) );
                     y2   := y2_axis;
        END;

        next := ABS( dx ) DIV 2;
        IF  ( x1 <> old_x1 )
            THEN
                tmp1 := ABS( LONGREAL(dy) * ( LONGREAL(x1) - LONGREAL(old_x1) ) );
                tmp2 := ABS(LONGREAL(dx))
                        * LONGREAL( LONGINT( ( LONGREAL(next) + tmp1 - 1.0 )
                                             /
                                             ABS(LONGREAL(dx)) ) );
                next := next + LONGINT( tmp2 - tmp1 );
        END;

        RETURN (
                 ( ( x1 >= x1_axis ) AND ( x1 <= x2_axis ) )
                 OR
                 ( ( x1 <= x1_axis ) AND ( x1 >= x2_axis ) )
               )
               AND
               (  ( ( y1 >= y1_axis ) AND ( y1 <= y2_axis ) )
                  OR
                  ( ( y1 <= y1_axis ) AND ( y1 >= y2_axis ) )
               );
    END CalcIntersections;


BEGIN
    IF  ( x1 > x2 )
        THEN
            tmp := x1;
            x1  := x2;
            x2  := tmp;
            tmp := y1;
            y1  := y2;
            y2  := tmp;
    END;
    dx := LONGINT( x2 ) - LONGINT( x1 );
    dx := dx + dx;
    dy := LONGINT( y2 ) - LONGINT( y1 );
    dy := dy + dy;

    IF      ( dy = 0 )
            THEN
                IF  CalcIntersections( x1, y1, x2, y2,
                                       win.xmin, win.xmax, win.ymin, win.ymax,
                                       dx, dy, next )
                    THEN
                        LineHorizontal( x1, y1, x2, y2 );
                END;
    ELSIF   ( dx >= ABS( dy ) )
            AND
            ( dy >= 0 )
            THEN
                IF  CalcIntersections( x1, y1, x2, y2,
                                       win.xmin, win.xmax, win.ymin, win.ymax,
                                       dx, dy, next )
                    THEN
                        LineLE45( x1, y1, x2, y2, dx, dy, next );
                END;
    ELSIF   ( dx >= ABS( dy ) )
            AND
            ( dy < 0 )
            THEN
                IF  CalcIntersections( x1, y1, x2, y2,
                                       win.xmin, win.xmax, win.ymax, win.ymin,
                                       dx, dy, next )
                    THEN
                        LineLE45( x1, y1, x2, y2, dx, dy, next );
                END;
    ELSIF   ( dy >= 0 )
            THEN
                IF  CalcIntersections( y1, x1, y2, x2,
                                       win.ymin, win.ymax, win.xmin, win.xmax,
                                       dy, dx, next )
                    THEN
                        LineGE45( x1, y1, x2, y2, dx, dy, next );
                END;
    ELSIF   ( dy < 0 )
            THEN
                IF  CalcIntersections( y1, x1, y2, x2,
                                       win.ymax, win.ymin, win.xmin, win.xmax,
                                       dy, dx, next )
                    THEN
                        LineGE45( x1, y1, x2, y2, dx, dy, next );
                END;
    END;
END Line;




PROCEDURE InitFont(  );
VAR
    FontPtr : POINTER TO ARRAY[0..255],[0..font_height-1]OF SHORTCARD;
    x, y    : CARDINAL;


    PROCEDURE InitFontPointer( VAR pointer : ARRAY OF WORD );
    VAR
        r : SYSTEM.Registers;
    BEGIN
        r.AX := 1130H;
        r.BH := 06H;
        AsmLib.Intr( r, 10H );
        pointer[1] := r.ES;
        pointer[0] := r.BP;
    END InitFontPointer;


BEGIN
    InitFontPointer( FontPtr );
    FOR x := 0 TO 255 DO
        FOR y := 0 TO font_height-1 DO
            FONT[x, y] := BITSET( CARDINAL( FontPtr^[x, y] ) << (16-font_width) );
        END;
    END;
END InitFont;




PROCEDURE SetFGColor ( red, green, blue : CARDINAL );
BEGIN
    SetFGColorPROC( red, green, blue );
END SetFGColor;




PROCEDURE SetBGColor ( red, green, blue : CARDINAL );
BEGIN
    SetBGColorPROC( red, green, blue );
END SetBGColor;




PROCEDURE GetXY( VAR x , y : INTEGER );
BEGIN
    GetXYPROC( x, y );
END GetXY;




PROCEDURE GotoXY( x, y : INTEGER );
BEGIN
    GotoXYPROC( x, y );
END GotoXY;




PROCEDURE Cursor( state : BOOLEAN );
BEGIN
    cursor_state := state;
END Cursor;




PROCEDURE CursorState(  ) : BOOLEAN;
BEGIN
    RETURN cursor_state;
END CursorState;




PROCEDURE Scroll( times : INTEGER );
BEGIN
    IF  ( ABS( times ) > ( win.ymax - win.ymin ) )
        THEN
            times := 0;
    END;
    ScrollPROC( win.xmin, win.ymin, win.xmax, win.ymax, times );
END Scroll;




PROCEDURE PutBitmap( bitmap        : ARRAY OF BITSET;
                     length        : INTEGER;
                     x, y          : INTEGER );
VAR
    start : INTEGER;
BEGIN
    IF  ( (x+length) >= win.xmin )
        AND
        ( y >= win.ymin )
        AND
        ( x <= win.xmax )
        AND
        ( y <= win.ymax )
        THEN
            IF  ( (x+length) > win.xmax)
                THEN
                    length := 1+win.xmax-x;
            END;
            IF  ( x >= win.xmin )
                THEN
                    start := 0;
                ELSE
                    start  := win.xmin - x;
                    x      := win.xmin;
            END;
            PutBitmapPROC( bitmap,
                           start, length,
                           x, y );
    END;
END PutBitmap;




PROCEDURE Write( ch : CHAR );
BEGIN
    WritePROC( ch );
END Write;




PROCEDURE WriteString( str : ARRAY OF CHAR );
VAR
    pos : INTEGER;
BEGIN
    pos := 0;
    WHILE
        ( pos <= INTEGER( HIGH( str ) ) )
        AND
        ( str[pos] <> CHR( 0 ) )
    DO
        Write( str[pos] );
        INC ( pos );
    END;
END WriteString;




PROCEDURE Read( VAR ch : CHAR );
VAR
    r : SYSTEM.Registers;
BEGIN
    r.AH := 07H;
    AsmLib.Intr( r, 21H );
    ch := CHAR( r.AL );
END Read;




PROCEDURE Scan( VAR ch      : CHAR;
                VAR aviable : BOOLEAN );
VAR
    r : SYSTEM.Registers;
BEGIN
    r.AH := 06H;
    r.DL := 0FFH;
    AsmLib.Intr( r, 21H );
    ch := CHAR( r.AL );
    IF  ( SYSTEM.ZeroFlag IN r.Flags )
        THEN
            aviable := FALSE;
        ELSE
            aviable := TRUE;
    END;
END Scan;




PROCEDURE ClearKeyBuffer(  );
VAR
    r : SYSTEM.Registers;
BEGIN
    r.AX :=0C00H;
    AsmLib.Intr( r, 21H );
END ClearKeyBuffer;




PROCEDURE ReadString( VAR str   : ARRAY OF CHAR;
                      VAR term  : CHAR;
                      boxlength : INTEGER );
VAR
    length : CARDINAL;
    ch     : CHAR;


    PROCEDURE Length ( VAR str : ARRAY OF CHAR ) : CARDINAL;
    VAR
        length : CARDINAL;
    BEGIN
        length := 0;
        WHILE
            ( length <= HIGH ( str ) )
            AND
            ( str[length] <> CHR( 0 ) )
        DO
            INC ( length );
        END;
        RETURN length;
    END Length;


    PROCEDURE Insert( VAR str : ARRAY OF CHAR;
                      VAR pos : CARDINAL;
                      ch      : CHAR );
    VAR
        length : CARDINAL;
    BEGIN
        length := Length( str );
        IF      ( HIGH( str ) > length )
                THEN
                    str[pos] := ch;
                    str[pos+1] := CHR( 0 );
        ELSIF   ( pos = 0 )
                AND
                ( HIGH( str ) = 0 )
                THEN
                    str[0] := ch;
        END;

        IF      ( HIGH( str ) > length )
                THEN
                    Write( ch );
                    INC( pos );
        ELSIF   ( pos = 0 )
                AND
                ( HIGH( str ) = 0 )
                THEN
                    Write( ch );
                    INC( pos );
        END;
    END Insert;


    PROCEDURE Delete( VAR str : ARRAY OF CHAR;
                      VAR pos : CARDINAL);
    BEGIN
        IF  ( pos > 0 )
            THEN
                str[pos-1] := CHR( 0 );
        END;
        IF  ( pos > 0 )
            THEN
                DEC( pos );
                Write( CHR( 8 ) );
                Write( CHR( 32 ) );
                Write( CHR( 8 ) );
        END;
    END Delete;


BEGIN
    length := Length( str );
    IF  ( HIGH( str ) < length )
        AND
        ( HIGH( str ) > 0 )
        THEN
            str[HIGH( str )] := CHR( 0 );
            length := HIGH( str );
    END;
    WriteString( str );

    REPEAT
        Read( ch );
        IF      ( ORD( ch ) >= 32 )
                THEN
                    Insert( str, length, ch );
        ELSIF   ( ORD( ch ) = 8 )
                THEN
                    Delete( str, length );
        END;
    UNTIL
        ( ch = CHR( 13 ) )
        OR
        ( ch = CHR( 9 ) )
        OR
        ( ch = CHR( 27 ) );

    IF  ( ch = CHR( 27 ) )
        THEN
            WHILE
                ( length > 0 )
            DO
                Write( CHR( 8 ) );
                Write( CHR( 32 ) );
                Write( CHR( 8 ) );
                DEC( length );
                str[0] := CHR( 0 );
          END;
    END;
    term := ch;
END ReadString;




PROCEDURE GetVesaInfo ( VAR Info : InfoType ) : BOOLEAN;
TYPE
    VesaInfoType = RECORD
        ModeAttributes        : WORD;
        WindowAAttributes     : BYTE;
        WindowBAttributes     : BYTE;
        WindowGranularityKB   : WORD;
        WindowSizeKB          : WORD;
        StartSegmentWindowA   : WORD;
        StartSegmentWindowB   : WORD;
        PROCADDRSetWindow     : ARRAY[0..1]OF WORD;
        BytesPerScanLine      : WORD;

        PixelWidth            : WORD;
        PixelHeight           : WORD;
        WidthOfCharacter      : BYTE;
        HeightOfCharacter     : BYTE;
        NumberOfMemoryPlanes  : BYTE;
        BitsPerPixel          : BYTE;
        NumberOfBanks         : BYTE;
        MemoryModel           : BYTE;
        SizeOfBankKB          : BYTE;
        NumberOfImagePages    : BYTE;
        Reserved1             : BYTE;

        RedMaskSize           : BYTE;
        RedFieldPosition      : BYTE;
        GreenMaskSize         : BYTE;
        GreenFieldPosition    : BYTE;
        BlueMaskSize          : BYTE;
        BlueFieldPosition     : BYTE;
        ReservedMaskSize      : BYTE;
        ReservedFieldPosition : BYTE;
        DirectColorModeInfo   : BYTE;
        Reserved2             : ARRAY[0..215]OF BYTE;
    END;

VAR
    r    : SYSTEM.Registers;
    VESA : VesaInfoType;
BEGIN
    r.AX := 4F01H;
    r.CX := GetMode(  );
    r.ES := SYSTEM.Seg ( VESA );
    r.DI := SYSTEM.Ofs ( VESA );
    AsmLib.Intr ( r , 10H );

    IF  ( r.AX = 004FH )
        AND
        ( 0 IN BITSET( VESA.ModeAttributes ) )
        AND
        ( 0 = ( INTEGER( VESA.WindowSizeKB )
                MOD
                INTEGER( VESA.WindowGranularityKB ) )
        )
        THEN
            Info.Width         := INTEGER( VESA.PixelWidth );
            Info.Height        := INTEGER( VESA.PixelHeight );

            Info.LineLength    := LONGINT( VESA.BytesPerScanLine );
            Info.WinSize       := 1024 * LONGINT( VESA.WindowSizeKB );
            Info.NextBank      := INTEGER( VESA.WindowSizeKB )
                                  DIV
                                  INTEGER( VESA.WindowGranularityKB );
            Info.BitsPerPixel  := INTEGER( VESA.BitsPerPixel );

            Info.RedSize       := INTEGER( VESA.RedMaskSize );
            Info.RedPos        := INTEGER( VESA.RedFieldPosition );
            Info.GreenSize     := INTEGER( VESA.GreenMaskSize );
            Info.GreenPos      := INTEGER( VESA.GreenFieldPosition );
            Info.BlueSize      := INTEGER( VESA.BlueMaskSize );
            Info.BluePos       := INTEGER( VESA.BlueFieldPosition );
            RETURN TRUE;
     ELSE
            RETURN FALSE;
    END;
END GetVesaInfo;




PROCEDURE Init256(  ) : BOOLEAN;
VAR
    ok : BOOLEAN;


    PROCEDURE InitPalette(  );
    VAR
        r                : SYSTEM.Registers;
        red, green, blue : SHORTCARD;
        palette          : ARRAY[0..255],[0..2]OF SHORTCARD;
        index            : INTEGER;
    BEGIN
        index := 0;
        FOR index := 0 TO 255 DO
            red   := SHORTCARD( BITSET( 224 ) * BITSET( index ) );
            green := SHORTCARD( BITSET(  28 ) * BITSET( index ) );
            blue  := SHORTCARD( BITSET(   3 ) * BITSET( index ) );
            palette[index,0] := red DIV 4 + 7*SHORTCARD( red <> 0 );
            palette[index,1] := green * 2 + 7*SHORTCARD( green <> 0 );
            palette[index,2] := blue * 16 + 15*SHORTCARD( blue <> 0 );;
        END;
        r.AX := 1012H;
        r.BX := 0000H;
        r.CX := 256;
        r.ES := SYSTEM.Seg( palette );
        r.DX := SYSTEM.Ofs( palette );
        AsmLib.Intr( r, 10H );
    END InitPalette;


    PROCEDURE Init(  );
    BEGIN
        SetFGColorPROC := C256.SetFGColor;
        SetBGColorPROC := C256.SetBGColor;
        WritePROC      := GRAPH.Write;
        GotoXYPROC     := GRAPH.GotoXY;
        GetXYPROC      := GRAPH.GetXY;
        PutFontPROC    := C256.PutFont;
        PutBitmapPROC  := C256.PutBitmap;
        ScrollPROC     := GRAPH.Scroll;

        GetLinePROC    := C256.GetLine;
        PutLinePROC    := C256.PutLine;

        PlotPROC       := C256.Plot;
        PointPROC      := C256.Point;
        LineLE45       := C256.LineLE45;
        LineGE45       := C256.LineGE45;
        LineHorizontal := C256.LineHorizontal;

        maxwin.xmin    := 0;
        maxwin.xmax    := VESA.Width - 1;
        maxwin.ymin    := 0;
        maxwin.ymax    := VESA.Height - 1;
        SetWindow ( maxwin.xmin , maxwin.ymin , maxwin.xmax , maxwin.ymax );
        SetFGColor( 65535, 65535, 65535 );
        SetBGColor( 0, 0, 0 );
    END Init;


BEGIN
    InitPalette(  );
    ok := GetVesaInfo( VESA );
    Init(  );
(*
    IO.WrLngInt( VESA.WinSize, 0 );
*)
    RETURN  ( ok )
END Init256;




PROCEDURE InitTEXT(  ) : BOOLEAN;
BEGIN
    SetFGColorPROC := TEXT.SetFGColor;
    SetBGColorPROC := TEXT.SetBGColor;
    WritePROC      := TEXT.Write;
    GotoXYPROC     := TEXT.GotoXY;
    GetXYPROC      := TEXT.GetXY;
    PutFontPROC    := C256.PutFont;
    PutBitmapPROC  := C256.PutBitmap;
    ScrollPROC     := TEXT.Scroll;

    GetLinePROC    := C256.GetLine;
    PutLinePROC    := C256.PutLine;
    PlotPROC       := C256.Plot;
    PointPROC      := C256.Point;
    LineLE45       := C256.LineLE45;
    LineGE45       := C256.LineGE45;
    LineHorizontal := C256.LineHorizontal;
    maxwin.xmin    := 0;
    maxwin.xmax    := 79 (* columns *);
    maxwin.ymin    := 0;
    maxwin.ymax    := 24 (*INTEGER( rows )*);
    SetWindow ( maxwin.xmin , maxwin.ymin , maxwin.xmax , maxwin.ymax );
    SetFGColor( 14, 0, 0 );
    SetBGColor(  1, 0, 0 );
    RETURN TRUE;
END InitTEXT;

PROCEDURE SetMaxWindow();
VAR xmax, ymax, xmin, ymin : INTEGER;
BEGIN
  GetMaxWindow(xmin,ymin,xmax,ymax);
  SetWindow(xmin,ymin,xmax,ymax);
END SetMaxWindow;

BEGIN
    xormode         := FALSE;
    InitFont( (* Font *) );
END NewTerm.