{**************************************************}
{                                                  }
{   Fractal Master Program                         }
{   Copyright (c) 1995 by Michael Wessel, MW-Soft  }
{                                                  }
{**************************************************}

{$R-}

program Fractal;

uses WinDos, Strings, Objects, WinTypes, WinProcs, OWindows, ODialogs,
     OStdDlgs, OPrinter, OMemory, CommDLG, BWCC;

{$R fractal.res}
{$I fractal.inc}

const
  TIM_Animate = 4711;
  TIM_ChangeCursor = 4712;
  zoomfacmin = 70;
  zoomfacmax = 300;
  id_Scroller = 500;
  id_ZoomFactor = 501;
  pi180 = pi / 180.0;

type

     TPosEditorStates =
  (SetPatternPoints, MeasurePattern,
   BoundFractal, ChangePatternPoints, FractalDraw);


   TOptMen = record
                isinvisible : BOOLEAN;
                isexpand : boolean;


                drawgrid : BOOLEAN;
                drawcross : boolean;
                drawline : boolean;

                showpoints : boolean;

             end;

  TEdState = record
                mode : TPosEditorStates;
                moveactmenu, moveactmouse : boolean;
                startpointset, endpointset : boolean;
                firstclick : boolean;
                selected : integer;
             end;




  TPalTable = Array[0..0] of TPaletteEntry;
  PLogPalette = ^TLogPalette;
  TLongArray = array[0..15] of longint;

const AppName = 'Fractal Master';
      CustColors: TLongArray =
        ($00000000, $10000000, $20000000, $30000000,
         $40000000, $50000000, $60000000, $70000000,
         $80000000, $90000000, $a0000000, $b0000000,
         $c0000000, $d0000000, $e0000000, $f0000000);

         vecstartcol : longint = $01000000;

type

  POptionDialog = ^TOptionDialog;
  TOptionDialog = object(TDialog)
    procedure ok(var message : TMessage); virtual id_first + id_ok;
    procedure SetupWindow; virtual;
  end;

  PPenWidthDialog = ^TPenWidthDialog;
  TPenWidthDialog = object(TDialog)
    procedure ok(var message : TMessage); virtual id_first + id_ok;
    procedure SetupWindow; virtual;
  end;

  PBackgroundDialog = ^TBackgroundDialog;
  TBackgroundDialog = object(TDialog)
    procedure ok(var message : TMessage); virtual id_first + id_ok;
    procedure SetupWindow; virtual;
  end;

  PScrollDialog = ^TScrollDialog;
  TScrollDialog = object(TDialog)
    savedfactor : real;
    savedmousex, savedmousey : integer;
    procedure cancel(var message : TMessage); virtual id_first + id_cancel;
    procedure Scroller(var Message: TMessage); virtual id_First +
                                                       id_Scroller;
    procedure SetupWindow; virtual;
  end;

  PFracWindow = ^TFracWindow;


  (* -------------------------------------- *)

  TWinObject = object(TObject)
    private
    xorout : boolean;
    OwnerWin : PFracWindow;
    curpoint : TPoint;
    lastpoint : TPoint;
    public
    constructor Init(MyOwner : PFracWindow);
    procedure DontErasePrevious; virtual;
    procedure Erase; virtual;
    procedure Draw; virtual;
    procedure DrawAt(x, y : INTEGER); virtual;
    procedure SetAt(x, y : integer); virtual;
  end;

  TWinCross = object(TWinObject)
      private

      drawcross : boolean;
      drawline : BOOLEAN;
      elinestart : TPoint;
      curtype : (crossmode, boxmode);
      boxsize : integer;

      public
      constructor Init(MyOwner : PFracWindow);
      procedure Draw; virtual;
      procedure DrawAt(x, y : integer); virtual;
      procedure SetEditlineStart(x, y : integer); virtual;
      procedure AllowDrawEditline; virtual;
      procedure DontDrawEditline; virtual;
      procedure AllowDraw; virtual;
      procedure DontDraw; virtual;
      function GetDrawMode:boolean; virtual;
      procedure GetEditlineStart(var x, y : integer);
      procedure SetCrossMode;
      procedure SetBoxMode;
      procedure SetBoxSize(size : integer);
  end;

  TWinRect = object(TWinObject)
    private
    curpoint1  : TPoint;
    lastpoint1 : TPoint;
    public
    constructor Init(MyOwner : PFracWindow);
    procedure DrawRectAt(xf, yf, xt, yt : integer); virtual;
    procedure SetRectAt(xf, yf, xt, yt : integer); virtual;
    procedure Draw; virtual;
  end;

  TMoveRect = object(TWinRect)
    private
    savedlastpoint : TPoint;
    savedlastadded  : TPoint;
    savednow : TPoint;
    savedcurpoint, savedcurpoint1 : TPoint;
    savedoffset : TPoint;
    public
    constructor Init(MyOwner : PFracWindow);
    procedure InitMove;
    procedure Update;
  end;

  TBoundRect = object(TWinRect)
    public
    constructor Init(MyOwner : PFracWindow);
    procedure GetDist(var x, y : integer); virtual;
    procedure SetDist(x, y : integer); virtual;
  end;

  (* ----------------------------------- *)


   TWinFractal = object(TObject)

      private

      OwnerWin : PFracWindow;
      depth : integer;
      pturtlemem : ARRAY[0..1] OF PCollection;
      switch : integer;

      fxmin, fymin, fxmax, fymax : real;
      xfac, yfac : REAL;
      offsx, offsy : real;

      sumang : real;

      public
      constructor Init(MyOwner: PFracWindow);

      procedure CleanUp; (* KEIN Dynamisches Objekt, rumt nur auf! *)

      procedure TurtleCalc;
      procedure Draw;

      private
      procedure Scale;
      procedure Turtle(actside : integer);
      procedure PreTurtle;
    end;

  (* -------------------------------------- *)

  PPatternItem = ^TPatternItem;
  TPatternItem = object(TObject)

    ipos : TPoint;
    icolor : TColorRef;
    iwidth : integer;
    irelangle : REAL;
    idrawflag : BOOLEAN;
    iexpandflag : boolean;

    ilength : REAL;
    constructor Init(MyOwner : PFracWindow);
    destructor Done; virtual; (* dynamisches Objekt ! *)
  end;


  TWinPattern = object(TObject)
     private

     OwnerWin : PFracWindow;
     ppoints : PCollection;

     nopoints : BOOLEAN;
     showallpoints : boolean;

     startpoint, endpoint : TPatternItem;
     offset : TPoint;

     MoveBox : TMoveRect;

     public
     constructor Init(MyOwner : PFracWindow);
     procedure CleanUp;
     procedure InsertPoint;
     procedure DeletePoint;
     procedure Draw;
     procedure TransformForTurtle;
     procedure AllignToGrid;
     procedure SetOffset(x, y : integer);
     procedure ChangePointTo(index, newx, newy : integer);
     procedure SetStartpoint;
     procedure SetEndpoint;
     procedure ShowPoints;
     procedure DontShowPoints;
     function IsEmpty:Boolean;
     function FindPoint(x, y, delta : integer):integer;
     function IsEndpoint(x, y, delta : integer):boolean;
     function IsStartpoint(x, y, delta : integer):boolean;
     procedure CalculateBoundingRect(var xf, yf, xt, yt : integer);
     function GetNumberOfPoints:INTEGER;
     function GetLastItem:PPatternItem;
  end;

  (* -------------------------------------- *)

  TWinGrid = object(TObject)
    private
    OwnerWin : PFracWindow;
    gridstep : integer;

    public
    constructor Init(MyOwner : PFracWindow);
    procedure AdjustGridstep(newgridstep : integer);
    procedure Draw;
  end;

  (* -------------------------------------- *)

  PTurtleItem = ^TTurtleItem;
  TTurtleItem = object(TObject)
    private
    relangle : REAL;
    length : REAL;
    drawflag : boolean;
    expandflag : boolean;
    patind : integer;
    public
    constructor Init(a, l : REAL; draw, expand : boolean; index : integer);
    destructor Done; virtual; (* dynamische Objekt *)
  end;

 (* -------------------------------------- *)

  TFracWindow = object(TWindow)

    printer : PPrinter;
    frac : TWinFractal;
    pattern : TWinPattern;

    options : record

                sides : integer;
                alpha, beta : integer;
                treemode, usealpha, usebeta : boolean;

                savedmenopt, menopt : TOptMen;

                colorref : TColorRef;
                brush, width : integer;

                zoomfactor : real;
                gridstep : integer;


             end;


    editor : record

                savededstate, edstate : TEdState;

                Grid : TWinGrid;
                Cross : TWinCross;

                last :  TPoint;
                lastadded : TPoint;
                now : TPoint;

          end;

    screen : record

               curmoverect : HCursor;
               cursetpoints : HCursor;

               blinkflag : boolean;
               WinPalette:HPalette;

               WinWidth, WinHeight : integer;
               WinSize : TRect;
               WinDC : HDC;
               MemDC : HDC;
               Picture : HBitmap;
               corfac : real;

               usebitmap : boolean;

               BoundBox : TBoundRect;

               flag : boolean;
               MustBeDeleted : boolean;

             end;

    (* Methoden *)

    constructor Init(AParent: PWindowsObject; ATitle: PChar);

    destructor Done; virtual;

    procedure SetupWindow; virtual;
    procedure GetWindowClass(var WndClass : TWndClass); virtual;

    function CanClose:boolean; virtual;

    procedure CMCloseWin(var Message : TMessage);
      virtual cm_First + cm_ExitWin;

    (* Men-Methoden *)

    procedure CMAbout(VAR Msg: TMessage);
      virtual cm_First + cm_About;

    procedure CMPrint(VAR msg: Tmessage);
      virtual cm_First + cm_Print;

    procedure CMPrintSetup(VAR msg: TMessage);
      virtual cm_First + cm_PrintSetup;

    procedure CMPenWidth(var msg : TMessage);
      virtual cm_First + cm_PenWidth;

    procedure CMGrid(VAR msg : TMessage);
      virtual cm_First + cm_Grid;

    procedure CMZoomFactor(var msg : TMessage);
      virtual cm_First + cm_ZoomFactor;

    procedure CMInvisible(VAR msg : TMessage);
      virtual cm_First + cm_invisible;

    procedure CMCross(VAR msg : TMessage);
      virtual cm_First + cm_Cross;

    procedure CMAllignToGrid(var msg : TMessage);
      virtual cm_First + cm_AllignToGrid;

    procedure CMNext(VAR msg: Tmessage);
      virtual cm_First + cm_Next;

    procedure CMBoundaries(VAR msg : TMessage);
       virtual cm_First + cm_Boundaries;

    procedure CMChangePoints(var msg : TMessage);
      virtual cm_First + cm_ChangePoints;

    procedure CMBackground(VAR msg : TMessage);
        virtual cm_First + cm_Background;

    procedure CMReset(VAR msg: TMessage);
      virtual cm_First + cm_Reset;

    procedure CMNew(var msg : TMEssage);
      virtual cm_First + cm_New;

    procedure CMOptions(var msg : TMessage);
      virtual cm_First + cm_Options;

    procedure CMExpand(var msg : TMessage);
      virtual cm_First + cm_Expand;

    procedure CMMeasure(var msg : TMessage);
      virtual cm_First + cm_Measure;

    procedure CMChooseColor(var msg : TMessage);
      virtual cm_First + cm_ChooseColor;

    procedure CMDragMove(var msg : TMessage);
      virtual cm_First + cm_DragMove;

    procedure CMShowPoints(var msg : TMessage);
      virtual cm_First + cm_ShowPoints;

    procedure CMDrawEditline(var msg : TMessage);
      virtual cm_First + cm_DrawEditline;

    (* Mouse-Methoden, Paint-Methode *)

    procedure WMClose(var Message : TMessage);
      virtual wm_First + wm_Close;

    procedure WMLButtonUp(vAR Msg: TMessage);
      virtual wm_First + wm_LButtonUp;

    procedure WMLButtonDown(vAR Msg: TMessage);
      virtual wm_First + wm_LButtonDown;

    procedure WMRButtonDown(VAR Msg: TMessage);
      virtual wm_First + wm_RButtonUp;

    procedure WMMouseMove(var Msg: TMessage);
      virtual wm_First + wm_MouseMove;

    procedure Paint(PaintDC: HDC; var PaintInfo: TPaintStruct); virtual;

    procedure WMSize(var Msg: TMessage); virtual
       wm_First + wm_Size;

    procedure FillPalette(var PalTable:TPalTable; index : byte;
                                      red, green, blue : integer);

    procedure WMTimer(var Msg:TMessage); virtual wm_First+wm_Timer;

    (* andere Methoden, intern! *)

    procedure ClearWindow;
    procedure Init2;
    procedure SetMenuItem(itemid : integer; on, checked : boolean);
    procedure SetupMenus;
    procedure DeleteObjects;
    procedure Restore;
    procedure Transform(var x, y : integer);
    procedure InvTransform(var x,y : integer);
    procedure GridTransform(var x, y : integer);
    procedure InvGridTransform(var x, y : integer);

    function CalcLength(source, destination : TPoint):real;
    procedure CopyLastitemIntoEditor;

  end;

  TMyApplication = object(TApplication)
    procedure InitMainWindow; virtual;
  end;



(* ----------------------------------------------- *)

constructor TWinObject.Init(MyOwner : PFracWindow);
begin
  OwnerWin:=MyOwner;
  xorout:=false;
  curpoint.x:=0;
  curpoint.y:=0;
  lastpoint.x:=0;
  lastpoint.y:=0;
end;

procedure TWinObject.DontErasePrevious;
begin
  xorout:=false;
end;

procedure TWinObject.Erase;
begin
  xorout:=false;
  self.Draw;
end;

procedure TWinObject.Draw;
begin
  RunError(211);
end;

procedure TWinObject.SetAt(x,y : integer);
begin
  lastpoint:=curpoint;
  curpoint.x:=x;
  curpoint.y:=y;
  xorout:=true;
end;

procedure TWinObject.DrawAt(x, y : integer);
begin
  SetAt(x,y);
  self.Draw;
end;

(* Cross-Methoden *)

constructor TWinCross.Init(MyOwner : PFracWindow);
begin
  inherited Init(MyOwner);
  drawline:=true;
  elinestart.x:=0;
  elinestart.y:=0;
  drawcross:=true;
  curtype:=crossmode;
  boxsize:=10;
end;

procedure TWinCross.SetCrossMode;
begin
  curtype:=crossmode;
end;

procedure TWinCross.SetBoxSize(size : integer);
begin
  boxsize:=size;
end;

procedure TWinCross.SetBoxMode;
begin
  curtype:=boxmode;
end;

procedure TWinCross.AllowDrawEditline;
begin
  drawline:=true;
end;

procedure TWinCross.DontDrawEditline;
begin
  drawline:=false;
end;

procedure TWinCross.AllowDraw;
begin
  drawcross:=true;
end;

procedure TWinCross.DontDraw;
begin
  drawcross:=false;
end;

function TWinCross.GetDrawMode:boolean;
begin
  GetDrawMode:=drawcross;
end;

procedure TWinCross.SetEditlineStart(x, y : integer);
begin
  elinestart.x:=x;
  elinestart.y:=y;
end;

procedure TWinCross.GetEditlineStart(var x, y : integer);
begin
  x:=elinestart.x;
  y:=elinestart.y;
end;

procedure TWinCross.Draw;

  procedure Cross(point : TPoint);
  begin
    with OwnerWin^.screen do begin
      SetROP2(WinDC,r2_NOTXORPEN);
      MoveTo(WinDC,point.x,WinSize.top);
      LineTO(WinDC,point.x,WinSize.bottom);
      MoveTo(WinDC,WinSize.left,point.y);
      LineTo(WinDC,WinSize.right,point.y);
    end;
  end;

  procedure Box(point : TPoint);
  begin
    with OwnerWin^.screen do begin
      SetROP2(WinDC,r2_NOTXORPEN);
      MoveTo(WinDC,point.x,point.y-boxsize);
      LineTO(WinDC,point.x+boxsize,point.y);
      LineTo(WinDC,point.x,point.y+boxsize);
      LineTo(WinDC,point.x-boxsize,point.y);
      LineTo(WinDC,point.x,point.y-boxsize);
    end;
  end;

  procedure Line(point : TPoint);
  begin
    with OwnerWin^.screen do begin
       SetROP2(WinDC,r2_NOTXORPEN);
       MoveTo(WinDC,elinestart.x,elinestart.y);
       LineTo(WinDC,point.x,point.y);
    end;
  end;

begin
  (* with OwnerWin^.screen do with OwnerWin^ do begin *)

    if drawcross then begin
      if xorout then
        if curtype=crossmode then Cross(lastpoint) else Box(lastpoint);

      if curtype=crossmode then Cross(curpoint) else Box(curpoint);
    end;

    if drawline and not(OwnerWin^.Pattern.IsEmpty) then begin
      if xorout then Line(lastpoint);
      Line(curpoint);

    end;

    xorout:=true;

(*  end; *)
end;

procedure TWinCross.DrawAt(x, y : integer);
begin
  inherited SetAt(x,y);
  Draw;
end;


(* TWin-Rect-Methoden *)

constructor TWinRect.Init(MyOwner : PFracWindow);
begin
  inherited Init(MyOwner);
  curpoint1.x:=0;
  curpoint1.y:=0;
  lastpoint1.x:=0;
  lastpoint1.y:=0;
end;

procedure TWinRect.Draw;
begin
  with OwnerWin^.screen do begin

   SetROP2(WinDC,r2_NOTXORPEN);

    if xorout then Rectangle(WinDC,lastpoint.x,lastpoint.y,lastpoint1.x,lastpoint1.y);

    Rectangle(WinDC,curpoint.x,curpoint.y,curpoint1.x,curpoint1.y);

  end;
end;

procedure TWinRect.DrawRectAt(xf, yf, xt, yt : integer);
begin
  SetRectAt(xf,yf,xt,yt);
  Draw;
end;

procedure TWinRect.SetRectAt(xf, yf, xt, yt : integer);
begin
  inherited SetAt(xf,yf);
  lastpoint1:=curpoint1;
  curpoint1.x:=xt;
  curpoint1.y:=yt;
end;

(* TMove-Rect-Methoden *)

constructor TMoveRect.Init(MyOwner : PFracWindow);
begin
  inherited Init(MyOwner);
  savedlastpoint.x:=0;
  savedlastpoint.y:=0;
  savedlastadded.x:=0;
  savedlastadded.y:=0;
end;

procedure TMoveRect.Update;
var dxi, dyi, dx, dy : integer;

begin
  with OwnerWin^.editor do begin

    dxi:=now.x-savednow.x;
    dyi:=now.y-savednow.y;

    dx:=round(dxi/OwnerWin^.options.zoomfactor);
    dy:=round(dyi/OwnerWin^.options.zoomfactor);

    SetRectAt(savedcurpoint.x+dxi,savedcurpoint.y+dyi,
              savedcurpoint1.x+dxi,savedcurpoint1.y+dyi);

    OwnerWin^.Pattern.SetOffset(savedoffset.x+dx,
                                savedoffset.y+dy);

    OwnerWin^.CopyLastItemIntoEditor;
    OwnerWin^.Editor.Cross.SetAt(now.x,now.y);
  end;
end;

procedure TMoveRect.InitMove;
begin
  with OwnerWin^ do with OwnerWin^.editor do with OwnerWin^.screen do begin

    SetMenuItem(cm_DragMove,false,true);
    SetWindowText(hWindow,'Move the pattern!');

    SetCursor(curmoverect);

    screen.WinDC:=GetDC(hWindow);

    OwnerWin^.Pattern.CalculateBoundingRect(curpoint.x, curpoint.y,
                                            curpoint1.x,curpoint1.y);


    savedcurpoint.x:=curpoint.x;
    savedcurpoint.y:=curpoint.y;
    savedcurpoint1.x:=curpoint1.x;
    savedcurpoint1.y:=curpoint1.y;

    savedoffset:=pattern.offset;

    savednow.x:=now.x;
    savednow.y:=now.y;

    DontErasePrevious;
    Draw;

    ReleaseDC(hWindow,screen.WinDC);

    SetupMenus;

  end;
end;

(* TBound-Rect-Methoden *)


procedure TBoundRect.GetDist(var x, y : integer);
begin
  x:=curpoint.x;
  y:=curpoint.y;
end;

procedure TBoundRect.SetDist(x, y : integer);
var winrect : TRect;
    dx, dy : integer;
begin
  GetClientRect(OwnerWin^.hwindow,winrect);
  with winrect do begin
    dx:=(right-left) div 2;
    dy:=(bottom-top) div 2;
    if x>dx then dx:=x;
    if y>dy then dy:=y;
    SetRectAt(x,y,right-x,bottom-y);
  end;
end;

constructor TBoundRect.Init(MyOwner : PFracWindow);
begin
  inherited Init(MyOwner);
  SetRectAt(60,60,300,300);
end;

(* ---------------------------------- *)

(* TTurtleItem-Methoden *)

constructor TTurtleItem.Init(a, l : REAL; draw, expand : boolean; index : integer);
begin
  relangle:=a;
  length:=l;
  drawflag:=draw;
  expandflag:=expand;
  patind:=index;
end;

destructor TTurtleItem.Done;
begin
end;

(* ---------------------------------- *)

constructor TWinGrid.Init(MyOwner : PFracWindow);
begin
  OwnerWin:=MyOwner;
end;

procedure TWinGrid.AdjustGridstep(newgridstep : integer);
begin
  gridstep:=newgridstep;
end;

procedure TWinGrid.Draw;
var x, y : integer;
    zf : real;
begin
  with OwnerWin^ do with OwnerWin^.screen do begin
    if options.menopt.drawgrid then begin

      SetROP2(WinDC,r2_NOTXORPEN);
      x:=WinSize.left;
      zf:=options.zoomfactor;

      while (x*zf)<=WinSize.right do begin
        y:=WinSize.top;
        while (y*zf)<=WinSize.bottom do begin
          SetPixel(WinDC,round(x*zf),round(y*zf),RGB(128,0,0));
          y:=y+(options.gridstep);
        end;
        x:=x+(options.gridstep);
      end;
    end;
  end;
end;


(* --------------------------------- *)

constructor TWinFractal.Init(MyOwner : PFracWindow);
begin
  new(pturtlemem[0],Init(10,10));
  new(pturtlemem[1],Init(10,10));
  switch:=0;
  depth:=0;
  xfac:=1.0;
  yfac:=1.0;
  fxmax:=0;
  fxmin:=0;
  fymax:=0;
  fymin:=0;
  offsx:=0;
  offsy:=0;
  OwnerWin:=MyOwner;
end;

procedure TWinFractal.CleanUp;
begin
   Dispose(pturtlemem[0],Done);
   Dispose(pturtlemem[1],Done);
end;


procedure TWinFractal.Scale;
var dx2,dy2,min : real;
    x, y : integer;

begin

  with OwnerWin^.screen do begin

    dx2:=fxmax-fxmin;
    dy2:=fymax-fymin;
    BoundBox.GetDist(x,y);

    if (dx2<>0) and (dy2<>0) then begin
      xfac:=(WinWidth-2*x)  / dx2;
      yfac:=(WinHeight-2*y) / dy2;
    end
    else begin
      xfac:=1.0;
      yfac:=1.0;
    end;

    offsx:=-round(fxmin*xfac)+x;
    offsy:=-round(fymin*yfac)+y;

  end;
end;

procedure TWinFractal.Draw;
var i : integer;
    step : real;
    str : array[0..40] of char;
    dc : hdc;

begin

  with OwnerWin^ do with OwnerWin^.screen do begin

    sumang:=0.0;
    offsx:=0;
    offsy:=0;
    fxmin:=0;
    fxmax:=0;
    fymin:=0;
    fymax:=0;

    SetWindowText(hWindow,'Painting - please wait!');

    for i:=1 to options.sides do begin
      PreTurtle;
      if options.usealpha then
        step:=options.alpha
      else
        if i=1 then step:=360/options.sides-frac.sumang;
      frac.sumang:=frac.sumang+step;
    end;

    Scale;

    WinDC:=GetDC(hWindow);

    if usebitmap then begin

      DeleteObjects;

      DC:=WinDC;

      Picture:=CreateCompatibleBitmap(WinDC,WinWidth,WinHeight);
      MemDC:=CreateCompatibleDC(WinDC);
      SelectObject(MemDc,Picture);

      WinDC:=MemDC;

      screen.mustBeDeleted:=true;

    end;

    SelectObject(WinDC,GetStockObject(options.brush));
    Rectangle(WinDC,WinSize.left,WinSize.bottom,WinSize.right,WinSize.top);
    MoveTo(WinDC,round(frac.offsx),round(frac.offsy));
    SetROP2(WinDC,r2_copypen);

    frac.sumang:=0.0;
    for i:=1 to options.sides do begin
      Turtle(i);
      if options.usealpha then
        step:=options.alpha
      else
        if i=1 then step:=360/options.sides-frac.sumang;

      frac.sumang:=frac.sumang+step;
    end;

    if usebitmap then begin
      WinDC:=DC;
      Restore;
    end;

    ReleaseDC(HWindow,WinDC);

    WVSPrintf(str,'Ready. Level: %d',depth);
    SetWindowText(hWindow,str);

  end;

end;


procedure TWinFractal.TurtleCalc;

var pp : PPatternItem;
    pcom : PTurtleItem;
    pertext : string[20];
    percent, iii, ii, i : INTEGER;
    text : array[0..40] of char;
    a,  l : REAL;
    comcol : PTurtleItem;
    startang, sangle : real;

begin

  with OwnerWin^ do begin

    if depth=1 then begin

      Dispose(pturtlemem[0],Done);
      Dispose(pturtlemem[1],Done);

      new(pturtlemem[0],Init(10,10));
      new(pturtlemem[1],Init(10,10));
      switch:=0;

      for i:=2 to pattern.ppoints^.count-1 do begin
        pp:=pattern.ppoints^.at(i);
        pturtlemem[switch]^.Insert(New(PTurtleItem,
          Init(pp^.irelangle,pp^.ilength,
            pp^.idrawflag,pp^.iexpandflag,i)));
      end;
    end

    else begin

      switch:=1-switch;
      Dispose(pturtlemem[switch],Done);
      new(pturtlemem[switch],Init(10,10));
      sangle:=0.0;

      for ii:=0 to pturtlemem[1-switch]^.count-1 do begin

        Yield;

        percent:=round((ii+1)/(pturtlemem[1-switch]^.count)*100.0);

        if (percent mod 10) =0 then begin
          Str(percent,pertext);
          StrPCopy(text,'Calculating, wait - '+pertext+' %');
          SetWindowText(hWindow,text);
        end;

        pcom:=pturtlemem[1-switch]^.At(ii);
        a:=pcom^.relangle;

        sangle:=sangle+a;
        l:=pcom^.length;

        if pcom^.expandflag and pcom^.drawflag then begin
          startang:=sangle;

          for iii:=2 to pattern.ppoints^.count-1 do begin

            pp:=pattern.ppoints^.At(iii);
            sangle:=sangle+pp^.irelangle;

            if iii=2 then
              pturtlemem[switch]^.Insert(New(PTurtleItem,
                Init(a+pp^.irelangle,l*pp^.ilength,
                  pp^.idrawflag,pp^.iexpandflag,iii)))
            else
              pturtlemem[switch]^.
               Insert(New(PTurtleItem,
                 Init(pp^.irelangle ,l*pp^.ilength,
                   pp^.idrawflag,pp^.iexpandflag,iii)));

          end;

          if options.treemode then
            pturtlemem[switch]^.
              Insert(New(PTurtleItem,Init(startang-sangle,
                l*pattern.endpoint.ilength,false,false,pcom^.patind)));

        end
        else
           pturtlemem[switch]^.
            Insert(New(PTurtleItem,Init(a,
              l*pattern.endpoint.ilength,pcom^.drawflag,false,pcom^.patind)));


      end;
    end;
  end;

end;


procedure TWinFractal.PreTurtle;

var
    pcom : PTurtleItem;
    iii, ii, i : INTEGER;
    a,  l : REAL;
    comcol : PTurtleItem;

begin
 with OwnerWin^ do begin

    for i:=0 to pturtlemem[switch]^.count-1 do begin

      Yield;

      pcom:=pturtlemem[switch]^.At(i);

      sumang:=sumang+pcom^.relangle;

      if (pcom^.patind=2) and options.usebeta then sumang:=sumang+options.beta;

      l:=pcom^.length;

      offsx:=offsx+cos(sumang*pi180)*l;
      offsy:=offsy+sin(sumang*pi180)*l;

      if pcom^.drawflag then begin
        if offsx>fxmax then fxmax:=offsx;
        if offsx<fxmin then fxmin:=offsx;
        if offsy>fymax then fymax:=offsy;
        if offsy<fymin then fymin:=offsy;
      end;

    end;

  end;

end;

procedure TWinFractal.Turtle(actside : integer);

var
    pcom : PTurtleItem;
    pp : PPatternItem;
    range, percent : longint;
    iii, ii, i : INTEGER;
    a,  l : REAL;
    oldpen, pen : hpen;
    comcol : PTurtleItem;
    pertext : string[20];
    text : array[0..40] of char;

begin

 with OwnerWin^ do begin

  MoveTo(screen.WinDC,round(offsx),round(offsy));
  range:=pturtlemem[switch]^.count-1;
  SetROP2(screen.WinDC,r2_copypen);

  for i:=0 to range do begin

    Yield;

    percent:=round((range*(actside-1)+i+1)/
                    (range*options.sides)*100.0);

    if (percent mod 10) = 0 then begin
      Str(percent,pertext);
      StrPCopy(text,'Painting, wait - '+pertext+' %');
      SetWindowText(hWindow,text);
    end;

    pcom:=pturtlemem[switch]^.At(i);

    sumang:=sumang+pcom^.relangle;

    if (pcom^.patind=2) and options.usebeta then sumang:=sumang+options.beta;

    l:=pcom^.length;
    offsx:=offsx+cos(sumang*pi180)*l*xfac;
    offsy:=offsy+sin(sumang*pi180)*l*yfac;
    with options do
     if pcom^.drawflag then begin
        pp:=pattern.ppoints^.At(pcom^.patind);
        Pen:=CreatePen(ps_solid,pp^.iwidth,pp^.icolor);
        OldPen:=SelectObject(screen.WinDc,Pen);
        LineTo(screen.WinDC,round(offsx),round(offsy));
        SelectObject(screen.WinDC,OldPen);
        DeleteObject(Pen);
      end
      else
        MoveTo(Screen.WinDC,round(offsx),round(offsy));
   end;

 end;

end;


(* ---------------------------------- *)

(* TPatternItem-Methoden *)

constructor TPatternItem.Init(MyOwner : PFracWindow);
var zf : real;
    tempx, tempy : integer;
begin

  with MyOwner^ do with MyOwner^.editor do begin
    zf:=options.zoomfactor;

    tempx:=last.x;
    tempy:=last.y;
    InvTransform(tempx,tempy);
    ipos.x:=tempx;
    ipos.y:=tempy;
  end;

  with MyOwner^.options DO begin
    idrawflag:=not(menopt.isinvisible);
    icolor:=colorref;
    iwidth:=width;
    iexpandflag:=menopt.isexpand;
  end;

end;

destructor TPatternItem.Done;
begin
end;

(* ---------------------------------------- *)


constructor TWinPattern.Init(MyOwner : PFracWindow);
begin
  new(ppoints,Init(100,10));
  nopoints:=TRUE;
  offset.x:=0;
  offset.y:=0;
  showallpoints:=true;
  OwnerWin:=MyOwner;
  MoveBox.Init(MyOwner);
end;

procedure TWinPattern.CleanUp;
begin
  Dispose(ppoints,Done);
end;

procedure TWinPattern.InsertPoint;
begin
  ppoints^.Insert(New(PPatternItem,Init(OwnerWin)));
  nopoints:=false;
end;

procedure TWinPattern.SetStartpoint;
begin
  startpoint.init(OwnerWin);
end;

procedure TWinPattern.SetEndpoint;
begin
  endpoint.Init(OwnerWin);
end;

procedure TWinPattern.DeletePoint;
begin
  if ppoints^.count>0 then
    ppoints^.AtFree(ppoints^.count-1);
  nopoints:=ppoints^.count=0;
end;


function TWinPattern.GetNumberOfPoints:INTEGER;
begin
  GetNumberOfPoints:=ppoints^.count-1;
end;



function TWinPattern.IsEmpty:boolean;
begin
  IsEmpty:=nopoints;
end;

function TWinPattern.GetLastItem:PPatternItem;
begin
  GetLastItem:=ppoints^.at(ppoints^.count-1);
end;

procedure TWinPattern.SetOffset(x, y : integer);
begin
  offset.x:=x;
  offset.y:=y;
end;

procedure TWinPattern.ChangePointTo(index, newx, newy : integer);
var pp : PPatternItem;
begin
  pp:=ppoints^.at(index);
  OwnerWin^.InvTransform(newx,newy);
  pp^.ipos.x:=newx;
  pp^.ipos.y:=newy;
end;


procedure TWinPattern.Draw;
var pp : PPatternItem;
   oldpen, pen : HPen;
   tempx, tempy, i : INTEGER;
   zf : real;

  procedure drawmarker(x, y : integer; col : tcolorref); far;
  begin
      if not(showallpoints) then exit;

       with ownerwin^.screen do begin
          Pen:=CreatePen(ps_solid,1,col);
          OldPen := SelectObject(WinDc,Pen);

          OwnerWin^.Transform(x,y);

          Rectangle(WinDC,x-4,y-4,x+4,y+4);

          SelectObject(WinDC,OldPen);
          DeleteObject(Pen);

      end;
    end;

  procedure doit(item : PPatternItem); far;
  var x, y : integer;

  begin
    with OwnerWin^ do with OwnerWin^.screen do begin
      x:=loword(GetCurrentPosition(WinDC));
      y:=hiword(GetCurrentPosition(WinDC));
      with item^ do begin

        tempx:=ipos.x;
        tempy:=ipos.y;
        Transform(tempx,tempy);

        if idrawflag then begin

          Pen:=CreatePen(ps_solid,iwidth,icolor);
          OldPen := SelectObject(WinDc,Pen);

          LineTo(WinDC,tempx,tempy);

          SelectObject(WinDC,OldPen);
          DeleteObject(Pen);

          if iexpandflag then begin
            MoveTo(WinDC,x,y);

            Pen:=CreatePen(ps_dot,0,$01000000);
            OldPen := SelectObject(WinDC,Pen);

            LineTo(WinDC,tempx,tempy);

            SelectObject(WinDC,OldPen);
            DeleteObject(Pen);

          end;
       end
        else
          MoveTo(WinDC,tempx,tempy);
      end;
    end;
  end;

begin

  with OwnerWin^ do with OwnerWin^.screen do begin

    zf:=options.zoomfactor;

    SelectObject(WinDC,GetStockObject(options.brush));
    Rectangle(WinDC,WinSize.left,WinSize.bottom,WinSize.right,WinSize.top);
    Editor.GRid.Draw;

    SetROP2(WinDC,r2_copypen);
    SetBkMode(WinDC,transparent);

    if ppoints^.count>1 then begin

      SelectPalette(WinDC, WinPalette, false);
      RealizePalette(WinDC);

      pp:=ppoints^.At(0);
      tempx:=pp^.ipos.x;
      tempy:=pp^.ipos.y;
      Transform(tempx,tempy);
      MoveTo(WinDC,tempx,tempy);

      pp:=ppoints^.At(1);
      tempx:=pp^.ipos.x;
      tempy:=pp^.ipos.y;
      Transform(tempx,tempy);

      Pen:=CreatePen(ps_dot,0,vecstartcol);
      OldPen := SelectObject(WinDC,Pen);

      LineTo(WinDC,tempx,tempy);
             SelectObject(WinDC,OldPen);
      DeleteObject(Pen);

      DrawMarker(pp^.ipos.x,pp^.ipos.y,vecstartcol);


      for i := 2 to ppoints^.Count-1 do
        doit(ppoints^.At(i));

         SetROP2(WinDC,r2_copypen);
    SetBkMode(WinDC,transparent);

      for i := 2 to ppoints^.Count-1 do begin
        pp:=ppoints^.At(i);
        drawmarker(pp^.ipos.x,pp^.ipos.y,pp^.icolor);
      end;
    end;

    Pen:=CreatePen(ps_solid,0,vecstartcol);
    OldPen := SelectObject(WinDC,Pen);

    if editor.edstate.startpointset then begin
      tempx:=startpoint.ipos.x;
      tempy:=startpoint.ipos.y;
      Transform(tempx,tempy);

      with pattern.startpoint do begin
        Ellipse(WinDC,tempx-7,tempy-7,
                      tempx+7,tempy+7);
        Ellipse(WinDC,tempx-5,tempy-5,
                      tempx+5,tempy+5);
      end;
    end;

    if editor.edstate.endpointset then begin
      tempx:=endpoint.ipos.x;
      tempy:=endpoint.ipos.y;
      Transform(tempx,tempy);

      with pattern.endpoint do begin
        Rectangle(WinDC,tempx-7,tempy-7,
                        tempx+7,tempy+7);
        Rectangle(WinDC,tempx-5,tempy-5,
                        tempx+5,tempy+5);
      end;

    end;

    SelectObject(WinDC,OldPen);
    DeleteObject(Pen);

    Editor.Cross.DontErasePrevious;
    Editor.Cross.Draw;

  end;
end;


procedure TWinPattern.ShowPoints;
begin
  showallpoints:=true;
end;


procedure TWinPattern.DontShowPoints;
begin
  showallpoints:=false;
end;

procedure TWinPattern.AllignToGrid;
var pp : PPatternItem;
    x,y, i : integer;
    lastx, lasty : integer;

begin
  for i:=0 to ppoints^.count-1 do begin
    pp:=ppoints^.at(i);
    x:=pp^.ipos.x;
    y:=pp^.ipos.y;
    OwnerWin^.GridTransform(x,y);
    OwnerWin^.InvGridTransform(x,y);
    if (i=0) or ((x<>lastx) or (y<>lasty)) then begin
      pp^.ipos.x:=x;
      pp^.ipos.y:=y;
    end;
    lastx:=x;
    lasty:=y;
  end;
  with startpoint do begin
    OwnerWin^.GridTransform(ipos.x,ipos.y);
    OwnerWin^.InvGridTransform(ipos.x,ipos.y);
  end;
   with endpoint do begin
    OwnerWin^.GridTransform(ipos.x,ipos.y);
    OwnerWin^.InvGridTransform(ipos.x,ipos.y);
  end;
  OwnerWin^.CopyLastItemIntoEditor;
end;

function TWinPattern.FindPoint(x, y, delta : integer):integer;
var pp : PPatternItem;
    i : integer;
    point : TPoint;
    length, newlength : real;
begin

  OwnerWin^.InvTransform(x,y);
  point.x:=x;
  point.y:=y;

  FindPoint:=-1;
  length:=0.0;
  for i:=0 to ppoints^.count-1 do begin
    pp:=ppoints^.at(i);
    if ((pp^.ipos.x-delta)<=x) and (x<=(pp^.ipos.x+delta)) and
       ((pp^.ipos.y-delta)<=y) and (y<=(pp^.ipos.y+delta)) then begin
      newlength:=OwnerWin^.CalcLength(point,pp^.ipos);
      if (newlength<length) or (length=0.0) then begin
        length:=newlength;
        FindPoint:=i;
      end;
    end;
  end;
end;

function TWinPattern.IsStartpoint(x, y, delta : integer):boolean;
begin

  OwnerWin^.InvTransform(x,y);
  IsStartPoint:=false;
  if ((startpoint.ipos.x-delta)<=x) and (x<=(startpoint.ipos.x+delta)) and
       ((startpoint.ipos.y-delta)<=y) and (y<=(startpoint.ipos.y+delta)) then
      IsStartPoint:=true;
end;

function TWinPattern.IsEndpoint(x, y, delta : integer):boolean;
begin

  OwnerWin^.InvTransform(x,y);
  IsEndPoint:=false;
  if ((endpoint.ipos.x-delta)<=x) and (x<=(endpoint.ipos.x+delta)) and
       ((endpoint.ipos.y-delta)<=y) and (y<=(endpoint.ipos.y+delta)) then
      IsEndPoint:=true;
end;


procedure TWinPattern.TransformForTurtle;
var pp : PPatternItem;
    now, last : TPoint;
    sx, sy, asx, asy, i : integer;
    dx, dy : real;
    sa, sb, length, rs : real;
    srel : real;

begin

  for i:=0 to ppoints^.count-1 do begin

    pp:=ppoints^.at(i);
    now:=pp^.ipos;
    srel:=pp^.irelangle;

    if i>0 then begin

     dy:=abs(now.y-last.y);
     dx:=abs(now.x-last.x);

      length:=OwnerWin^.CalcLength(now,last);

      if (length<1e-10) then
        rs:=0.0
      else begin
        rs:=(dy*OwnerWin^.screen.corfac)/length;
      end;

      if sqrt(1.0-sqr(rs))=0.0 then
        sa:=90.0
      else
        sa:=arctan(rs/sqrt(1.0-sqr(rs)))*180.0/pi;


      if now.x>last.x then sx:=1 else sx:=-1;
      if now.y>last.y then sy:=1 else sy:=-1;

      if (asx=1) and (asy=1) then begin
        if (sx=1) and (sy=1) then srel:=360-sb+sa;
        if (sx=1) and (sy=-1) then srel:=360-sb-sa;
        if (sx=-1) and (sy=1) then srel:=180-sb-sa;
        if (sx=-1) and (sy=-1) then srel:=180-sb+sa;
      end;

      if (asx=1) and (asy=-1) then begin
        if (sx=1) and (sy=1) then srel:=sa+sb;
        if (sx=1) and (sy=-1) then srel:=360+sb-sa;
        if (sx=-1) and (sy=1) then srel:=180+sb-sa;
        if (sx=-1) and (sy=-1) then srel:=180+sb+sa;
      end;

      if (asx=-1) and (asy=1) then begin
        if (sx=1) and (sy=1) then srel:=180+sa+sb;
        if (sx=1) and (sy=-1) then srel:=180+sb-sa;
        if (sx=-1) and (sy=1) then srel:=360+sb-sa;
        if (sx=-1) and (sy=-1) then srel:=sa+sb;
      end;

      if (asx=-1) and (asy=-1) then begin
        if (sx=1) and (sy=1) then srel:=180-sb+sa;
        if (sx=1) and (sy=-1) then srel:=180-sb-sa;
        if (sx=-1) and (sy=1) then srel:=360-sb-sa;
        if (sx=-1) and (sy=-1) then srel:=360-sb+sa;
      end;

      if srel>=360.0 then srel:=srel-360.0;

      pp^.irelangle:=srel;
      pp^.ilength:=length;

    end
    else begin

      pp^.irelangle:=0;
      pp^.ilength:=0;

    end; (* if *)

    last.x:=now.x;
    last.y:=now.y;

    sb:=sa;
    asx:=sx;
    asy:=sy;

  end;

  endpoint.ilength:=OwnerWin^.CalcLength(startpoint.ipos,endpoint.ipos);

end;


procedure TWinPattern.CalculateBoundingRect(var xf, yf, xt, yt : integer);

procedure Reset(pp : PPatternItem);
begin
  with pp^ do begin
    xf:=ipos.x;
    yf:=ipos.y;
    xt:=ipos.x;
    yt:=ipos.y;
  end;
end;

procedure Update(pp : PPatternItem);
begin
   with pp^ do begin
     if ipos.x<xf then xf:=ipos.x;
     if ipos.x>xt then xt:=ipos.x;
     if ipos.y<yf then yf:=ipos.y;
     if ipos.y>yt then yt:=ipos.y;
  end;
end;

var i : integer;
    pp : PPatternItem;
begin

 if ppoints^.count>1 then
   for i := 0 to ppoints^.Count-1 do begin
     pp:=ppoints^.at(i);
     if i=0 then Reset(pp) else Update(pp);
 end;

 OWnerWin^.Transform(xf,yf);
 OWnerWin^.Transform(xt,yt);

end;

(* ------------------------------------------- *)

(* TFrac-Window-Methoden *)

constructor TFracWindow.Init(AParent: PWindowsObject; ATitle: PChar);

begin
  inherited Init(AParent, ATitle);

  Attr.Menu := LoadMenu(HInstance,'FractalMenu');
  Attr.W:=400;
  Attr.H:=400;

  Init2;

  Printer:= New(PPrinter, Init);

end;

procedure TFracWindow.Init2;
begin

  with editor do begin

    edstate.mode:=SetPatternPoints;

    edstate.moveactmouse:=false;
    edstate.moveactmenu:=false;

    edstate.startpointset:=false;
    edstate.endpointset:=false;

    Cross.Init(@self);
    Grid.Init(@self);
    Grid.AdjustGridstep(20);
    Cross.SetBoxSize(10);
    lastadded.x:=0;
    lastadded.y:=0;
    last.x:=0;
    last.y:=0;
    edstate.firstclick:=false;
  end;

  with options do begin
    menopt.drawgrid:=TRUE;
    menopt.drawcross:=TRUE;
    menopt.drawline:=true;
    menopt.showpoints:=true;
    gridstep:=20;
    zoomfactor:=1.0;
  end;

  with screen do begin
    curmoverect:=LoadCursor(0,idc_size);
    cursetpoints:=LoadCursor(0,idc_Arrow);
    corfac:=4.0/3.0;
    usebitmap:=true;
    mustbedeleted:=false;
    winwidth:=attr.w;
    winheight:=attr.h;
  end;

  with options do begin
    menopt.isinvisible:=false;
    menopt.isexpand:=true;

    usealpha:=false;
    usebeta:=false;
    treemode:=false;
    alpha:=0;
    beta:=0;

    colorref:=0;
    width:=0;
    brush:=ltgray_brush;
    sides:=1;

  end;

  with Screen do begin
    BoundBox.Init(@self);
    BoundBox.SetDist(30,30);
  end;

  Frac.Init(@self);
  Pattern.Init(@self);
  SetupMenus;

end;

procedure TFracWindow.GetWindowClass(var WndClass : TWndClass);
begin
  Inherited GetWindowClass(WndClass);
  WndClass.Style:=cs_HRedraw or cs_VRedraw;
  WndClass.hCursor:=0; (* LoadCursor(0, idc_Arrow); *)
  WndClass.hbrbackground:=ltgray_brush;
  WndClass.hIcon:=LoadIcon(HInstance, 'FRACICON');
end;

destructor TFracWindow.Done;
begin
  Frac.CleanUp;
  Pattern.CleanUp;
  if screen.usebitmap then DeleteObjects;
  inherited Done;
end;

procedure TFracWindow.Transform(var x, y : integer);
begin
  x:=round((x + Pattern.offset.x)*options.zoomfactor);
  y:=round((y + Pattern.offset.y)*options.zoomfactor);
end;


procedure TFracWindow.InvTransform(var x, y : integer);
begin

  x:=round(x/options.zoomfactor)-Pattern.offset.x;
  y:=round(y/options.zoomfactor)-Pattern.offset.y;

end;


procedure TFracWindow.GridTransform(var x, y : integer);
var gs, zf : Real;
begin
  Transform(x,y);
  gs:=options.gridstep;
  zf:=options.zoomfactor;
  x:=round(round(x / (gs*zf)) * gs * zf);
  y:=round(round(y / (gs*zf)) * gs * zf);
end;

procedure TFracWindow.InvGridTransform(var x, y : integer);
begin
  InvTransform(x,y);
end;

function TFracWindow.CalcLength(source, destination : TPoint):real;
var dx, dy : real;
begin
  dx:=abs(source.x-destination.x);
  dy:=abs(source.y-destination.y);
  CalcLength:=sqrt(sqr(dx)+sqr(dy*screen.corfac));
end;

procedure TFracWindow.ClearWindow;
begin
 Editor.Cross.DontErasePrevious;
 PostMessage(hWindow,wm_Paint,0,0);
end;

procedure TFracWindow.SetMenuItem(itemid : integer; on, checked : boolean);
begin
   if on then
     EnableMenuItem(GetMenu(hWindow),itemid,mf_bycommand or mf_enabled)
   else
     EnableMenuItem(GetMenu(hWindow),itemid,mf_bycommand or mf_grayed);

   if checked then
     CheckMenuItem(GetMenu(hWindow),itemid,mf_bycommand or mf_checked)
   else
     CheckMenuItem(GetMenu(hWindow),itemid,mf_bycommand or mf_unchecked);

end;


procedure TFracWindow.SetupMenus;
begin

  case editor.edstate.mode of

    SetPatternPoints : begin
                         SetMenuItem(cm_PenWidth,true,false);
                         SetMenuItem(cm_ChooseColor,true,false);
                         SetMenuItem(cm_Background,true,false);

                         SetMenuItem(cm_Expand,true,options.menopt.isexpand);
                         SetMenuItem(cm_Measure,not(pattern.IsEmpty),false);
                         SetMenuItem(cm_ChangePoints,editor.edstate.endpointset or
                                                     not(pattern.IsEmpty),false);

                         SetMenuItem(cm_invisible,true,options.menopt.isinvisible);

                         SetMenuItem(cm_ZoomFactor,true,false);
                         SetMenuItem(cm_DragMove,not(editor.edstate.moveactmenu or
                                                 editor.edstate.moveactmouse)
                                                 and not(pattern.isEmpty),
                                                 editor.edstate.moveactmenu or
                                                 editor.edstate.moveactmouse);

                         SetMenuItem(cm_AllignToGrid,not(pattern.IsEmpty),false);

                         SetMenuItem(cm_Grid,true,options.menopt.drawgrid);
                         SetMenuItem(cm_ShowPoints,true,options.menopt.showpoints);
                         SetMenuItem(cm_Cross,true,options.menopt.drawcross);
                         SetMenuItem(cm_DrawEditline,true,options.menopt.drawline);

                         SetMenuItem(cm_Next,editor.edstate.endpointset and
                                             not(pattern.IsEmpty),false);
                         SetMenuItem(cm_Boundaries,false,false);

                         SetMenuItem(cm_Reset,false,false);

                       end;

    ChangePatternPoints : begin
                                SetMenuItem(cm_PenWidth,false,false);
                         SetMenuItem(cm_ChooseColor,false,false);
                         SetMenuItem(cm_Background,true,false);

                         SetMenuItem(cm_Expand,false,options.menopt.isexpand);
                         SetMenuItem(cm_Measure,false,false);
                         SetMenuItem(cm_ChangePoints,true,true);

                         SetMenuItem(cm_invisible,false,options.menopt.isinvisible);

                         SetMenuItem(cm_zoomfactor,true,false);
                         SetMenuItem(cm_DragMove,not(editor.edstate.moveactmenu or
                                                 editor.edstate.moveactmouse)
                                                 and not(pattern.isEmpty),
                                                 editor.edstate.moveactmenu or
                                                 editor.edstate.moveactmouse);

                         SetMenuItem(cm_AllignToGrid,not(pattern.IsEmpty),false);

                         SetMenuItem(cm_Grid,true,options.menopt.drawgrid);
                         SetMenuItem(cm_ShowPoints,true,options.menopt.showpoints);
                         SetMenuItem(cm_Cross,true,options.menopt.drawcross);
                         SetMenuItem(cm_DrawEditline,false,options.menopt.drawline);

                         SetMenuItem(cm_Next,editor.edstate.endpointset and
                                             not(pattern.IsEmpty),false);
                         SetMenuItem(cm_Boundaries,false,false);

                         SetMenuItem(cm_Reset,false,false);

                          end;

    measurepattern : begin

                      SetMenuItem(cm_PenWidth,false,false);
                      SetMenuItem(cm_ChooseColor,false,false);
                      SetMenuItem(cm_Background,true,false);

                         SetMenuItem(cm_Expand,false,options.menopt.isexpand);
                         SetMenuItem(cm_Measure,false,true);
                         SetMenuItem(cm_ChangePoints,false,false);

                         SetMenuItem(cm_invisible,false,options.menopt.isinvisible);

                         SetMenuItem(cm_zoomfactor,true,false);
                         SetMenuItem(cm_DragMove,not(editor.edstate.moveactmenu or
                                                 editor.edstate.moveactmouse)
                                                 and not(pattern.isEmpty),
                                                 editor.edstate.moveactmenu or
                                                 editor.edstate.moveactmouse);

                         SetMenuItem(cm_AllignToGrid,not(pattern.IsEmpty),false);

                         SetMenuItem(cm_Grid,true,options.menopt.drawgrid);
                         SetMenuItem(cm_ShowPoints,true,options.menopt.showpoints);
                         SetMenuItem(cm_Cross,true,options.menopt.drawcross);
                         SetMenuItem(cm_DrawEditline,false,options.menopt.drawline);

                         SetMenuItem(cm_Next,editor.edstate.endpointset and
                                             not(pattern.IsEmpty),false);
                         SetMenuItem(cm_Boundaries,false,false);

                         SetMenuItem(cm_Reset,false,false);


                     end;

    boundfractal : begin


                      SetMenuItem(cm_PenWidth,false,false);
                      SetMenuItem(cm_ChooseColor,false,false);
                      SetMenuItem(cm_Background,true,false);

                         SetMenuItem(cm_Expand,false,options.menopt.isexpand);
                         SetMenuItem(cm_Measure,false,false);
                         SetMenuItem(cm_ChangePoints,false,false);

                         SetMenuItem(cm_invisible,false,options.menopt.isinvisible);

                         SetMenuItem(cm_zoomfactor,false,false);
                         SetMenuItem(cm_DragMove,false,false);

                         SetMenuItem(cm_AllignToGrid,false,false);

                         SetMenuItem(cm_Grid,false,options.menopt.drawgrid);
                         SetMenuItem(cm_ShowPoints,false,options.menopt.showpoints);
                         SetMenuItem(cm_Cross,false,options.menopt.drawcross);
                         SetMenuItem(cm_DrawEditline,false,options.menopt.drawline);

                         SetMenuItem(cm_Next,true,false);
                         SetMenuItem(cm_Boundaries,false,true);

                         SetMenuItem(cm_Reset,true,false);

              end;

   fractaldraw : begin


                      SetMenuItem(cm_PenWidth,false,false);
                      SetMenuItem(cm_ChooseColor,false,false);
                      SetMenuItem(cm_Background,true,false);

                         SetMenuItem(cm_Expand,false,options.menopt.isexpand);
                         SetMenuItem(cm_Measure,false,false);
                         SetMenuItem(cm_ChangePoints,false,false);

                         SetMenuItem(cm_invisible,false,options.menopt.isinvisible);

                         SetMenuItem(cm_zoomfactor,false,false);
                         SetMenuItem(cm_DragMove,false,false);

                         SetMenuItem(cm_AllignToGrid,false,false);

                         SetMenuItem(cm_Grid,false,options.menopt.drawgrid);
                         SetMenuItem(cm_ShowPoints,false,options.menopt.showpoints);
                         SetMenuItem(cm_Cross,false,options.menopt.drawcross);
                         SetMenuItem(cm_DrawEditline,false,options.menopt.drawline);

                         SetMenuItem(cm_Next,true,false);
                         SetMenuItem(cm_Boundaries,true,false);

                         SetMenuItem(cm_Reset,true,false);

       end;
   end;

   if editor.edstate.moveactmenu or editor.edstate.moveactmouse then begin

                      SetMenuItem(cm_PenWidth,false,false);
                      SetMenuItem(cm_ChooseColor,false,false);
                      SetMenuItem(cm_Background,false,false);

                         SetMenuItem(cm_Expand,false,options.menopt.isexpand);
                         SetMenuItem(cm_Measure,false,editor.edstate.mode=MeasurePattern);
                         SetMenuItem(cm_ChangePoints,false,editor.edstate.mode=ChangePatternPoints);

                         SetMenuItem(cm_invisible,false,options.menopt.isinvisible);

                         SetMenuItem(cm_zoomfactor,false,false);
                         SetMenuItem(cm_DragMove,false,true);

                         SetMenuItem(cm_AllignToGrid,false,false);

                         SetMenuItem(cm_Grid,false,options.menopt.drawgrid);
                         SetMenuItem(cm_ShowPoints,false,options.menopt.showpoints);
                         SetMenuItem(cm_Cross,false,options.menopt.drawcross);
                         SetMenuItem(cm_DrawEditline,false,options.menopt.drawline);

                         SetMenuItem(cm_Next,false,false);
                         SetMenuItem(cm_Boundaries,false,false);

                         SetMenuItem(cm_Reset,false,false);
   end;

end;

function TFracWindow.CanClose:boolean;
begin
 if Pattern.IsEmpty or (MessageBox(HWindow,
     'Are you sure you want to quit?',
     'EXIT selected!',
    mb_IconInformation or mb_okcancel)=id_Ok) then
    CanClose:=True
 else
   CanClose:=false;
end;

procedure TFracWindow.CMCloseWin(var Message : TMessage);
begin
  if CanClose then
    Done;
end;

procedure TFracWindow.WMClose(var Message : TMessage);
begin
  if canclose then Done;
end;

procedure TFracWindow.Paint(PaintDC: HDC; var PaintInfo: TPaintStruct);
begin
  with screen do begin
    WinDC:=PaintDC;

    if Frac.depth=0 then begin
      screen.windc:=GetDC(hWindow);
      Pattern.Draw;
      ReleaseDC(hWindow,screen.windc);
      end
    else
      if usebitmap then
        Restore
      else
        Frac.Draw;
  end;

end;

procedure TFracWindow.WMSize(var Msg: TMessage);
var oldwinwidth, oldwinheight : integer;
    x, y : integer;

begin

  if msg.wparam=1 (* size_minimized *) then
    SetWindowText(hWindow,appname);

  GetClientRect(hWindow,screen.WinSize);

  with screen do begin

    oldWinWidth:=WinWidth;
    oldWinHeight:=WinHeight;

    WinWidth:=WinSize.right-WinSize.left;
    WinHeight:=WinSize.bottom-WinSize.top;
  end;

  screen.boundbox.GetDist(x,y);
  if oldwinwidth<>0 then
    x:=round(x*(screen.winwidth/oldwinwidth));
  if oldwinheight<>0 then
    y:=round(y*(screen.winheight/oldwinheight));

   screen.BoundBox.SetDist(x,y);

  if frac.depth=0 then begin
    screen.WinDC:=GetDC(hWindow);
    Pattern.Draw;
    ReleaseDC(hWindow,screen.WinDC);
  end
   else
     Frac.Draw;

end;


procedure TFracWindow.Restore;
begin

  with screen do
    BitBlt(WinDC,0,0,WinWidth,WinHeight,
           MemDC, 0,0, SRCCOPY);

end;


procedure TFracWindow.DeleteObjects;
begin
  if screen.mustbedeleted then begin
    DeleteDC(screen.MemDC);
    DeleteObject(screen.Picture);
  end;
end;


{ =======================================================================
  PROCEDURE : FillPalette
  AUFGABE   : Baut eine Farbtabelle fr die Palette auf.
  PARAMETER : -----------------------------------------------------------
              PalTable  : Hier die Farbtabelle reinschreiben
  ======================================================================= }

procedure TFracWindow.FillPalette(var PalTable:TPalTable; index : byte;
                                      red, green, blue : integer);
begin
    PalTable[index].peRed   := red;
    PalTable[index].peGreen := green;
    PalTable[index].peBlue  := blue;
    PalTable[index].peFlags := PC_RESERVED;
end;

{ =======================================================================
  PROCEDURE : SetupWindow
  AUFGABE   : Erstelle die Farbpalette
  ======================================================================= }

procedure TFracWindow.SetupWindow;
var
  PalSize:Word;
  MyLogPal:PLogPalette;
  DC:HDC;
begin
  TWindow.SetupWindow;
  PalSize  := SizeOf(TLogPalette)+(SizeOF(TPaletteEntry))*1;
  MyLogPal := PLogPalette(MemAlloc(PalSize));
  MyLogPal^.palVersion := $0300;
  MyLogPal^.palNumEntries := 1;

  { Flle die Palettentabelle mit Farbeintrag }

  FillPalette(TPalTable(MyLogPal^.palPalEntry),0,255,255,255);

  screen.WinPalette := CreatePalette(MyLogPal^);
  FreeMem(MyLogPal, PalSize);
  SetTimer(HWindow, TIM_Animate, 200, nil);
end; { proc }

{ =======================================================================
  PROCEDURE : WMTimer
  AUFGABE   : Bei jedem Timer-Ereignis wird die Farbpalette weiteranimiert.
  ======================================================================= }

procedure TFracWindow.WMTimer(var Msg:TMessage);
var
  NewPalTable:^TPalTable;
  i, PalSize:integer;
  HOldPalette:HPalette;
begin
  if Msg.WParam = TIM_Animate then begin
    PalSize     := SizeOF(TPaletteEntry)*1; { Berechne Gre der Tabelle}
    NewPalTable := MemAlloc(PalSize);   { Reserviere Speicher fr Tabelle }

   if screen.blinkflag then
       FillPalette(NewPalTable^,0,0,0,0)
      else
        FillPalette(NewPalTable^,0,255,255,255);

    screen.blinkflag:=not(screen.blinkflag);

    AnimatePalette(screen.WinPalette,0,1, NewPalTable^); { Ersetze die Farben }
    FreeMem(NewPalTable, PalSize);

  end
  else if Msg.WParam=TIM_ChangeCursor then
    with editor do begin
      KillTimer(hWindow,TIM_ChangeCursor);
      editor.edstate.moveactmouse:=true;
      pattern.MoveBox.InitMove;
  end;

end;

function min(a, b : integer):integer;
begin
  if a<b then min:=a else min:=b;
end;


function max(a, b : integer):integer;
begin
  if a<b then max:=b else max:=a;
end;


procedure TFracWindow.WMMouseMove(var Msg: TMessage);
var gs : integer;
    l, zf, rs, dy, dx : REAL;
    pp : PPatternItem;

    text : string;
    str, str2 : ARRAY[0..40] OF CHAR;

    info : ARRAY[0..3] OF INTEGER;
    info2: array[0..1] of integer;
    info3, info4 : array[0..2] of integer;

begin

  if (frac.depth>0) and not(editor.edstate.mode=BoundFractal) then exit;

  with screen do with editor do begin

    now.x:=LOWORD(Msg.LParam);
    now.y:=HIWORD(Msg.LParam);

    gs:=options.gridstep;
    zf:=options.zoomfactor;

    if options.menopt.drawgrid then begin
      now.x:=round(round(now.x / (gs*zf)) * gs * zf);
      now.y:=round(round(now.y / (gs*zf)) * gs * zf);
    end;

    if (now.x=last.x) and (now.y=last.y) then exit;

    last.x:=now.x;
    last.y:=now.y;


    WinDC:=GetDC(hWindow);

    l:=CalcLength(now,lastadded);

    info[0]:=Pattern.GetNumberOfPoints+1;
    info[1]:=now.x-lastadded.x;
    info[2]:=now.y-lastadded.y;
    info[3]:=round(l);

    info2[0]:=now.x;
    info2[1]:=now.y;

    info3[0]:=edstate.selected;
    info3[1]:=now.x;
    info3[2]:=now.y;

    info4[0]:=now.x-lastadded.x;
    info4[1]:=now.y-lastadded.y;
    info4[2]:=round(l);

    if edstate.moveactmenu or edstate.moveactmouse then
       StrPCopy(str,'Move the pattern!')
    else if edstate.mode=SetPatternPoints then begin
      if info[0]=0 then WVSPrintf(str,'Set prev. turtle pos. -> [%d %d]',info2)
      else if info[0]=1 then WVSPrintf(str,'Set origin (Nr. %d) -> [%d %d %d]',info)
      else begin
       with options.menopt do begin
         if isexpand and isinvisible           then text:='Set point Nr. %d -> [%d %d %d] EXP ---';
         if isexpand and not(isinvisible)      then text:='Set point Nr. %d -> [%d %d %d] EXP VIS';
         if not(isexpand) and isinvisible      then text:='Set point Nr. %d -> [%d %d %d] --- ---';
         if not(isexpand) and not(isinvisible) then text:='Set point Nr. %d -> [%d %d %d] --- VIS';
       end;

       StrPCopy(str2,text);
       WVSPrintf(str,str2,info);

      end
    end

    else if edstate.mode=ChangePatternPoints then begin
      if edstate.firstclick then begin
        if edstate.selected>-1 then WVSPrintf(str,'Point Nr. %d SELECTED -> [%d %d]',info3);
        if edstate.selected=-2 then WVSPrintf(str,'Measure-Startpoint SELECTED -> [%d %d]',info2);
        if edstate.selected=-3 then WVSPrintf(str,'Measure-Endpoint SELECTED -> [%d %d]',info2);
      end
      else
        WVSPrintf(str,'Select point to change -> [%d %d]',info2);
    end
    else if edstate.mode=MeasurePattern then begin
      if not(edstate.startpointset) then
        WVSPrintf(str,'Set Measure-Startpoint -> [%d %d]',info2)
      else
        WVSPrintf(str,'Set Measure-Endpoint -> [%d %d %d]',info4);
    end

    else if edstate.mode=BoundFractal then
      StrPCopy(str,'Fix the bounding-rectangle!');

    SetWindowText(hWindow,str);

    (* --------------- *)

    if (edstate.moveactmenu) or (edstate.moveactmouse) then
      with Pattern do with Pattern.MoveBox do begin
        SetCursor(curmoverect);
        MoveBox.Update;
        MoveBox.Draw;
    end
    else begin

      if (edstate.mode=SetPatternPoints) or (edstate.mode=MeasurePattern) then begin
        SetCursor(cursetpoints);
        Cross.DrawAt(now.x,now.y);
      end;

      if edstate.mode=BoundFractal then

        with screen do begin

         BoundBox.SetDist(now.x,now.y);
         if flag then begin
            flag:=false;
            BoundBox.DontErasePrevious;
          end;

          BoundBox.Draw;

      end; (* if *)

      if edstate.mode=ChangePatternPoints then begin
        Cross.DrawAt(now.x,now.y);

        if edstate.firstclick and (edstate.selected<>-1) then begin
          if edstate.selected>-1 then Pattern.ChangePointTo(edstate.selected,now.x,now.y);
          if edstate.selected=-2 then Pattern.SetStartpoint;
          if edstate.selected=-3 then Pattern.SetEndpoint;
          ClearWindow;
        end;
      end;

    end; (* if *)

  end; (* with *)

  ReleaseDC(HWindow,screen.WinDC);

end;


procedure TFracWindow.WMLButtonDown(var msg : TMessage);
begin
  if (frac.depth>0) and not(editor.edstate.mode=BoundFractal) then exit;

  if not((editor.edstate.moveactmouse) or (editor.edstate.moveactmenu)) then
    SetTimer(HWindow, TIM_ChangeCursor, 200, nil);

end;


procedure TFracWindow.WMLButtonUp(var Msg: TMessage);
var tempx, tempy : integer;
begin

  if (frac.depth>0) and not(editor.edstate.mode=BoundFractal) then exit;

  KillTimer(hWindow,TIM_ChangeCursor);
  SetCursor(screen.cursetpoints);
  Editor.Cross.DontErasePrevious;

  if editor.edstate.mode=BoundFractal then begin
    editor.edstate.mode:=FractalDraw;
    SetWindowText(hWindow,'Boundaries FIXED!');
    PostMessage(hWindow,wm_Size,0,0);
    SetupMenus;
  end

  else if (editor.edstate.moveactmouse) or (editor.edstate.moveactmenu) then begin
    SetMenuItem(cm_DragMove,true,false);
    SetWindowText(hWindow,'Pattern MOVED!');

    editor.edstate.moveactmouse:=false;
    editor.edstate.moveactmenu:=false;
  end

  else if editor.edstate.mode=MeasurePattern then begin

    if not(editor.edstate.startpointset) then begin
      editor.Cross.SetEditlineStart(editor.now.x,editor.now.y);
      Pattern.SetStartpoint;
      editor.edstate.startpointset:=true;
      options.menopt.drawline:=options.savedmenopt.drawline;

      if options.menopt.drawline then
        editor.Cross.AllowDrawEditline
      else
        editor.Cross.DontDrawEditline;

      SetWindowText(hWindow,'Measure-Startpoint SET!');
    end
    else begin
      editor.edstate.mode:=SetPatternPoints;
      Pattern.SetEndpoint;
      editor.edstate.endpointset:=true;
      CopyLastItemIntoEditor;

      SetWindowText(hWindow,'Measure-Endpoint SET!');

    end;
  end

  else if editor.edstate.mode=SetPatternPoints then begin

    with editor do begin

      if (now.x=lastadded.x) and (now.y=lastadded.y) then
        SetWindowText(hWindow, '@!?* Double Point - Ignored! ?*@!')
      else begin
        Pattern.InsertPoint;
        SetWindowText(hWindow,'Point SET!');
      end;
      Cross.SetEditlineStart(now.x,now.y);
      lastadded.x:=now.x;
      lastadded.y:=now.y;
    end;

  end

  else if editor.edstate.mode=ChangePatternPoints then begin

    editor.edstate.firstclick:=not(editor.edstate.firstclick);
    if editor.edstate.firstclick then begin
      editor.edstate.selected:=Pattern.FindPoint(editor.now.x,editor.now.y,options.gridstep div 2);
      if editor.edstate.selected=-1 then begin

        if editor.edstate.startpointset and
           Pattern.IsStartpoint(editor.now.x,editor.now.y,options.gridstep div 2) then begin
          editor.edstate.selected:=-2;
          SetWindowText(hWindow, 'Measure-Startpoint SELECTED!');
        end
        else if editor.edstate.endpointset and
          Pattern.IsEndpoint(editor.now.x,editor.now.y,options.gridstep div 2) then begin
            editor.edstate.selected:=-3;
            SetWindowText(hWindow, 'Measure-Endpoint SELECTED!');
        end
        else begin
          SetWindowText(hWindow, '@!?* No point found! ?*@!');
          editor.edstate.firstclick:=false;
        end;

      end
      else begin
        SetWindowText(hWindow, 'Pattern-Point SELECTED!');
      end;

    end
    else begin
      CopyLastItemIntoEditor;
      SetWindowText(hWindow,'Point CHANGED!');
    end;
  end;

  SetupMenus;
  ClearWindow;

end;

procedure TFracWindow.CopyLastItemIntoEditor;
var pp : PPatternItem;
begin

  with pattern do
    if not(IsEmpty) then begin

       pp:=GetLastItem;

       with editor do begin
         lastadded.x:=round((pp^.ipos.x + offset.x)*options.zoomfactor);
         lastadded.y:=round((pp^.ipos.y + offset.y)*options.zoomfactor);
        end;
     end;

     with Editor do
       Cross.SetEditlineStart(lastadded.x,lastadded.y);

end;

procedure TFracWindow.WMRButtonDown(var Msg : TMessage);
var pp : PPatternItem;
begin

 if (frac.depth>0) or (editor.edstate.mode<>SetPatternPoints) then exit;

 with Pattern do begin
   if not(IsEmpty) then begin
     SetWindowText(hWindow, 'Point DELETED!');
     DeletePoint;
     CopyLastItemIntoEditor;
   end
   else
     SetWindowText(hWindow,'@!?* No more points! ?*@!');

 end;

 SetupMenus;
 ClearWindow;

end;

procedure TFracWindow.CMDragMove(var Msg : TMessage);
var pp : PPatternItem;
    x, y : integer;
begin

  with editor do with screen do begin

    WinDC:=GetDC(hWindow);
    Cross.DontErasePrevious;
    Cross.Draw;
    Cross.DontErasePrevious;

    pp:=Pattern.GetLastItem;

    x:=pp^.ipos.x;
    y:=pp^.ipos.y;


    Transform(x,y);

    editor.now.x:=x;
    editor.now.y:=y;

    SetCursorPos(x + loword(GetDCOrg(WinDC)),
                 y + hiword(GetDCOrg(WinDC)));

    ReleaseDC(hWindow,WinDC);

    editor.edstate.moveactmenu:=true;
    Pattern.MoveBox.InitMove;


  end;


end;

procedure TFracWindow.CMPrint(var Msg: TMessage);
var
  BildschirmObjekt:PWindowPrintOut;
begin
  BildschirmObjekt := New(PWindowPrintOut, Init('Fractal Printout', @self));
  Printer^.Print(@self, BildschirmObjekt);
  Dispose(BildschirmObjekt);
end;

procedure TFracWindow.CMPrintSetup(var msg: TMessage);
begin
  Printer^.Setup(@self);
end;

procedure TFracWindow.CMNew(VAR msg: TMessage);

begin
  if not(Pattern.IsEmpty) and (MessageBox(HWindow,
      'Are you sure you want to clear your work?',
      'NEW selected!',
      mb_IconInformation or mb_okcancel)=id_Ok) then begin


    CMReset(msg);
    Pattern.CleanUp;
    Frac.CleanUp;
    Init2;
    ClearWindow;

  end;

end;


procedure TFracWindow.CMChooseColor(var Msg : TMessage);
var ChooseClr : TChooseColor;

begin
  with ChooseClr do begin
    HWndOwner:=hWindow;
    lStructSize:=SizeOf(TChooseColor);
    rgbresult:=options.ColorRef;
    lpCustColors:=@CustColors;
    Flags := cc_RGBInit;
  end;
  if ChooseColor(ChooseClr) then
    options.ColorRef:=ChooseClr.RGBResult;
end;

procedure TFracWindow.CMAbout(var Msg: TMessage);
begin
  Application^.ExecDialog(New(PDialog, Init(@Self, 'ABOUTBOX')));
  ClearWindow;
end;

procedure TFracWindow.CMShowPoints(var Msg : TMessage);
begin
  options.menopt.showpoints:=not(options.menopt.showpoints);
  if options.menopt.showpoints then
    Pattern.ShowPoints
  else
     Pattern.DontShowPoints;

  SetupMenus;
  ClearWindow;

end;

procedure TFracWindow.CMOptions(var msg: TMessage);

var Dialog : TOptionDialog;
    i : integer;

begin

  Dialog.Init(@Self,'SIDESBOX');
  if Dialog.Execute=id_ok then begin end;
  Dialog.Done;
  PostMessage(hWindow,wm_Size,0,0);
end;


procedure TFracWindow.CMPenWidth(var msg: TMessage);

var Dialog : TPenWidthDialog;
    i : integer;

begin
  Dialog.Init(@Self,'PENWIDTH');
  if Dialog.Execute=id_ok then begin end;
  Dialog.Done;
  ClearWindow;
end;

procedure TFracWindow.CMExpand(var msg : TMessage);
begin
  options.menopt.isexpand:=not(options.menopt.isexpand);
  SetupMenus;
end;

procedure TFracWindow.CMMeasure(var msg : TMessage);
begin


  editor.savededstate:=editor.edstate;
  options.savedmenopt:=options.menopt;

  editor.edstate.mode:=MeasurePattern;
  editor.edstate.startpointset:=false;
  editor.edstate.endpointset:=false;

  options.menopt.drawline:=false;
  editor.cross.DontDrawEditline;


  pattern.movebox.savedlastpoint.x:=editor.cross.lastpoint.x;
  pattern.movebox.savedlastpoint.y:=editor.cross.lastpoint.y;

  SetupMenus;
  ClearWindow;
end;

procedure TFracWindow.CMBoundaries(var Msg : TMessage);
begin
  if frac.depth>0 then begin
    editor.edstate.mode:=BoundFractal;
    screen.flag:=true;
  end;
end;

procedure TFracWindow.CMGrid(VAR msg : TMessage);
begin

  options.menopt.drawgrid:=not(options.menopt.drawgrid);
  SetupMenus;
  ClearWindow;
end;

procedure TFracWindow.CMZoomFactor(var msg : TMessage);

var Dialog : TScrollDialog;
    i : integer;

begin
  Dialog.Init(@Self,'ZOOMDLG');
  if Dialog.Execute=id_ok then begin end;
  Dialog.Done;
  ClearWindow;
end;

procedure TFracWindow.CMCross(VAR msg : TMessage);
begin

  options.menopt.drawcross:=not(options.menopt.drawcross);

  if options.menopt.drawcross then
    editor.Cross.AllowDraw
  else
    editor.Cross.DontDraw;

  SetupMenus;
  ClearWindow;

end;

procedure TFracWindow.CMDrawEditline(VAR msg : TMessage);
begin

  options.menopt.drawline:=not(options.menopt.drawline);

  if options.menopt.drawline then
    editor.Cross.AllowDrawEditline
  else
    editor.Cross.DontDrawEditline;

  SetupMenus;
  ClearWindow;

end;


procedure TFracWindow.CMChangePoints(var msg : TMessage);
begin

  if editor.edstate.mode<>ChangePatternPoints then begin
    editor.edstate.mode:=ChangePatternPoints;
    options.savedmenopt:=options.menopt;
    options.menopt.drawline:=false;
    editor.Cross.DontDrawEditline;
    editor.Cross.SetBoxMode;
  end
  else begin
    editor.edstate.mode:=setpatternpoints;
    if options.savedmenopt.drawline then begin
      editor.Cross.AllowDrawEditline;
      options.menopt.drawline:=true;
    end;
    editor.Cross.SetCrossMode;
  end;

  SetupMenus;
  ClearWindow;

end;

procedure TFracWindow.CMInvisible(VAR msg : TMessage);
begin

  options.menopt.isinvisible:=not(options.menopt.isinvisible);

  SetupMenus;
  ClearWindow;

end;

procedure TFracWindow.CMBackground(VAR msg : TMessage);

var Dialog : TBackgroundDialog;
    i : integer;

begin

  Dialog.Init(@Self,'BACKDLG');
  if Dialog.Execute=id_ok then begin end;
  Dialog.Done;
  PostMessage(hWindow,wm_Size,0,0);
end;


procedure TFracWindow.CMNext(VAR msg : TMessage);

var step : real;
    i : INTEGER;
    pp : PPatternItem;
    str : ARRAY[0..40] OF char;

begin


  if Pattern.GetNumberOfPoints>1 then begin

    if (frac.depth)=0 then begin
      Pattern.TransformForTurtle;
      editor.savededstate:=editor.edstate;
      options.savedmenopt:=options.menopt;
      editor.edstate.mode:=FractalDraw;
      SetupMenus;
    end;

    inc(frac.depth);

    frac.TurtleCalc;
    frac.Draw;

  end;

end;

procedure TFracWindow.CMReset(var msg : TMessage);
begin

  frac.depth:=0;

  if screen.usebitmap then
    DeleteObjects;

  editor.edstate:=editor.savededstate;
  options.menopt:=options.savedmenopt;
  SetupMenus;
  ClearWindow;

end;

procedure TFracWindow.CMAllignToGrid(var msg: TMessage);
begin
   if not(Pattern.IsEmpty) and (MessageBox(HWindow,
      'Are you sure you want to allign your pattern? Pattern will be distorted!',
      'Allign Pattern selected!',
      mb_IconInformation or mb_okcancel)=id_Ok) then begin

    Pattern.AllignToGrid;
    ClearWindow;
 end;
end;

(* TOptionDialog-Methoden *)

procedure TOptionDialog.SetUpWindow;
var i : integer;

begin

  TDialog.SetUpWindow;

  with PFracWindow(Parent)^.options do begin
    SetDlgItemInt(hWindow,59,alpha,false);
    SetDlgItemInt(hWindow,62,beta,false);

    for i:=1 to 8 do
      if sides=i then
        SendDlgItemMsg(49+i,bm_SetCheck,1,0)
      else
        SendDlgItemMsg(49+i,bm_SetCheck,0,0);

    if usealpha then
      SendDlgItemMsg(60,bm_SetCheck,1,0)
    else
      SendDlgItemMsg(60,bm_SetCheck,0,0);

    if usebeta then
      SendDlgItemMsg(61,bm_SetCheck,1,0)
    else
      SendDlgItemMsg(61,bm_SetCheck,0,0);

    if treemode then
      SendDlgItemMsg(58,bm_SetCheck,1,0)
    else
      SendDlgItemMsg(58,bm_SetCheck,0,0);
  end;
end;


procedure TOptionDialog.Ok(var message : tmessage);
var i : integer;
    translate : LongBool;
begin

  with PFracWindow(Parent)^.options do begin
    for i:=1 to 8 do
      if IsDlgButtonChecked(hWindow,49+i)<>0 then
        sides:=i;

    usealpha:=IsDlgButtonChecked(hWindow,60)<>0;
    usebeta:=IsDlgButtonChecked(hWindow,61)<>0;
    treemode:=IsDlgButtonChecked(hWindow,58)<>0;
    alpha:=GetDlgItemInt(hWindow,59,@translate,false);
    beta:=GetDlgItemInt(hWindow,62,@translate,false);
    EndDlg(id_ok);
  end;
end;

(* TColorDialog-Methoden *)

procedure TPenWidthDialog.SetUpWindow;
var i : integer;
begin

  TDialog.SetUpWindow;

  with PFracWindow(Parent)^.options do
    SetDlgItemInt(hWindow,1099,width,false);

end;

procedure TPenWidthDialog.Ok(var message : tmessage);
var i : integer;
    translate : LongBool;

begin

 with PFracWindow(Parent)^.options do
   width:=GetDlgItemInt(hWindow,1099,@translate,false);

  EndDlg(id_ok);
end;


(* TBackgroundDialog-Methoden *)

procedure TBackgroundDialog.SetUpWindow;
begin
  with PFracWindow(Parent)^.options do begin

    case brush of
      white_brush : begin SendDlgItemMsg(102,bm_SetCheck,0,0);
                          SendDlgItemMsg(103,bm_SetCheck,1,0);
                          SendDlgItemMsg(104,bm_setCheck,0,0); end;
      ltgray_brush : begin SendDlgItemMsg(102,bm_SetCheck,0,0);
                           SendDlgItemMsg(103,bm_SetCheck,0,0);
                           SendDlgItemMsg(104,bm_setCheck,1,0); end;
      black_brush : begin SendDlgItemMsg(102,bm_SetCheck,1,0);
                          SendDlgItemMsg(103,bm_SetCheck,0,0);
                          SendDlgItemMsg(104,bm_setCheck,0,0); end;
    end;
  end;
end;

procedure TBackgroundDialog.Ok(var message : tmessage);
begin

  with PFracWindow(Parent)^.options do begin
    if IsDlgButtonChecked(hWindow,102)<>0 then brush:=black_brush;
    if IsDlgButtonChecked(hWindow,103)<>0 then brush:=white_brush;
    if IsDlgButtonChecked(hWindow,104)<>0 then brush:=ltgray_brush;
    EndDlg(id_ok);
  end;

end;

(* TScrollDialog-Methoden *)

{ =======================================================================
  PROCEDURE : SetUpWindow

  AUFGABE   : Initialisiere alle Dialogelemente
  ======================================================================= }

procedure TScrollDialog.SetUpWindow;
var
  FracWin: PFracWindow;
begin
  TDialog.SetUpWindow;
  FracWin := PFracWindow(Parent);
  SetDlgItemInt (HWindow,id_ZoomFactor, round(FracWin^.options.zoomfactor*100.0), false);
  SetScrollRange(GetItemHandle(id_Scroller), sb_Ctl, zoomfacmin, zoomfacmax, false);
  SetScrollPos(GetItemHandle(id_Scroller), sb_Ctl, round(FracWin^.options.zoomfactor*100.0), true);

  savedfactor:=FracWin^.options.zoomfactor;
  FracWin^.editor.cross.GetEditlinestart(savedmousex,savedmousey);

  FracWin^.pattern.movebox.savedoffset.x:=FracWin^.pattern.offset.x;
  FracWin^.pattern.movebox.savedoffset.y:=FracWin^.pattern.offset.y;
end;

{ =======================================================================
  PROCEDURE : Scroller

  AUFGABE   : Wird bei Scroller-Aktionen aufgerufen.

  PARAMETER : -----------------------------------------------------------
  ======================================================================= }

procedure TScrollDialog.Scroller(var Message: TMessage);
var
  FracWin: PFracWindow;
  pos: integer;
begin
  FracWin := PFracWindow(Parent);

  pos:=round(FracWin^.options.zoomfactor*100.0);

  case Message.wParam of
    sb_LineUp:
      dec(pos);
    sb_LineDown:
      inc(pos);
    sb_PageUp:
      pos:=pos-(zoomfacmax-zoomfacmin) div 10;
    sb_PageDown:
      pos:=pos+(zoomfacmax-zoomfacmin) div 10;
    sb_ThumbPosition, sb_ThumbTrack:
      pos:=Message.lParamLo
  end;

  if pos>zoomfacmax then pos:=zoomfacmax;
  if pos<zoomfacmin then pos:=zoomfacmin;

  FracWin^.options.zoomfactor:=pos/100.0;
  SetScrollPos(GetItemHandle(id_Scroller), sb_Ctl, pos, true);
  SetDlgItemInt (HWindow, id_ZoomFactor, pos, false);

  FracWin^.editor.cross.SetEditlinestart(round(savedmousex/savedfactor*pos/100.0),
                                         round(savedmousey/savedfactor*pos/100.0));

  FracWin^.ClearWindow;

end;

procedure TScrollDialog.Cancel(var Message : TMessage);
var FracWin : PFracWindow;

begin
  FracWin := PFracWindow(Parent);
  FracWin^.options.zoomfactor:=savedfactor;
  EndDlg(id_cancel);
end;

(* TApplication *)

procedure TMyApplication.InitMainWindow;
begin
  MainWindow := New(PFracWindow, Init(nil,AppName));
  if Status=0 then
    HAccTable := LoadAccelerators(HInstance, 'ACCELERATORS');
end;

(* Hautpprogramm *)

var
  MyApp: TMyApplication;

begin
  MyApp.Init(AppName);
  MyApp.Run;
  MyApp.Done;
end.